@@ -28,7 +28,7 @@ void bind_sha1_hash()
         .def(self != self)
         .def(self < self)
         .def(self_ns::str(self))
-        .def(init<char const*>())
+        .def(init<std::string>())
         .def("clear", &sha1_hash::clear)
         .def("is_all_zeros", &sha1_hash::is_all_zeros)
         .def("to_string", &sha1_hash::to_string)
