@@ -62,7 +62,7 @@ namespace
     char const* filestorage_name(file_storage const& fs)
     { return fs.name().c_str(); }
 
-    bool call_python_object2(boost::python::object const& obj, std::string& i)
+    bool call_python_object2(boost::python::object const& obj, std::string const& i)
     {
        return obj(i);
     }
