@@ -63,6 +63,7 @@ void bind_torrent_status()
         .def_readonly("num_incomplete", &torrent_status::num_incomplete)
         .def_readonly("list_seeds", &torrent_status::list_seeds)
         .def_readonly("list_peers", &torrent_status::list_peers)
+        .def_readonly("connect_candidates", &torrent_status::connect_candidates)
         .add_property("pieces", &pieces)
         .add_property("verified_pieces", &verified_pieces)
         .def_readonly("num_pieces", &torrent_status::num_pieces)
@@ -92,6 +93,7 @@ void bind_torrent_status()
         .def_readonly("seed_mode", &torrent_status::seed_mode)
         .def_readonly("upload_mode", &torrent_status::upload_mode)
         .def_readonly("share_mode", &torrent_status::share_mode)
+        .def_readonly("super_seeding", &torrent_status::super_seeding)
         .def_readonly("error", &torrent_status::error)
         .def_readonly("priority", &torrent_status::priority)
         .def_readonly("added_time", &torrent_status::added_time)
@@ -101,6 +103,7 @@ void bind_torrent_status()
         .def_readonly("time_since_download", &torrent_status::time_since_download)
         .def_readonly("queue_position", &torrent_status::queue_position)
         .def_readonly("need_save_resume", &torrent_status::need_save_resume)
+        .def_readonly("ip_filter_applies", &torrent_status::ip_filter_applies)
         ;
 
     enum_<torrent_status::state_t>("states")
