@@ -30,11 +30,7 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
-#include <iostream>
-#include <fstream>
-#include <iterator>
-#include <exception>
-
+#include <stdlib.h>
 #include "libtorrent/entry.hpp"
 #include "libtorrent/bencode.hpp"
 #include "libtorrent/session.hpp"
@@ -42,40 +38,40 @@ POSSIBILITY OF SUCH DAMAGE.
 int main(int argc, char* argv[])
 {
 	using namespace libtorrent;
-#if BOOST_VERSION < 103400
-	namespace fs = boost::filesystem;
-	fs::path::default_name_check(fs::no_check);
-#endif
 
 	if (argc != 2)
 	{
-		std::cerr << "usage: ./simple_client torrent-file\n"
-			"to stop the client, press return.\n";
+		fputs("usage: ./simple_client torrent-file\n"
+			"to stop the client, press return.\n", stderr);
 		return 1;
 	}
 
 	session s;
-	s.listen_on(std::make_pair(6881, 6889));
+	error_code ec;
+	s.listen_on(std::make_pair(6881, 6889), ec);
+	if (ec)
+	{
+		fprintf(stderr, "failed to open listen socket: %s\n", ec.message().c_str());
+		return 1;
+	}
 	add_torrent_params p;
 	p.save_path = "./";
-	error_code ec;
 	p.ti = new torrent_info(argv[1], ec);
 	if (ec)
 	{
-		std::cout << ec.message() << std::endl;
+		fprintf(stderr, "%s\n", ec.message().c_str());
 		return 1;
 	}
 	s.add_torrent(p, ec);
 	if (ec)
 	{
-		std::cerr << ec.message() << std::endl;
+		fprintf(stderr, "%s\n", ec.message().c_str());
 		return 1;
 	}
 
 	// wait for the user to end
 	char a;
-	std::cin.unsetf(std::ios_base::skipws);
-	std::cin >> a;
+	scanf("%c\n", &a);
 	return 0;
 }
 
