@@ -34,6 +34,15 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/session.hpp"
 #include "libtorrent/alert_types.hpp"
 
+char const* timestamp()
+{
+	time_t t = std::time(0);
+	tm* timeinfo = std::localtime(&t);
+	static char str[200];
+	std::strftime(str, 200, "%b %d %X", timeinfo);
+	return str;
+}
+
 void print_alert(libtorrent::alert const* a)
 {
 	using namespace libtorrent;
@@ -47,7 +56,7 @@ void print_alert(libtorrent::alert const* a)
 		printf("%s","\x1b[33m");
 	}
 
-	printf("[%s] %s\n", time_now_string(), a->message().c_str());
+	printf("[%s] %s\n", timestamp(), a->message().c_str());
 	printf("%s", "\x1b[0m");
 }
 
