@@ -97,6 +97,7 @@ typedef boost::uint16_t UTF16;
 typedef boost::uint8_t  UTF8;
 extern "C" {
 #else
+#define TORRENT_EXTRA_EXPORT
 #ifdef _MSC_VER
 // msvc doesn't seem to have stdint.h
 typedef unsigned __int32 UTF32;
@@ -131,31 +132,32 @@ typedef enum {
 	lenientConversion
 } ConversionFlags;
 
-ConversionResult ConvertUTF8toUTF16 (
+TORRENT_EXTRA_EXPORT ConversionResult ConvertUTF8toUTF16 (
 		const UTF8** sourceStart, const UTF8* sourceEnd, 
 		UTF16** targetStart, UTF16* targetEnd, ConversionFlags flags);
 
-ConversionResult ConvertUTF16toUTF8 (
+TORRENT_EXTRA_EXPORT ConversionResult ConvertUTF16toUTF8 (
 		const UTF16** sourceStart, const UTF16* sourceEnd, 
 		UTF8** targetStart, UTF8* targetEnd, ConversionFlags flags);
 		
-ConversionResult ConvertUTF8toUTF32 (
+TORRENT_EXTRA_EXPORT ConversionResult ConvertUTF8toUTF32 (
 		const UTF8** sourceStart, const UTF8* sourceEnd, 
 		UTF32** targetStart, UTF32* targetEnd, ConversionFlags flags);
 
-ConversionResult ConvertUTF32toUTF8 (
+TORRENT_EXTRA_EXPORT ConversionResult ConvertUTF32toUTF8 (
 		const UTF32** sourceStart, const UTF32* sourceEnd, 
 		UTF8** targetStart, UTF8* targetEnd, ConversionFlags flags);
 		
-ConversionResult ConvertUTF16toUTF32 (
+TORRENT_EXTRA_EXPORT ConversionResult ConvertUTF16toUTF32 (
 		const UTF16** sourceStart, const UTF16* sourceEnd, 
 		UTF32** targetStart, UTF32* targetEnd, ConversionFlags flags);
 
-ConversionResult ConvertUTF32toUTF16 (
+TORRENT_EXTRA_EXPORT ConversionResult ConvertUTF32toUTF16 (
 		const UTF32** sourceStart, const UTF32* sourceEnd, 
 		UTF16** targetStart, UTF16* targetEnd, ConversionFlags flags);
 
-Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd);
+TORRENT_EXTRA_EXPORT Boolean isLegalUTF8Sequence(const UTF8 *source,
+		const UTF8 *sourceEnd);
 
 #ifdef __cplusplus
 }
