@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2007, Arvid Norberg
+Copyright (c) 2007-2014, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -34,21 +34,19 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/config.hpp"
 
-#if (!defined TORRENT_DEBUG && !TORRENT_RELEASE_ASSERTS) \
-	|| TORRENT_NO_ASSERTS
-#define TORRENT_ASSERT(a) do {} while(false)
-#define TORRENT_ASSERT_VAL(a, b) do {} while(false)
-#else
+#if defined TORRENT_DEBUG || defined TORRENT_ASIO_DEBUGGING \
+	|| TORRENT_RELEASE_ASSERTS || defined TORRENT_DEBUG_BUFFERS
+#include <string>
+std::string demangle(char const* name);
+TORRENT_EXPORT void print_backtrace(char* out, int len, int max_depth = 0);
+#endif
+
+#if TORRENT_USE_ASSERTS
 
 #if TORRENT_PRODUCTION_ASSERTS
 extern char const* libtorrent_assert_log;
 #endif
 
-#include <string>
-
-std::string demangle(char const* name);
-void print_backtrace(char* out, int len, int max_depth = 0);
-
 #if (defined __linux__ || defined __MACH__) && defined __GNUC__ && !TORRENT_USE_SYSTEM_ASSERT
 
 #if TORRENT_USE_IOSTREAM
@@ -56,22 +54,36 @@ void print_backtrace(char* out, int len, int max_depth = 0);
 #endif
 
 TORRENT_EXPORT void assert_fail(const char* expr, int line, char const* file
-	, char const* function, char const* val);
+	, char const* function, char const* val, int kind = 0);
+
+#define TORRENT_ASSERT_PRECOND(x) \
+	do { if (x) {} else assert_fail(#x, __LINE__, __FILE__, __PRETTY_FUNCTION__, "", 1); } while (false)
+
+#define TORRENT_ASSERT(x) \
+	do { if (x) {} else assert_fail(#x, __LINE__, __FILE__, __PRETTY_FUNCTION__, "", 0); } while (false)
 
-#define TORRENT_ASSERT(x) do { if (x) {} else assert_fail(#x, __LINE__, __FILE__, __PRETTY_FUNCTION__, 0); } while (false)
 #if TORRENT_USE_IOSTREAM
-#define TORRENT_ASSERT_VAL(x, y) do { if (x) {} else { std::stringstream __s__; __s__ << #y ": " << y; assert_fail(#x, __LINE__, __FILE__, __PRETTY_FUNCTION__, __s__.str().c_str()); } } while (false)
+#define TORRENT_ASSERT_VAL(x, y) \
+	do { if (x) {} else { std::stringstream __s__; __s__ << #y ": " << y; \
+	assert_fail(#x, __LINE__, __FILE__, __PRETTY_FUNCTION__, __s__.str().c_str(), 0); } } while (false)
 #else
 #define TORRENT_ASSERT_VAL(x, y) TORRENT_ASSERT(x)
 #endif
 
 #else
 #include <cassert>
+#define TORRENT_ASSERT_PRECOND(x) assert(x)
 #define TORRENT_ASSERT(x) assert(x)
 #define TORRENT_ASSERT_VAL(x, y) assert(x)
 #endif
 
-#endif
+#else // TORRENT_USE_ASSERTS
+
+#define TORRENT_ASSERT_PRECOND(a) do {} while(false)
+#define TORRENT_ASSERT(a) do {} while(false)
+#define TORRENT_ASSERT_VAL(a, b) do {} while(false)
+
+#endif // TORRENT_USE_ASSERTS
 
 #endif
 
