@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2007, Arvid Norberg
+Copyright (c) 2007-2014, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -62,7 +62,7 @@ public:
 	void enqueue(boost::function<void(int)> const& on_connect
 		, boost::function<void()> const& on_timeout
 		, time_duration timeout, int priority = 0);
-	void done(int ticket);
+	bool done(int ticket);
 	void limit(int limit);
 	int limit() const;
 	void close();
@@ -84,7 +84,7 @@ public:
 	}
 #endif
 
-#ifdef TORRENT_DEBUG
+#if TORRENT_USE_INVARIANT_CHECKS
 	void check_invariant() const;
 #endif
 
@@ -98,12 +98,14 @@ private:
 
 	struct entry
 	{
-		entry(): connecting(false), ticket(0), expires(max_time()), priority(0) {}
+		entry()
+			: expires(max_time())
+			, ticket(0)
+			, connecting(false)
+			, priority(0)
+		{}
 		// called when the connection is initiated
 		// this is when the timeout countdown starts
-		// TODO: if we don't actually need the connection queue
-		// to hold ownership of objects, replace these boost functions
-		// with pointer to a pure virtual interface class
 		boost::function<void(int)> on_connect;
 		// called if done hasn't been called within the timeout
 		// or if the connection queue aborts. This means there
@@ -112,16 +114,13 @@ private:
 		// 2. on_connect, on_timeout
 		// 3. on_timeout
 		boost::function<void()> on_timeout;
-		bool connecting;
-		int ticket;
 		ptime expires;
 		time_duration timeout;
-		int priority;
+		boost::int32_t ticket;
+		bool connecting;
+		boost::uint8_t priority;
 	};
 
-	// TODO: split this into a queue and connecting map. The key for the map
-	// is the ticket. Most field in entry would only be necessary for the
-	// connecting map.
 	std::list<entry> m_queue;
 
 	// the next ticket id a connection will be given
