@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003, Arvid Norberg
+Copyright (c) 2003-2014, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -78,13 +78,18 @@ namespace libtorrent
 {
 	struct lazy_entry;
 
+	// thrown by any accessor function of entry if the accessor
+	// function requires a type different than the actual type
+	// of the entry object.
 	struct TORRENT_EXPORT type_error: std::runtime_error
 	{
+		// internal
 		type_error(const char* error): std::runtime_error(error) {}
 	};
 
-	class entry;
-
+	// The ``entry`` class represents one node in a bencoded hierarchy. It works as a
+	// variant type, it can be either a list, a dictionary (``std::map``), an integer
+	// or a string.
 	class TORRENT_EXPORT entry
 	{
 	public:
@@ -97,6 +102,7 @@ namespace libtorrent
 		typedef std::list<entry> list_type;
 		typedef size_type integer_type;
 
+		// the types an entry can have
 		enum data_type
 		{
 			int_t,
@@ -106,20 +112,36 @@ namespace libtorrent
 			undefined_t
 		};
 
+		// returns the concrete type of the entry
 		data_type type() const;
 
+		// constructors directly from a specific type.
+		// The content of the argument is copied into the
+		// newly constructed entry
 		entry(dictionary_type const&);
 		entry(string_type const&);
 		entry(list_type const&);
 		entry(integer_type const&);
 
-		entry();
+		// construct an empty entry of the specified type.
+		// see data_type enum.
 		entry(data_type t);
+
+		// hidden
 		entry(entry const& e);
+
+		// hidden
+		entry();
+
+		// hidden
 		~entry();
 
+		// hidden
 		bool operator==(entry const& e) const;
+		bool operator!=(entry const& e) const { return !(*this == e); }
 		
+		// copies the structure of the right hand side into this
+		// entry.
 		void operator=(lazy_entry const&);
 		void operator=(entry const&);
 		void operator=(dictionary_type const&);
@@ -127,6 +149,53 @@ namespace libtorrent
 		void operator=(list_type const&);
 		void operator=(integer_type const&);
 
+		// The ``integer()``, ``string()``, ``list()`` and ``dict()`` functions
+		// are accessors that return the respective type. If the ``entry`` object
+		// isn't of the type you request, the accessor will throw
+		// libtorrent_exception (which derives from ``std::runtime_error``). You
+		// can ask an ``entry`` for its type through the ``type()`` function.
+		// 
+		// If you want to create an ``entry`` you give it the type you want it to
+		// have in its constructor, and then use one of the non-const accessors
+		// to get a reference which you then can assign the value you want it to
+		// have.
+		// 
+		// The typical code to get info from a torrent file will then look like
+		// this:
+		//
+		// .. code:: c++
+		// 
+		// 	entry torrent_file;
+		// 	// ...
+		// 
+		// 	// throws if this is not a dictionary
+		// 	entry::dictionary_type const& dict = torrent_file.dict();
+		// 	entry::dictionary_type::const_iterator i;
+		// 	i = dict.find("announce");
+		// 	if (i != dict.end())
+		// 	{
+		// 		std::string tracker_url = i->second.string();
+		// 		std::cout << tracker_url << "\n";
+		// 	}
+		// 
+		// 
+		// The following code is equivalent, but a little bit shorter:
+		//
+		// .. code:: c++
+		// 
+		// 	entry torrent_file;
+		// 	// ...
+		// 
+		// 	// throws if this is not a dictionary
+		// 	if (entry* i = torrent_file.find_key("announce"))
+		// 	{
+		// 		std::string tracker_url = i->string();
+		// 		std::cout << tracker_url << "\n";
+		// 	}
+		// 
+		// 
+		// To make it easier to extract information from a torrent file, the
+		// class torrent_info exists.
 		integer_type& integer();
 		const integer_type& integer() const;
 		string_type& string();
@@ -136,24 +205,41 @@ namespace libtorrent
 		dictionary_type& dict();
 		const dictionary_type& dict() const;
 
+		// swaps the content of *this* with ``e``.
 		void swap(entry& e);
 
-		// these functions requires that the entry
-		// is a dictionary, otherwise they will throw	
-		entry& operator[](char const* key);
+		// All of these functions requires the entry to be a dictionary, if it
+		// isn't they will throw ``libtorrent::type_error``.
+		//
+		// The non-const versions of the ``operator[]`` will return a reference
+		// to either the existing element at the given key or, if there is no
+		// element with the given key, a reference to a newly inserted element at
+		// that key.
+		//
+		// The const version of ``operator[]`` will only return a reference to an
+		// existing element at the given key. If the key is not found, it will
+		// throw ``libtorrent::type_error``.
+ 		entry& operator[](char const* key);
 		entry& operator[](std::string const& key);
 #ifndef BOOST_NO_EXCEPTIONS
 		const entry& operator[](char const* key) const;
 		const entry& operator[](std::string const& key) const;
 #endif
+
+		// These functions requires the entry to be a dictionary, if it isn't
+		// they will throw ``libtorrent::type_error``.
+		//
+		// They will look for an element at the given key in the dictionary, if
+		// the element cannot be found, they will return 0. If an element with
+		// the given key is found, the return a pointer to it.
 		entry* find_key(char const* key);
 		entry const* find_key(char const* key) const;
 		entry* find_key(std::string const& key);
 		entry const* find_key(std::string const& key) const;
-		
-#if (defined TORRENT_VERBOSE_LOGGING || defined TORRENT_DEBUG) && TORRENT_USE_IOSTREAM
-		void print(std::ostream& os, int indent = 0) const;
-#endif
+
+		// returns a pretty-printed string representation
+		// of the bencoded structure, with JSON-style syntax
+		std::string to_string() const;
 
 	protected:
 
@@ -163,26 +249,7 @@ namespace libtorrent
 
 	private:
 
-#ifndef TORRENT_DEBUG
-		data_type m_type;
-#else
-		// the bitfield is used so that the m_type_queried
-		// field still fits, so that the ABI is the same for
-		// debug builds and release builds. It appears to be
-		// very hard to match debug builds with debug versions
-		// of libtorrent
-		data_type m_type:31;
-
-	public:
-		// in debug mode this is set to false by bdecode
-		// to indicate that the program has not yet queried
-		// the type of this entry, and sould not assume
-		// that it has a certain type. This is asserted in
-		// the accessor functions. This does not apply if
-		// exceptions are used.
-		mutable bool m_type_queried:1;
-	protected:
-#endif // TORRENT_DEBUG
+		void to_string_impl(std::string& out, int indent) const;
 
 #if (defined(_MSC_VER) && _MSC_VER < 1310) || TORRENT_COMPLETE_TYPES_REQUIRED
 		// workaround for msvc-bug.
@@ -204,17 +271,32 @@ namespace libtorrent
 #endif
 		integer_type data[(union_size + sizeof(integer_type) - 1)
 			/ sizeof(integer_type)];
+
+		// the bitfield is used so that the m_type_queried field still fits, so
+		// that the ABI is the same for debug builds and release builds. It
+		// appears to be very hard to match debug builds with debug versions of
+		// libtorrent
+		boost::uint8_t m_type:7;
+
+	public:
+		// in debug mode this is set to false by bdecode to indicate that the
+		// program has not yet queried the type of this entry, and sould not
+		// assume that it has a certain type. This is asserted in the accessor
+		// functions. This does not apply if exceptions are used.
+		mutable boost::uint8_t m_type_queried:1;
 	};
 
-#if defined TORRENT_DEBUG && TORRENT_USE_IOSTREAM
+#if TORRENT_USE_IOSTREAM
+	// prints the bencoded structure to the ostream as a JSON-style structure.
 	inline std::ostream& operator<<(std::ostream& os, const entry& e)
 	{
-		e.print(os, 0);
+		os << e.to_string();
 		return os;
 	}
 #endif
 
 #ifndef BOOST_NO_EXCEPTIONS
+	// internal
 	inline void throw_type_error()
 	{
 		throw libtorrent_exception(error_code(errors::invalid_entry_type
