@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2006, Arvid Norberg
+Copyright (c) 2006-2014, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -36,17 +36,26 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/kademlia/node_id.hpp"
 #include "libtorrent/socket.hpp"
 #include "libtorrent/address.hpp"
+#include "libtorrent/union_endpoint.hpp"
+#include "libtorrent/time.hpp" // for time_now()
+
+#ifdef TORRENT_DHT_VERBOSE_LOGGING
+#include "libtorrent/time.hpp"
+#endif
 
 namespace libtorrent { namespace dht
 {
 
 struct node_entry
 {
-	node_entry(node_id const& id_, udp::endpoint ep, bool pinged = false)
-		: addr(ep.address())
-		, port(ep.port())
-		, timeout_count(pinged ? 0 : 0xffff)
+	node_entry(node_id const& id_, udp::endpoint ep, int roundtriptime = 0xffff
+		, bool pinged = false)
+		: last_queried(pinged ? time_now() : min_time())
 		, id(id_)
+		, a(ep.address().to_v4().to_bytes())
+		, p(ep.port())
+		, rtt(roundtriptime & 0xffff)
+		, timeout_count(pinged ? 0 : 0xff)
 	{
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
 		first_seen = time_now();
@@ -54,10 +63,12 @@ struct node_entry
 	}
 
 	node_entry(udp::endpoint ep)
-		: addr(ep.address())
-		, port(ep.port())
-		, timeout_count(0xffff)
+		: last_queried(min_time())
 		, id(0)
+		, a(ep.address().to_v4().to_bytes())
+		, p(ep.port())
+		, rtt(0xffff)
+		, timeout_count(0xff)
 	{
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
 		first_seen = time_now();
@@ -65,32 +76,53 @@ struct node_entry
 	}
 
 	node_entry()
-		: timeout_count(0xffff)
+		: last_queried(min_time())
 		, id(0)
+		, p(0)
+		, rtt(0xffff)
+		, timeout_count(0xff)
 	{
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
 		first_seen = time_now();
 #endif
 	}
 	
-	bool pinged() const { return timeout_count != 0xffff; }
-	void set_pinged() { if (timeout_count == 0xffff) timeout_count = 0; }
-	void timed_out() { if (pinged()) ++timeout_count; }
+	bool pinged() const { return timeout_count != 0xff; }
+	void set_pinged() { if (timeout_count == 0xff) timeout_count = 0; }
+	void timed_out() { if (pinged() && timeout_count < 0xfe) ++timeout_count; }
 	int fail_count() const { return pinged() ? timeout_count : 0; }
 	void reset_fail_count() { if (pinged()) timeout_count = 0; }
-	udp::endpoint ep() const { return udp::endpoint(addr, port); }
+	udp::endpoint ep() const { return udp::endpoint(address_v4(a), p); }
 	bool confirmed() const { return timeout_count == 0; }
+	void update_rtt(int new_rtt)
+	{
+		TORRENT_ASSERT(new_rtt <= 0xffff);
+		TORRENT_ASSERT(new_rtt >= 0);
+		if (new_rtt == 0xffff) return;
+		if (rtt == 0xffff) rtt = new_rtt;
+		else rtt = int(rtt) * 2 / 3 + int(new_rtt) / 3;
+	}
+	address addr() const { return address_v4(a); }
+	int port() const { return p; }
 
-	// TODO: replace with a union of address_v4 and address_v6
-	address addr;
-	boost::uint16_t port;
-	// the number of times this node has failed to
-	// respond in a row
-	boost::uint16_t timeout_count;
-	node_id id;
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
 	ptime first_seen;
 #endif
+
+	// the time we last received a response for a request to this peer
+	ptime last_queried;
+
+	node_id id;
+
+	address_v4::bytes_type a;
+	boost::uint16_t p;
+
+	// the average RTT of this node
+	boost::uint16_t rtt;
+
+	// the number of times this node has failed to
+	// respond in a row
+	boost::uint8_t timeout_count;
 };
 
 } } // namespace libtorrent::dht
