@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2006, Arvid Norberg & Daniel Wallin
+Copyright (c) 2006-2014, Arvid Norberg & Daniel Wallin
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -67,22 +67,30 @@ struct traversal_algorithm : boost::noncopyable
 	void failed(observer_ptr o, int flags = 0);
 	virtual ~traversal_algorithm();
 	void status(dht_lookup& l);
+	void abort();
 
 	void* allocate_observer();
 	void free_observer(void* ptr);
 
-	virtual char const* name() const { return "traversal_algorithm"; }
+	virtual char const* name() const;
 	virtual void start();
 
 	node_id const& target() const { return m_target; }
 
+	void resort_results();
 	void add_entry(node_id const& id, udp::endpoint addr, unsigned char flags);
 
 	traversal_algorithm(node_impl& node, node_id target);
+	int invoke_count() const { return m_invoke_count; }
+	int branch_factor() const { return m_branch_factor; }
+
+	node_impl& node() const { return m_node; }
 
 protected:
 
-	void add_requests();
+	// returns true if we're done
+	bool add_requests();
+
 	void add_router_entries();
 	void init();
 
@@ -108,7 +116,7 @@ protected:
 	int m_ref_count;
 
 	node_impl& m_node;
-	node_id m_target;
+	node_id const m_target;
 	std::vector<observer_ptr> m_results;
 	int m_invoke_count;
 	int m_branch_factor;
@@ -117,6 +125,18 @@ protected:
 	int m_num_target_nodes;
 };
 
+struct traversal_observer : observer
+{
+	traversal_observer(
+		boost::intrusive_ptr<traversal_algorithm> const& algorithm
+		, udp::endpoint const& ep, node_id const& id)
+		: observer(algorithm, ep, id)
+	{}
+
+	// parses out "nodes" and keeps traversing
+	virtual void reply(msg const&);
+};
+
 } } // namespace libtorrent::dht
 
 #endif // TRAVERSAL_ALGORITHM_050324_HPP
