@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003, Arvid Norberg
+Copyright (c) 2003-2014, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -79,6 +79,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/socket_type_fwd.hpp"
 #include "libtorrent/error_code.hpp"
 #include "libtorrent/sliding_average.hpp"
+#include "libtorrent/io_service_fwd.hpp"
 
 #ifdef TORRENT_STATS
 #include "libtorrent/aux_/session_impl.hpp"
@@ -174,14 +175,6 @@ namespace libtorrent
 			, policy::peer* peerinfo
 			, bool outgoing = true);
 
-		// with this constructor we have been contacted and we still don't
-		// know which torrent the connection belongs to
-		peer_connection(
-			aux::session_impl& ses
-			, boost::shared_ptr<socket_type> s
-			, tcp::endpoint const& remote
-			, policy::peer* peerinfo);
-
 		// this function is called after it has been constructed and properly
 		// reference counted. It is safe to call self() in this function
 		// and schedule events with references to itself (that is not safe to
@@ -196,6 +189,19 @@ namespace libtorrent
 			m_peer_info = pi;
 		}
 
+		// this is called when the peer object is created, in case
+		// it was let in by the connections limit slack. This means
+		// the peer needs to, as soon as the handshake is done, either
+		// disconnect itself or another peer.
+		void peer_exceeds_limit()
+		{ m_exceeded_limit = true; }
+
+		// this is called if this peer causes another peer
+		// to be disconnected, in which case it has fulfilled
+		// its requirement.
+		void peer_disconnected_other()
+		{ m_exceeded_limit = false; }
+
 		policy::peer* peer_info_struct() const
 		{ return m_peer_info; }
 
@@ -266,6 +272,8 @@ namespace libtorrent
 			m_priority = p;
 		}
 
+		boost::uint32_t peer_rank() const;
+
 		void fast_reconnect(bool r);
 		bool fast_reconnect() const { return m_fast_reconnect; }
 
@@ -275,13 +283,17 @@ namespace libtorrent
 		
 		// this will tell the peer to announce the given piece
 		// and only allow it to request that piece
-		void superseed_piece(int index);
-		int superseed_piece() const { return m_superseed_piece; }
+		void superseed_piece(int replace_piece, int new_piece);
+		bool super_seeded_piece(int index) const
+		{
+			return m_superseed_piece[0] == index
+				|| m_superseed_piece[1] == index;
+		}
 
 		// tells if this connection has data it want to send
 		// and has enough upload bandwidth quota left to send it.
 		bool can_write() const;
-		bool can_read(char* state = 0) const;
+		bool can_read(boost::uint8_t* state = 0) const;
 
 		bool is_seed() const;
 		int num_have_pieces() const { return m_num_pieces; }
@@ -387,15 +399,11 @@ namespace libtorrent
 		// It will reset the used bandwidth to 0.
 		void reset_upload_quota();
 
-		// free upload.
-		size_type total_free_upload() const;
-		void add_free_upload(size_type free_upload);
-
 		// trust management.
-		void received_valid_data(int index);
-		void received_invalid_data(int index);
-
-		size_type share_diff() const;
+		virtual void received_valid_data(int index);
+		// returns false if the peer should not be
+		// disconnected
+		virtual bool received_invalid_data(int index, bool single_peer);
 
 		// a connection is local if it was initiated by us.
 		// if it was an incoming connection, it is remote
@@ -421,7 +429,7 @@ namespace libtorrent
 		{
 			// this peer is in end-game mode we only want
 			// one outstanding request
-			return (m_endgame_mode || m_snubbed) ? 1: m_desired_queue_size;
+			return (m_endgame_mode || m_snubbed) ? 1 : m_desired_queue_size;
 		}
 
 		bool bittyrant_unchoke_compare(
@@ -472,6 +480,7 @@ namespace libtorrent
 		void start_receive_piece(peer_request const& r);
 		void incoming_cancel(peer_request const& r);
 
+		bool can_disconnect(error_code const& ec) const;
 		void incoming_dht_port(int listen_port);
 		
 		void incoming_reject_request(peer_request const& r);
@@ -495,7 +504,9 @@ namespace libtorrent
 
 		bool can_request_time_critical() const;
 
-		void make_time_critical(piece_block const& block);
+		// returns true if the specified block was actually made time-critical.
+		// if the block was already time-critical, it returns false.
+		bool make_time_critical(piece_block const& block);
 
 		// adds a block to the request queue
 		// returns true if successful, false otherwise
@@ -520,12 +531,10 @@ namespace libtorrent
 
 		void assign_bandwidth(int channel, int amount);
 
-#ifdef TORRENT_DEBUG
+#if TORRENT_USE_INVARIANT_CHECKS
 		void check_invariant() const;
-		ptime m_last_choke;
 #endif
 
-
 		// is true until we can be sure that the other end
 		// speaks our protocol (be it bittorrent or http).
 		virtual bool in_handshake() const = 0;
@@ -600,6 +609,9 @@ namespace libtorrent
 
 		int receive_pos() const { return m_recv_pos; }
 
+		void max_out_request_queue(int s);
+		int max_out_request_queue() const;
+
 #ifdef TORRENT_DEBUG
 		bool piece_failed;
 #endif
@@ -607,10 +619,6 @@ namespace libtorrent
 		time_t last_seen_complete() const { return m_last_seen_complete; }
 		void set_last_seen_complete(int ago) { m_last_seen_complete = time(0) - ago; }
 
-		// upload and download channel state
-		// enum from peer_info::bw_state
-		char m_channel_state[2];
-
 		size_type uploaded_in_last_round() const
 		{ return m_statistics.total_payload_upload() - m_uploaded_at_last_round; }
 
@@ -704,9 +712,103 @@ namespace libtorrent
 
 		void update_desired_queue_size();
 
-		// the bandwidth channels, upload and download
-		// keeps track of the current quotas
-		bandwidth_channel m_bandwidth_channel[num_channels];
+		void set_timeout(int s) { m_timeout = s; }
+
+		boost::intrusive_ptr<peer_connection> self()
+		{
+			TORRENT_ASSERT(!m_in_constructor);
+			return boost::intrusive_ptr<peer_connection>(this);
+		}
+
+		// TODO: make this private
+	public:
+
+		// upload and download channel state
+		// enum from peer_info::bw_state
+		boost::uint8_t m_channel_state[2];
+
+	private:
+
+		// is true if we learn the incoming connections listening
+		// during the extended handshake
+		bool m_received_listen_port:1;
+
+		// this is set to true when a have_all
+		// message is received. This information
+		// is used to fill the bitmask in init()
+		bool m_have_all:1;
+
+		// other side says that it's interested in downloading
+		// from us.
+		bool m_peer_interested:1;
+
+		// the other side has told us that it won't send anymore
+		// data to us for a while
+		bool m_peer_choked:1;
+
+		// the peer has pieces we are interested in
+		bool m_interesting:1;
+
+		// we have choked the upload to the peer
+		bool m_choked:1;
+
+		// this is set to true if the connection timed
+		// out or closed the connection. In that
+		// case we will not try to reconnect to
+		// this peer
+		bool m_failed:1;
+
+		// this is true if this connection has been added
+		// to the list of connections that will be closed.
+		bool m_disconnecting:1;
+
+		// this is set to true once the bitfield is received
+		bool m_bitfield_received:1;
+
+		// this is set to true if the last time we tried to
+		// pick a piece to download, we could only find
+		// blocks that were already requested from other
+		// peers. In this case, we should not try to pick
+		// another piece until the last one we requested is done
+		bool m_endgame_mode:1;
+
+		// set to true when we've sent the first round of suggests
+		bool m_sent_suggests:1;
+
+		// set to true while we're trying to holepunch
+		bool m_holepunch_mode:1;
+
+		// when this is set, the transfer stats for this connection
+		// is not included in the torrent or session stats
+		bool m_ignore_stats:1;
+
+		// when this is set, the peer_connection socket is
+		// corked, similar to the linux TCP feature TCP_CORK.
+		// we won't send anything to the actual socket, just
+		// buffer messages up in the application layer send
+		// buffer, and send it once we're uncorked.
+		bool m_corked:1;
+
+		// set to true if this peer has metadata, and false
+		// otherwise.
+		bool m_has_metadata:1;
+
+		// this is set to true if this peer was accepted exceeding
+		// the connection limit. It means it has to disconnect
+		// itself, or some other peer, as soon as it's completed
+		// the handshake. We need to wait for the handshake in
+		// order to know which torrent it belongs to, to know which
+		// other peers to compare it to.
+		bool m_exceeded_limit:1;
+
+		// this is slow-start at the bittorrent layer. It affects how we increase
+		// desired queue size (i.e. the number of outstanding requests we keep).
+		// While the underlying transport protocol is in slow-start, the number of
+		// outstanding requests need to increase at the same pace to keep up.
+		bool m_slow_start:1;
+
+		// TODO: make these private as well
+	protected:
 
 		// number of bytes this peer can send and receive
 		int m_quota[2];
@@ -720,6 +822,11 @@ namespace libtorrent
 		// the peer belongs to.
 		aux::session_impl& m_ses;
 
+#ifndef TORRENT_DISABLE_EXTENSIONS
+		typedef std::list<boost::shared_ptr<peer_plugin> > extension_list_t;
+		extension_list_t m_extensions;
+#endif
+
 		// called from the main loop when this connection has any
 		// work to do.
 		void on_send_data(error_code const& error
@@ -727,39 +834,10 @@ namespace libtorrent
 		void on_receive_data(error_code const& error
 			, std::size_t bytes_transferred);
 
-		// this is the limit on the number of outstanding requests
-		// we have to this peer. This is initialized to the settings
-		// in the session_settings structure. But it may be lowered
-		// if the peer is known to require a smaller limit (like BitComet).
-		// or if the extended handshake sets a limit.
-		// web seeds also has a limit on the queue size.
-		int m_max_out_request_queue;
-
 		// the average rate of receiving complete piece messages
 		sliding_average<20> m_piece_rate;
 		sliding_average<20> m_send_rate;
 
-		void set_timeout(int s) { m_timeout = s; }
-
-#ifndef TORRENT_DISABLE_EXTENSIONS
-		typedef std::list<boost::shared_ptr<peer_plugin> > extension_list_t;
-		extension_list_t m_extensions;
-#endif
-
-#ifndef TORRENT_DISABLE_RESOLVE_COUNTRIES	
-		// in case the session settings is set
-		// to resolve countries, this is set to
-		// the two character country code this
-		// peer resides in.
-		char m_country[2];
-#endif
-
-		boost::intrusive_ptr<peer_connection> self()
-		{
-			TORRENT_ASSERT(!m_in_constructor);
-			return boost::intrusive_ptr<peer_connection>(this);
-		}
-
 	private:
 
 		std::pair<int, int> preferred_caching() const;
@@ -799,6 +877,10 @@ namespace libtorrent
 		// was the time
 		ptime m_last_unchoked;
 
+		// the time we last choked this peer. min_time() in
+		// case we never unchoked it
+		ptime m_last_choke;
+
 		// timeouts
 		ptime m_last_receive;
 		ptime m_last_sent;
@@ -825,14 +907,6 @@ namespace libtorrent
 		// this peer the last time.
 		ptime m_became_uninteresting;
 
-		// the amount of data this peer has been given
-		// as free upload. This is distributed from
-		// peers from which we get free download
-		// this will be negative on a peer from which
-		// we get free download, and positive on peers
-		// that we give the free upload, to keep the balance.
-		size_type m_free_upload;
-
 		// the total payload download bytes
 		// at the last unchoke round. This is used to
 		// measure the number of bytes transferred during
@@ -849,6 +923,24 @@ namespace libtorrent
 		// for the round-robin unchoke algorithm.
 		size_type m_uploaded_at_last_unchoke;
 
+		template <std::size_t Size>
+		struct handler_storage
+		{
+#ifdef TORRENT_DEBUG
+			handler_storage()
+			  : used(false)
+			{}
+
+			bool used;
+#else
+			handler_storage() {}
+#endif
+			boost::aligned_storage<Size> bytes;
+		};
+
+		handler_storage<TORRENT_READ_HANDLER_MAX_SIZE> m_read_handler_storage;
+		handler_storage<TORRENT_WRITE_HANDLER_MAX_SIZE> m_write_handler_storage;
+
 #ifndef TORRENT_DISABLE_GEO_IP
 		std::string m_inet_as_name;
 #endif
@@ -864,11 +956,7 @@ namespace libtorrent
 		chained_buffer m_send_buffer;
 
 		boost::shared_ptr<socket_type> m_socket;
-		// this is the peer we're actually talking to
-		// it may not necessarily be the peer we're
-		// connected to, in case we use a proxy
-		tcp::endpoint m_remote;
-		
+
 		// this is the torrent this connection is
 		// associated with. If the connection is an
 		// incoming connection, this is set to zero
@@ -876,12 +964,9 @@ namespace libtorrent
 		// set to the torrent it belongs to.
 		boost::weak_ptr<torrent> m_torrent;
 
-		// remote peer's id
-		peer_id m_peer_id;
-
 		// the pieces the other end have
 		bitfield m_have_piece;
-
+		
 		// the queue of requests we have got
 		// from this peer that haven't been issued
 		// to the disk thread yet
@@ -916,19 +1001,42 @@ namespace libtorrent
 		// the piece requests
 		std::vector<int> m_requests_in_buffer;
 
-		// the block we're currently receiving. Or
-		// (-1, -1) if we're not receiving one
-		piece_block m_receiving_block;
+		// this peer's peer info struct. This may
+		// be 0, in case the connection is incoming
+		// and hasn't been added to a torrent yet.
+		policy::peer* m_peer_info;
 
 		// the time when this peer last saw a complete copy
 		// of this torrent
 		time_t m_last_seen_complete;
 
+		// the block we're currently receiving. Or
+		// (-1, -1) if we're not receiving one
+		piece_block m_receiving_block;
+
+		// this is the peer we're actually talking to
+		// it may not necessarily be the peer we're
+		// connected to, in case we use a proxy
+		tcp::endpoint m_remote;
+		
+		// the bandwidth channels, upload and download
+		// keeps track of the current quotas
+		bandwidth_channel m_bandwidth_channel[num_channels];
+
+		// remote peer's id
+		peer_id m_peer_id;
+
 		// if the timeout is extended for the outstanding
 		// requests, this is the number of seconds it was
 		// extended.
 		int m_timeout_extend;
 
+		// the number of payload bytes downloaded last second tick
+		boost::int32_t m_downloaded_last_second;
+
+		// the number of payload bytes uploaded last second tick
+		boost::int32_t m_uploaded_last_second;
+
 		// the number of bytes that the other
 		// end has to send us in order to respond
 		// to all outstanding piece requests we
@@ -995,11 +1103,6 @@ namespace libtorrent
 		int m_upload_limit;
 		int m_download_limit;
 
-		// this peer's peer info struct. This may
-		// be 0, in case the connection is incoming
-		// and hasn't been added to a torrent yet.
-		policy::peer* m_peer_info;
-
 		// this is a measurement of how fast the peer
 		// it allows some variance without changing
 		// back and forth between states
@@ -1011,12 +1114,12 @@ namespace libtorrent
 		// once the connection completes
 		int m_connection_ticket;
 
-		// if this is -1, superseeding is not active. If it is >= 0
+		// if [0] is -1, superseeding is not active. If it is >= 0
 		// this is the piece that is available to this peer. Only
-		// this piece can be downloaded from us by this peer.
+		// these two pieces can be downloaded from us by this peer.
 		// This will remain the current piece for this peer until
 		// another peer sends us a have message for this piece
-		int m_superseed_piece;
+		int m_superseed_piece[2];
 
 		// pieces downloaded since last second
 		// timer timeout; used for determining 
@@ -1034,6 +1137,14 @@ namespace libtorrent
 		int m_download_rate_peak;
 		int m_upload_rate_peak;
 
+		// this is the limit on the number of outstanding requests
+		// we have to this peer. This is initialized to the settings
+		// in the session_settings structure. But it may be lowered
+		// if the peer is known to require a smaller limit (like BitComet).
+		// or if the extended handshake sets a limit.
+		// web seeds also has a limit on the queue size.
+		int m_max_out_request_queue;
+
 		// when using the BitTyrant choker, this is our
 		// estimated reciprocation rate. i.e. the rate
 		// we need to send to this peer for it to unchoke
@@ -1048,8 +1159,17 @@ namespace libtorrent
 
 		// the number of request we should queue up
 		// at the remote end.
+		// TODO: 2 rename this target queue size
 		boost::uint16_t m_desired_queue_size;
 
+#ifndef TORRENT_DISABLE_RESOLVE_COUNTRIES	
+		// in case the session settings is set
+		// to resolve countries, this is set to
+		// the two character country code this
+		// peer resides in.
+		char m_country[2];
+#endif
+
 		// if set to non-zero, this peer will always prefer
 		// to request entire n pieces, rather than blocks.
 		// where n is the value of this variable.
@@ -1058,14 +1178,6 @@ namespace libtorrent
 		// are preferred.
 		boost::uint8_t m_prefer_whole_pieces;
 		
-		// the number of piece requests we have rejected
-		// in a row because the peer is choked. This is
-		// used to re-send the choked message in case the
-		// other end keeps requesting pieces while being
-		// choked, and eventuelly disconnect if it keeps
-		// requesting too many pieces while being choked
-		boost::uint8_t m_choke_rejects;
-
 		// if this is true, the disconnection
 		// timestamp is not updated when the connection
 		// is closed. This means the time until we can
@@ -1078,30 +1190,6 @@ namespace libtorrent
 		// could be considered: true = local, false = remote
 		bool m_outgoing:1;
 
-		// is true if we learn the incoming connections listening
-		// during the extended handshake
-		bool m_received_listen_port:1;
-
-		// other side says that it's interested in downloading
-		// from us.
-		bool m_peer_interested:1;
-
-		// the other side has told us that it won't send anymore
-		// data to us for a while
-		bool m_peer_choked:1;
-
-		// the peer has pieces we are interested in
-		bool m_interesting:1;
-
-		// we have choked the upload to the peer
-		bool m_choked:1;
-
-		// this is set to true if the connection timed
-		// out or closed the connection. In that
-		// case we will not try to reconnect to
-		// this peer
-		bool m_failed:1;
-
 		// if this is set to true, the peer will not
 		// request bandwidth from the limiter, but instead
 		// just send and receive as much as possible.
@@ -1112,15 +1200,6 @@ namespace libtorrent
 		// unchoker
 		bool m_ignore_unchoke_slots:1;
 
-		// this is set to true when a have_all
-		// message is received. This information
-		// is used to fill the bitmask in init()
-		bool m_have_all:1;
-
-		// this is true if this connection has been added
-		// to the list of connections that will be closed.
-		bool m_disconnecting:1;
-
 		// this is true until this socket has become
 		// writable for the first time (i.e. the
 		// connection completed). While connecting
@@ -1155,57 +1234,10 @@ namespace libtorrent
 		// is set to 1
 		bool m_snubbed:1;
 
-		// this is set to true once the bitfield is received
-		bool m_bitfield_received:1;
-
 		// if this is set to true, the client will not
 		// pick any pieces from this peer
 		bool m_no_download:1;
 
-		// this is set to true if the last time we tried to
-		// pick a piece to download, we could only find
-		// blocks that were already requested from other
-		// peers. In this case, we should not try to pick
-		// another piece until the last one we requested is done
-		bool m_endgame_mode:1;
-
-		// set to true when we've sent the first round of suggests
-		bool m_sent_suggests:1;
-
-		// set to true while we're trying to holepunch
-		bool m_holepunch_mode:1;
-
-		// when this is set, the transfer stats for this connection
-		// is not included in the torrent or session stats
-		bool m_ignore_stats:1;
-
-		// when this is set, the peer_connection socket is
-		// corked, similar to the linux TCP feature TCP_CORK.
-		// we won't send anything to the actual socket, just
-		// buffer messages up in the application layer send
-		// buffer, and send it once we're uncorked.
-		bool m_corked:1;
-
-		// set to true if this peer has metadata, and false
-		// otherwise.
-		bool m_has_metadata:1;
-
-		template <std::size_t Size>
-		struct handler_storage
-		{
-#ifdef TORRENT_DEBUG
-			handler_storage()
-			  : used(false)
-			{}
-
-			bool used;
-#endif
-			boost::aligned_storage<Size> bytes;
-		};
-
-		handler_storage<TORRENT_READ_HANDLER_MAX_SIZE> m_read_handler_storage;
-		handler_storage<TORRENT_WRITE_HANDLER_MAX_SIZE> m_write_handler_storage;
-
 		template <class Handler, std::size_t Size>
 		struct allocating_handler
 		{
@@ -1275,7 +1307,7 @@ namespace libtorrent
 			);
 		}
 
-#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+#if TORRENT_USE_ASSERTS
 	public:
 		bool m_in_constructor:1;
 		bool m_disconnect_started:1;
