@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003, Arvid Norberg
+Copyright (c) 2003-2014, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -135,6 +135,22 @@ namespace libtorrent
 	};
 #endif
 
+#ifdef IPV6_TCLASS
+	struct traffic_class
+	{
+		traffic_class(char val): m_value(val) {}
+		template<class Protocol>
+		int level(Protocol const&) const { return IPPROTO_IPV6; }
+		template<class Protocol>
+		int name(Protocol const&) const { return IPV6_TCLASS; }
+		template<class Protocol>
+		int const* data(Protocol const&) const { return &m_value; }
+		template<class Protocol>
+		size_t size(Protocol const&) const { return sizeof(m_value); }
+		int m_value;
+	};
+#endif
+
 	struct type_of_service
 	{
 #ifdef WIN32
