@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2008, Arvid Norberg
+Copyright (c) 2008-2014, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -146,6 +146,20 @@ public:
 		return m_sock.next_layer().set_option(opt, ec);
 	}
 
+#ifndef BOOST_NO_EXCEPTIONS
+	template <class GettableSocketOption>
+	void get_option(GettableSocketOption& opt)
+	{
+		m_sock.next_layer().get_option(opt);
+	}
+#endif
+
+	template <class GettableSocketOption>
+	error_code get_option(GettableSocketOption& opt, error_code& ec)
+	{
+		return m_sock.next_layer().get_option(opt, ec);
+	}
+
 #ifndef BOOST_NO_EXCEPTIONS
 	template <class Mutable_Buffers>
 	std::size_t read_some(Mutable_Buffers const& buffers)
