@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2012, Arvid Norberg
+Copyright (c) 2012-2014, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -33,9 +33,12 @@ POSSIBILITY OF SUCH DAMAGE.
 #ifndef TORRENT_STRING_UTIL_HPP_INCLUDED
 #define TORRENT_STRING_UTIL_HPP_INCLUDED
 
+#include "libtorrent/config.hpp"
+
 namespace libtorrent
 {
 	TORRENT_EXTRA_EXPORT bool is_alpha(char c);
+	// this is used by bdecode_recursive's header file
 	TORRENT_EXPORT bool is_digit(char c);
 	TORRENT_EXTRA_EXPORT bool is_print(char c);
 	TORRENT_EXTRA_EXPORT bool is_space(char c);
@@ -56,6 +59,15 @@ namespace libtorrent
 	// x cannot be greater than 7
 	void* align_pointer(void* p);
 
+	// searches for separator in the string 'last'. the pointer last points to
+	// is set to point to the first character following the separator.
+	// returns a pointer to a null terminated string starting at last, ending
+	// at the separator (the string is mutated to replace the separator with
+	// a '\0' character). If there is no separator, but the end of the string,
+	// the pointer next points to is set to the last null terminator, which will
+	// make the following invocation return NULL, to indicate the end of the
+	// string.
+	TORRENT_EXTRA_EXPORT char* string_tokenize(char* last, char sep, char** next);
 }
 
 #endif
