@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003, Arvid Norberg
+Copyright (c) 2003-2014, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -63,6 +63,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/intrusive_ptr_base.hpp"
 #include "libtorrent/size_type.hpp"
 #include "libtorrent/union_endpoint.hpp"
+#include "libtorrent/udp_socket.hpp" // for udp_socket_observer
 #ifdef TORRENT_USE_OPENSSL
 #include <boost/asio/ssl/context.hpp>
 #endif
@@ -126,8 +127,6 @@ namespace libtorrent
 		std::string trackerid;
 		boost::uint32_t key;
 		int num_want;
-		std::string ipv6;
-		std::string ipv4;
 		address bind_ip;
 		bool send_stats;
 		bool apply_ip_filter;
@@ -155,6 +154,7 @@ namespace libtorrent
 			, int min_interval
 			, int complete
 			, int incomplete
+			, int downloaded
 			, address const& external_ip
 			, std::string const& trackerid) = 0;
 		virtual void tracker_request_error(
@@ -190,12 +190,7 @@ namespace libtorrent
 
 		io_service& get_io_service() { return m_timeout.get_io_service(); }
 
-#if !defined TORRENT_VERBOSE_LOGGING \
-	&& !defined TORRENT_LOGGING \
-	&& !defined TORRENT_ERROR_LOGGING
-	// necessary for logging member offsets
 	private:
-#endif
 	
 		void timeout_callback(error_code const&);
 
@@ -246,12 +241,7 @@ namespace libtorrent
 		boost::intrusive_ptr<tracker_connection> self()
 		{ return boost::intrusive_ptr<tracker_connection>(this); }
 
-#if !defined TORRENT_VERBOSE_LOGGING \
-	&& !defined TORRENT_LOGGING \
-	&& !defined TORRENT_ERROR_LOGGING
-	// necessary for logging member offsets
 	protected:
-#endif
 
 		void fail_impl(error_code const& ec, int code = -1, std::string msg = std::string()
 			, int interval = 0, int min_interval = 0);
@@ -260,17 +250,12 @@ namespace libtorrent
 
 		tracker_manager& m_man;
 
-#if !defined TORRENT_VERBOSE_LOGGING \
-	&& !defined TORRENT_LOGGING \
-	&& !defined TORRENT_ERROR_LOGGING
-	// necessary for logging member offsets
 	private:
-#endif
 
 		const tracker_request m_req;
 	};
 
-	class TORRENT_EXTRA_EXPORT tracker_manager: boost::noncopyable
+	class TORRENT_EXTRA_EXPORT tracker_manager: public udp_socket_observer, boost::noncopyable
 	{
 	public:
 
@@ -296,11 +281,13 @@ namespace libtorrent
 		void sent_bytes(int bytes);
 		void received_bytes(int bytes);
 
-		bool incoming_udp(error_code const& e, udp::endpoint const& ep, char const* buf, int size);
+		virtual bool incoming_packet(error_code const& e, udp::endpoint const& ep
+			, char const* buf, int size);
 
 		// this is only used for SOCKS packets, since
 		// they may be addressed to hostname
-		bool incoming_udp(error_code const& e, char const* hostname, char const* buf, int size);
+		virtual bool incoming_packet(error_code const& e, char const* hostname
+			, char const* buf, int size);
 		
 	private:
 
