@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2007, Arvid Norberg
+Copyright (c) 2007-2014, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -42,24 +42,35 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/deadline_timer.hpp"
 
 #include <deque>
-#include <boost/function/function4.hpp>
 
 namespace libtorrent
 {
 	class connection_queue;
 
+	struct udp_socket_observer
+	{
+		// return true if the packet was handled (it won't be
+		// propagated to the next observer)
+		virtual bool incoming_packet(error_code const& ec
+			, udp::endpoint const&, char const* buf, int size) = 0;
+		virtual bool incoming_packet(error_code const& ec
+			, char const* hostname, char const* buf, int size) { return false; }
+
+		// called when the socket becomes writeable, after having
+		// failed with EWOULDBLOCK
+		virtual void writable() {}
+
+		// called every time the socket is drained of packets
+		virtual void socket_drained() {}
+	};
+
 	class udp_socket
 	{
 	public:
-		typedef boost::function<void(error_code const& ec
-			, udp::endpoint const&, char const* buf, int size)> callback_t;
-		typedef boost::function<void(error_code const& ec
-			, char const*, char const* buf, int size)> callback2_t;
-
-		udp_socket(io_service& ios, callback_t const& c, callback2_t const& c2, connection_queue& cc);
+		udp_socket(io_service& ios, connection_queue& cc);
 		~udp_socket();
 
-		enum flags_t { dont_drop = 1, peer_connection = 2 };
+		enum flags_t { dont_drop = 1, peer_connection = 2, dont_queue = 4 };
 
 		bool is_open() const
 		{
@@ -71,12 +82,16 @@ namespace libtorrent
 		}
 		io_service& get_io_service() { return m_ipv4_sock.get_io_service(); }
 
-		// this is only valid when using a socks5 proxy
-		void send_hostname(char const* hostname, int port, char const* p, int len, error_code& ec);
+		void subscribe(udp_socket_observer* o);
+		void unsubscribe(udp_socket_observer* o);
 
-		void send(udp::endpoint const& ep, char const* p, int len, error_code& ec, int flags = 0);
+		// this is only valid when using a socks5 proxy
+		void send_hostname(char const* hostname, int port, char const* p
+			, int len, error_code& ec, int flags = 0);
+
+		void send(udp::endpoint const& ep, char const* p, int len
+			, error_code& ec, int flags = 0);
 		void bind(udp::endpoint const& ep, error_code& ec);
-		void bind(int port);
 		void close();
 		int local_port() const { return m_bind_port; }
 
@@ -93,6 +108,15 @@ namespace libtorrent
 
 		void set_buf_size(int s);
 
+		template <class SocketOption>
+		void get_option(SocketOption const& opt, error_code& ec)
+		{
+			m_ipv4_sock.get_option(opt, ec);
+#if TORRENT_USE_IPV6
+			m_ipv6_sock.get_option(opt, ec);
+#endif
+		}
+
 		template <class SocketOption>
 		void set_option(SocketOption const& opt, error_code& ec)
 		{
@@ -114,6 +138,11 @@ namespace libtorrent
 
 		struct queued_packet
 		{
+			queued_packet()
+				: hostname(NULL)
+				, flags(0)
+			{}
+
 			udp::endpoint ep;
 			char* hostname;
 			buffer buf;
@@ -126,25 +155,42 @@ namespace libtorrent
 		{
 			return m_v4_outstanding
 #if TORRENT_USE_IPV6
-			  	+ m_v6_outstanding
+				+ m_v6_outstanding
 #endif
 				;
 		}
 
 	private:
-#if defined TORRENT_VERBOSE_LOGGING || defined TORRENT_LOGGING || defined TORRENT_ERROR_LOGGING
-	// necessary for logging member offsets
-	public:
-#endif
 
-		// callback for regular incoming packets
-		callback_t m_callback;
+		// non-copyable
+		udp_socket(udp_socket const&);
+		udp_socket& operator=(udp_socket const&);
 
-		// callback for proxied incoming packets with a domain
-		// name as source
-		callback2_t m_callback2;
+		// observers on this udp socket
+		std::vector<udp_socket_observer*> m_observers;
+		std::vector<udp_socket_observer*> m_added_observers;
 
-		void on_read(udp::socket* sock, error_code const& e, std::size_t bytes_transferred);
+		// this is true while iterating over the observers
+		// vector, invoking observer hooks. We may not
+		// add new observers during this time, since it
+		// may invalidate the iterator. If this is true,
+		// instead add new observers to m_added_observers
+		// and they will be added later
+		bool m_observers_locked;
+
+		void call_handler(error_code const& ec, udp::endpoint const& ep
+			, char const* buf, int size);
+		void call_handler(error_code const& ec, const char* host
+			, char const* buf, int size);
+		void call_drained_handler();
+		void call_writable_handler();
+
+		void on_writable(error_code const& ec, udp::socket* s);
+
+		void setup_read(udp::socket* s);
+		void on_read(error_code const& ec, udp::socket* s);
+		void on_read_impl(udp::socket* sock, udp::endpoint const& ep
+			, error_code const& e, std::size_t bytes_transferred);
 		void on_name_lookup(error_code const& e, tcp::resolver::iterator i);
 		void on_timeout();
 		void on_connect(int ticket);
@@ -164,10 +210,8 @@ namespace libtorrent
 		void wrap(char const* hostname, int port, char const* p, int len, error_code& ec);
 		void unwrap(error_code const& e, char const* buf, int size);
 
-		void maybe_realloc_buffers(int which = 3);
-		bool maybe_clear_callback();
+#if TORRENT_USE_ASSERTS
 
-#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
 #if defined BOOST_HAS_PTHREADS
 		mutable pthread_t m_thread;
 #endif
@@ -183,25 +227,18 @@ namespace libtorrent
 #endif
 
 		udp::socket m_ipv4_sock;
-		udp::endpoint m_v4_ep;
-		int m_v4_buf_size;
-		char* m_v4_buf;
-		// this is set to true to indicate that the
-		// m_v4_buf should be reallocated to the size
-		// of the buffer size members the next time their
-		// read handler gets triggered
-		bool m_reallocate_buffer4;
+		int m_buf_size;
+
+		// if the buffer size is attempted
+		// to be changed while the buffer is
+		// being used, this member is set to
+		// the desired size, and it's resized
+		// later
+		int m_new_buf_size;
+		char* m_buf;
 
 #if TORRENT_USE_IPV6
 		udp::socket m_ipv6_sock;
-		udp::endpoint m_v6_ep;
-		int m_v6_buf_size;
-		char* m_v6_buf;
-		// this is set to true to indicate that the
-		// m_v6_buf should be reallocated to the size
-		// of the buffer size members the next time their
-		// read handler gets triggered
-		bool m_reallocate_buffer6;
 #endif
 
 		boost::uint16_t m_bind_port;
@@ -241,7 +278,12 @@ namespace libtorrent
 		// operations hanging on this socket
 		int m_outstanding_ops;
 
-#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+#if TORRENT_USE_IPV6
+		bool m_v6_write_subscribed:1;
+#endif
+		bool m_v4_write_subscribed:1;
+
+#if TORRENT_USE_ASSERTS
 		bool m_started;
 		int m_magic;
 		int m_outstanding_when_aborted;
@@ -250,14 +292,17 @@ namespace libtorrent
 		int m_outstanding_resolve;
 		int m_outstanding_connect_queue;
 		int m_outstanding_socks;
+
+		char timeout_stack[2000];
 #endif
 	};
 
 	struct rate_limited_udp_socket : public udp_socket
 	{
-		rate_limited_udp_socket(io_service& ios, callback_t const& c, callback2_t const& c2, connection_queue& cc);
+		rate_limited_udp_socket(io_service& ios, connection_queue& cc);
 		void set_rate_limit(int limit) { m_rate_limit = limit; }
-		bool send(udp::endpoint const& ep, char const* p, int len, error_code& ec, int flags = 0);
+		bool send(udp::endpoint const& ep, char const* p, int len
+			, error_code& ec, int flags = 0);
 
 	private:
 
