@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003, Arvid Norberg
+Copyright (c) 2003-2014, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -78,12 +78,7 @@ namespace libtorrent
 		void start();
 		void close();
 
-#if !defined TORRENT_VERBOSE_LOGGING \
-	&& !defined TORRENT_LOGGING \
-	&& !defined TORRENT_ERROR_LOGGING
-	// necessary for logging member offsets
 	private:
-#endif
 
 		enum action_t
 		{
@@ -120,8 +115,6 @@ namespace libtorrent
 
 		udp::endpoint pick_target_endpoint() const;
 
-//		tracker_manager& m_man;
-
 		bool m_abort;
 		std::string m_hostname;
 		udp::endpoint m_target;
