@@ -33,15 +33,24 @@ POSSIBILITY OF SUCH DAMAGE.
 #ifndef TORRENT_VERSION_HPP_INCLUDED
 #define TORRENT_VERSION_HPP_INCLUDED
 
+#include "libtorrent/export.hpp"
+
 #define LIBTORRENT_VERSION_MAJOR 1
 #define LIBTORRENT_VERSION_MINOR 0
-#define LIBTORRENT_VERSION_TINY 6
+#define LIBTORRENT_VERSION_TINY 7
 
 // the format of this version is: MMmmtt
 // M = Major version, m = minor version, t = tiny version
 #define LIBTORRENT_VERSION_NUM ((LIBTORRENT_VERSION_MAJOR * 10000) + (LIBTORRENT_VERSION_MINOR * 100) + LIBTORRENT_VERSION_TINY)
 
-#define LIBTORRENT_VERSION "1.0.6.0"
-#define LIBTORRENT_REVISION "$Rev$" 
+#define LIBTORRENT_VERSION "1.0.7.0"
+#define LIBTORRENT_REVISION "0dff874"
+namespace libtorrent {
+
+	// returns the libtorrent version as string form in this format:
+	// "<major>.<minor>.<tiny>.<tag>.<revision>"
+	TORRENT_EXPORT char const* version();
+
+}
 
 #endif
