@@ -7,6 +7,8 @@
 
 #include "libtorrent/config.hpp"
 
+#ifdef TORRENT_BUILDING_SHARED
+
 #define TORRENT_HAS_ASIO_DECL x ## BOOST_ASIO_DECL
 
 // only define BOOST_ASIO_DECL if it hasn't already been defined
@@ -15,6 +17,8 @@
 #define BOOST_ASIO_DECL BOOST_SYMBOL_EXPORT
 #endif
 
+#endif // TORRENT_BUILDING_SHARED
+
 #if BOOST_VERSION >= 104500
 #include <boost/asio/impl/src.hpp>
 #elif BOOST_VERSION >= 104400
