@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2007-2011, Arvid Norberg
+Copyright (c) 2007-2014, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -47,6 +47,10 @@ namespace libtorrent
 	disk_buffer_pool::disk_buffer_pool(int block_size)
 		: m_block_size(block_size)
 		, m_in_use(0)
+#ifndef TORRENT_DISABLE_POOL_ALLOCATOR
+		, m_using_pool_allocator(false)
+		, m_pool(block_size, m_settings.cache_buffer_chunk_size)
+#endif
 	{
 #if defined TORRENT_DISK_STATS || defined TORRENT_STATS
 		m_allocations = 0;
@@ -58,12 +62,12 @@ namespace libtorrent
 
 		m_disk_access_log.open("disk_access.log", std::ios::trunc);
 #endif
-#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+#if TORRENT_USE_ASSERTS
 		m_magic = 0x1337;
 #endif
 	}
 
-#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+#if TORRENT_USE_ASSERTS
 	disk_buffer_pool::~disk_buffer_pool()
 	{
 		TORRENT_ASSERT(m_magic == 0x1337);
@@ -71,7 +75,7 @@ namespace libtorrent
 	}
 #endif
 
-#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS || defined TORRENT_DISK_STATS
+#if TORRENT_USE_ASSERTS || defined TORRENT_DISK_STATS
 	bool disk_buffer_pool::is_disk_buffer(char* buffer
 		, mutex::scoped_lock& l) const
 	{
@@ -80,7 +84,14 @@ namespace libtorrent
 		if (m_buf_to_category.find(buffer)
 			== m_buf_to_category.end()) return false;
 #endif
+#ifdef TORRENT_DISABLE_POOL_ALLOCATOR
 		return true;
+#else
+		if (m_using_pool_allocator)
+			return m_pool.is_from(buffer);
+		else
+			return true;
+#endif
 	}
 
 	bool disk_buffer_pool::is_disk_buffer(char* buffer) const
@@ -94,7 +105,20 @@ namespace libtorrent
 	{
 		mutex::scoped_lock l(m_pool_mutex);
 		TORRENT_ASSERT(m_magic == 0x1337);
-		char* ret = page_aligned_allocator::malloc(m_block_size);
+		char* ret;
+#ifdef TORRENT_DISABLE_POOL_ALLOCATOR
+		ret = page_aligned_allocator::malloc(m_block_size);
+#else
+		if (m_using_pool_allocator)
+		{
+			ret = (char*)m_pool.malloc();
+			m_pool.set_next_size(m_settings.cache_buffer_chunk_size);
+		}
+		else
+		{
+			ret = page_aligned_allocator::malloc(m_block_size);
+		}
+#endif
 		++m_in_use;
 #if TORRENT_USE_MLOCK
 		if (m_settings.lock_disk_cache)
@@ -185,13 +209,34 @@ namespace libtorrent
 #endif		
 		}
 #endif
+#ifdef TORRENT_DISABLE_POOL_ALLOCATOR
 		page_aligned_allocator::free(buf);
+#else
+		if (m_using_pool_allocator)
+			m_pool.free(buf);
+		else
+			page_aligned_allocator::free(buf);
+#endif
 		--m_in_use;
+
+#ifndef TORRENT_DISABLE_POOL_ALLOCATOR
+		// should we switch which allocator to use?
+		if (m_in_use == 0 && m_settings.use_disk_cache_pool != m_using_pool_allocator)
+		{
+			m_pool.release_memory();
+			m_using_pool_allocator = m_settings.use_disk_cache_pool;
+		}
+#endif
 	}
 
 	void disk_buffer_pool::release_memory()
 	{
 		TORRENT_ASSERT(m_magic == 0x1337);
+#ifndef TORRENT_DISABLE_POOL_ALLOCATOR
+		mutex::scoped_lock l(m_pool_mutex);
+		if (m_using_pool_allocator)
+			m_pool.release_memory();
+#endif
 	}
 }
 
