@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003-2008, Arvid Norberg
+Copyright (c) 2003-2014, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -30,8 +30,6 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
-#include "libtorrent/pch.hpp"
-
 #include "libtorrent/file_storage.hpp"
 #include "libtorrent/string_util.hpp" // for allocate_string_copy
 #include "libtorrent/file.hpp"
@@ -48,6 +46,41 @@ namespace libtorrent
 		, m_piece_length(0)
 	{}
 
+	file_storage::~file_storage() {}
+
+	// even though this copy constructor and the copy assignment
+	// operator are identical to what the compiler would have
+	// generated, they are put here to explicitly make them part
+	// of libtorrent and properly exported by the .dll.
+	file_storage::file_storage(file_storage const& f)
+		: m_files(f.m_files)
+		, m_file_hashes(f.m_file_hashes)
+		, m_symlinks(f.m_symlinks)
+		, m_mtime(f.m_mtime)
+		, m_file_base(f.m_file_base)
+		, m_paths(f.m_paths)
+		, m_name(f.m_name)
+		, m_total_size(f.m_total_size)
+		, m_num_pieces(f.m_num_pieces)
+		, m_piece_length(f.m_piece_length)
+	{
+	}
+
+	file_storage& file_storage::operator=(file_storage const& f)
+	{
+		m_files = f.m_files;
+		m_file_hashes = f.m_file_hashes;
+		m_symlinks = f.m_symlinks;
+		m_mtime = f.m_mtime;
+		m_file_base = f.m_file_base;
+		m_paths = f.m_paths;
+		m_name = f.m_name;
+		m_total_size = f.m_total_size;
+		m_num_pieces = f.m_num_pieces;
+		m_piece_length = f.m_piece_length;
+		return *this;
+	}
+
 	void file_storage::reserve(int num_files)
 	{
 		m_files.reserve(num_files);
@@ -55,7 +88,7 @@ namespace libtorrent
 
 	int file_storage::piece_size(int index) const
 	{
-		TORRENT_ASSERT(index >= 0 && index < num_pieces());
+		TORRENT_ASSERT_PRECOND(index >= 0 && index < num_pieces());
 		if (index == num_pieces()-1)
 		{
 			size_type size_except_last = num_pieces() - 1;
@@ -71,7 +104,13 @@ namespace libtorrent
 
 	void file_storage::update_path_index(internal_file_entry& e)
 	{
-		std::string parent = parent_path(e.filename());
+		std::string fname = e.filename();
+		if (is_complete(fname))
+		{
+			e.path_index = -2;
+			return;
+		}
+		std::string parent = parent_path(fname);
 
 		if (parent.empty())
 		{
@@ -123,19 +162,22 @@ namespace libtorrent
 
 	file_entry::~file_entry() {}
 
-	internal_file_entry::~internal_file_entry() { if (name_len == 0) free((void*)name); }
+	internal_file_entry::~internal_file_entry()
+	{
+		if (name_len == name_is_owned) free((void*)name);
+	}
 
 	internal_file_entry::internal_file_entry(internal_file_entry const& fe)
-		: name(0)
-		, offset(fe.offset)
+		: offset(fe.offset)
 		, symlink_index(fe.symlink_index)
+		, no_root_dir(fe.no_root_dir)
 		, size(fe.size)
 		, name_len(fe.name_len)
 		, pad_file(fe.pad_file)
 		, hidden_attribute(fe.hidden_attribute)
 		, executable_attribute(fe.executable_attribute)
 		, symlink_attribute(fe.symlink_attribute)
-		, no_root_dir(fe.no_root_dir)
+		, name(0)
 		, path_index(fe.path_index)
 	{
 		set_name(fe.filename().c_str());
@@ -156,26 +198,40 @@ namespace libtorrent
 		return *this;
 	}
 
-	void internal_file_entry::set_name(char const* n, int borrow_chars)
+	// if borrow_chars >= 0, don't take ownership over n, just
+	// point to it. It points to borrow_chars number of characters.
+	// if borrow_chars == -1, n is a null terminated string that
+	// should be copied 
+	void internal_file_entry::set_name(char const* n, bool borrow_string, int string_len)
 	{
-		TORRENT_ASSERT(borrow_chars >= 0);
-		if (borrow_chars > 1023) borrow_chars = 1023;
-		if (name_len == 0) free((void*)name);
-		if (n == 0 || *n == 0)
+		TORRENT_ASSERT(string_len >= 0);
+
+		// we have limited space in the length field. truncate string
+		// if it's too long
+		if (string_len >= name_is_owned) string_len = name_is_owned - 1;
+
+		// free the current string, before assigning the new one
+		if (name_len == name_is_owned) free((void*)name);
+		if (n == NULL)
 		{
-			TORRENT_ASSERT(borrow_chars == 0);
-			name = 0;
+			TORRENT_ASSERT(borrow_string == false);
+			name = NULL;
+		}
+		else if (borrow_string)
+		{
+			name = n;
+			name_len = string_len;
 		}
 		else
 		{
-			name = borrow_chars ? n : allocate_string_copy(n);
+			name = allocate_string_copy(n);
+			name_len = name_is_owned;
 		}
-		name_len = borrow_chars;
 	}
 
 	std::string internal_file_entry::filename() const
 	{
-		if (name_len) return std::string(name, name_len);
+		if (name_len != name_is_owned) return std::string(name, name_len);
 		return name ? name : "";
 	}
 
@@ -188,9 +244,9 @@ namespace libtorrent
 		m_name = utf8;
 	}
 
-	void file_storage::rename_file(int index, std::wstring const& new_filename)
+	void file_storage::rename_file_deprecated(int index, std::wstring const& new_filename)
 	{
-		TORRENT_ASSERT(index >= 0 && index < int(m_files.size()));
+		TORRENT_ASSERT_PRECOND(index >= 0 && index < int(m_files.size()));
 		std::string utf8;
 		wchar_utf8(new_filename, utf8);
 		m_files[index].set_name(utf8.c_str());
@@ -204,16 +260,27 @@ namespace libtorrent
 		wchar_utf8(file, utf8);
 		add_file(utf8, size, flags, mtime, symlink_path);
 	}
+
+	void file_storage::rename_file(int index, std::wstring const& new_filename)
+	{
+		rename_file_deprecated(index, new_filename);
+	}
 #endif // TORRENT_NO_DEPRECATE
 #endif // TORRENT_USE_WSTRING
 
 	void file_storage::rename_file(int index, std::string const& new_filename)
 	{
-		TORRENT_ASSERT(index >= 0 && index < int(m_files.size()));
+		TORRENT_ASSERT_PRECOND(index >= 0 && index < int(m_files.size()));
 		m_files[index].set_name(new_filename.c_str());
 		update_path_index(m_files[index]);
 	}
 
+	void file_storage::rename_file_borrow(int index, char const* new_filename, int len)
+	{
+		TORRENT_ASSERT_PRECOND(index >= 0 && index < int(m_files.size()));
+		m_files[index].set_name(new_filename, true, len);
+	}
+
 	namespace
 	{
 		bool compare_file_offset(internal_file_entry const& lhs, internal_file_entry const& rhs)
@@ -222,7 +289,8 @@ namespace libtorrent
 		}
 	}
 
-	file_storage::iterator file_storage::file_at_offset(size_type offset) const
+#ifndef TORRENT_NO_DEPRECATE
+	file_storage::iterator file_storage::file_at_offset_deprecated(size_type offset) const
 	{
 		// find the file iterator and file offset
 		internal_file_entry target;
@@ -230,17 +298,50 @@ namespace libtorrent
 		TORRENT_ASSERT(!compare_file_offset(target, m_files.front()));
 
 		std::vector<internal_file_entry>::const_iterator file_iter = std::upper_bound(
-			begin(), end(), target, compare_file_offset);
+			begin_deprecated(), end_deprecated(), target, compare_file_offset);
 
-		TORRENT_ASSERT(file_iter != begin());
+		TORRENT_ASSERT(file_iter != begin_deprecated());
 		--file_iter;
 		return file_iter;
 	}
 
+	file_storage::iterator file_storage::file_at_offset(size_type offset) const
+	{
+		return file_at_offset_deprecated(offset);
+	}
+#endif
+
+	int file_storage::file_index_at_offset(size_type offset) const
+	{
+		// find the file iterator and file offset
+		internal_file_entry target;
+		target.offset = offset;
+		TORRENT_ASSERT(!compare_file_offset(target, m_files.front()));
+
+		std::vector<internal_file_entry>::const_iterator file_iter = std::upper_bound(
+			m_files.begin(), m_files.end(), target, compare_file_offset);
+
+		TORRENT_ASSERT(file_iter != m_files.begin());
+		--file_iter;
+		return file_iter - m_files.begin();
+	}
+
+	char const* file_storage::file_name_ptr(int index) const
+	{
+		return m_files[index].name;
+	}
+
+	int file_storage::file_name_len(int index) const
+	{
+		if (m_files[index].name_len == internal_file_entry::name_is_owned)
+			return -1;
+		return m_files[index].name_len;
+	}
+
 	std::vector<file_slice> file_storage::map_block(int piece, size_type offset
 		, int size) const
 	{
-		TORRENT_ASSERT(num_files() > 0);
+		TORRENT_ASSERT_PRECOND(num_files() > 0);
 		std::vector<file_slice> ret;
 
 		if (m_files.empty()) return ret;
@@ -248,25 +349,25 @@ namespace libtorrent
 		// find the file iterator and file offset
 		internal_file_entry target;
 		target.offset = piece * (size_type)m_piece_length + offset;
-		TORRENT_ASSERT(target.offset + size <= m_total_size);
+		TORRENT_ASSERT_PRECOND(size_type(target.offset + size) <= m_total_size);
 		TORRENT_ASSERT(!compare_file_offset(target, m_files.front()));
 
 		std::vector<internal_file_entry>::const_iterator file_iter = std::upper_bound(
-			begin(), end(), target, compare_file_offset);
+			m_files.begin(), m_files.end(), target, compare_file_offset);
 
-		TORRENT_ASSERT(file_iter != begin());
+		TORRENT_ASSERT(file_iter != m_files.begin());
 		--file_iter;
 
 		size_type file_offset = target.offset - file_iter->offset;
 		for (; size > 0; file_offset -= file_iter->size, ++file_iter)
 		{
-			TORRENT_ASSERT(file_iter != end());
-			if (file_offset < file_iter->size)
+			TORRENT_ASSERT(file_iter != m_files.end());
+			if (file_offset < size_type(file_iter->size))
 			{
 				file_slice f;
-				f.file_index = file_iter - begin();
-				f.offset = file_offset + file_base(*file_iter);
-				f.size = (std::min)(file_iter->size - file_offset, boost::uint64_t(size));
+				f.file_index = file_iter - m_files.begin();
+				f.offset = file_offset + file_base(f.file_index);
+				f.size = (std::min)(boost::uint64_t(file_iter->size) - file_offset, boost::uint64_t(size));
 				TORRENT_ASSERT(f.size <= size);
 				size -= int(f.size);
 				file_offset += f.size;
@@ -278,34 +379,31 @@ namespace libtorrent
 		return ret;
 	}
 
-	file_entry file_storage::at(file_storage::iterator i) const
-	{ return at(i - begin()); }
-
 	file_entry file_storage::at(int index) const
 	{
-		TORRENT_ASSERT(index >= 0 && index < int(m_files.size()));
+		TORRENT_ASSERT_PRECOND(index >= 0 && index < int(m_files.size()));
 		file_entry ret;
 		internal_file_entry const& ife = m_files[index];
-		ret.path = file_path(ife);
+		ret.path = file_path(index);
 		ret.offset = ife.offset;
 		ret.size = ife.size;
-		ret.file_base = file_base(ife);
-		ret.mtime = mtime(ife);
+		ret.file_base = file_base(index);
+		ret.mtime = mtime(index);
 		ret.pad_file = ife.pad_file;
 		ret.hidden_attribute = ife.hidden_attribute;
 		ret.executable_attribute = ife.executable_attribute;
 		ret.symlink_attribute = ife.symlink_attribute;
-		if (ife.symlink_index != internal_file_entry::no_symlink_idx)
-			ret.symlink_path = symlink(ife);
-		ret.filehash = hash(ife);
+		if (ife.symlink_index != internal_file_entry::not_a_symlink)
+			ret.symlink_path = symlink(index);
+		ret.filehash = hash(index);
 		return ret;
 	}
 
 	peer_request file_storage::map_file(int file_index, size_type file_offset
 		, int size) const
 	{
-		TORRENT_ASSERT(file_index < num_files());
-		TORRENT_ASSERT(file_index >= 0);
+		TORRENT_ASSERT_PRECOND(file_index < num_files());
+		TORRENT_ASSERT_PRECOND(file_index >= 0);
 		TORRENT_ASSERT(m_num_pieces >= 0);
 
 		peer_request ret;
@@ -317,7 +415,7 @@ namespace libtorrent
 			return ret;
 		}
 
-		size_type offset = file_offset + internal_at(file_index).offset;
+		size_type offset = file_offset + this->file_offset(file_index);
 
 		if (offset >= total_size())
 		{
@@ -331,7 +429,7 @@ namespace libtorrent
 			ret.start = int(offset % piece_length());
 			ret.length = size;
 			if (offset + size > total_size())
-				ret.length = total_size() - offset;
+				ret.length = int(total_size() - offset);
 		}
 		return ret;
 	}
@@ -339,8 +437,8 @@ namespace libtorrent
 	void file_storage::add_file(std::string const& file, size_type size, int flags
 		, std::time_t mtime, std::string const& symlink_path)
 	{
-		TORRENT_ASSERT(!is_complete(file));
-		TORRENT_ASSERT(size >= 0);
+		TORRENT_ASSERT_PRECOND(!is_complete(file));
+		TORRENT_ASSERT_PRECOND(size >= 0);
 		if (size < 0) size = 0;
 		if (!has_parent_path(file))
 		{
@@ -348,7 +446,7 @@ namespace libtorrent
 			// path to the file (branch_path), which means that
 			// all the other files need to be in the same top
 			// directory as the first file.
-			TORRENT_ASSERT(m_files.empty());
+			TORRENT_ASSERT_PRECOND(m_files.empty());
 			m_name = file;
 		}
 		else
@@ -356,7 +454,7 @@ namespace libtorrent
 			if (m_files.empty())
 				m_name = split_path(file).c_str();
 		}
-		TORRENT_ASSERT(m_name == split_path(file).c_str());
+		TORRENT_ASSERT_PRECOND(m_name == split_path(file).c_str());
 		m_files.push_back(internal_file_entry());
 		internal_file_entry& e = m_files.back();
 		e.set_name(file.c_str());
@@ -365,12 +463,15 @@ namespace libtorrent
 		e.pad_file = (flags & pad_file) != 0;
 		e.hidden_attribute = (flags & attribute_hidden) != 0;
 		e.executable_attribute = (flags & attribute_executable) != 0;
-		e.symlink_attribute = (flags & attribute_symlink) != 0;
-		if (e.symlink_attribute && m_symlinks.size() < internal_file_entry::no_symlink_idx - 1)
+		if ((flags & attribute_symlink) && m_symlinks.size() < internal_file_entry::not_a_symlink - 1)
 		{
+			e.symlink_attribute = 1;
 			e.symlink_index = m_symlinks.size();
 			m_symlinks.push_back(symlink_path);
 		}
+		else
+			e.symlink_attribute = 0;
+
 		if (mtime)
 		{
 			if (m_mtime.size() < m_files.size()) m_mtime.resize(m_files.size());
@@ -381,16 +482,19 @@ namespace libtorrent
 		m_total_size += size;
 	}
 
+	// TODO: 2 it would be nice if file_entry::filehash could be taken into
+	// account as well, and if the file_storage object could actually hold
+	// copies of filehash
 	void file_storage::add_file(file_entry const& ent, char const* filehash)
 	{
-		TORRENT_ASSERT(ent.size >= 0);
+		TORRENT_ASSERT_PRECOND(ent.size >= 0);
 		if (!has_parent_path(ent.path))
 		{
 			// you have already added at least one file with a
 			// path to the file (branch_path), which means that
 			// all the other files need to be in the same top
 			// directory as the first file.
-			TORRENT_ASSERT(m_files.empty());
+			TORRENT_ASSERT_PRECOND(m_files.empty());
 			m_name = ent.path;
 		}
 		else
@@ -399,6 +503,7 @@ namespace libtorrent
 				m_name = split_path(ent.path).c_str();
 		}
 		internal_file_entry ife(ent);
+		int file_index = m_files.size();
 		m_files.push_back(ife);
 		internal_file_entry& e = m_files.back();
 		e.offset = m_total_size;
@@ -408,7 +513,7 @@ namespace libtorrent
 			if (m_file_hashes.size() < m_files.size()) m_file_hashes.resize(m_files.size());
 			m_file_hashes[m_files.size() - 1] = filehash;
 		}
-		if (!ent.symlink_path.empty() && m_symlinks.size() < internal_file_entry::no_symlink_idx - 1)
+		if (!ent.symlink_path.empty() && m_symlinks.size() < internal_file_entry::not_a_symlink - 1)
 		{
 			e.symlink_index = m_symlinks.size();
 			m_symlinks.push_back(ent.symlink_path);
@@ -418,7 +523,7 @@ namespace libtorrent
 			if (m_mtime.size() < m_files.size()) m_mtime.resize(m_files.size());
 			m_mtime[m_files.size() - 1] = ent.mtime;
 		}
-		if (ent.file_base) set_file_base(e, ent.file_base);
+		if (ent.file_base) set_file_base(file_index, ent.file_base);
 		update_path_index(e);
 	}
 
@@ -430,7 +535,7 @@ namespace libtorrent
 	
 	std::string const& file_storage::symlink(int index) const
 	{
-		TORRENT_ASSERT(index >= 0 && index < int(m_files.size()));
+		TORRENT_ASSERT_PRECOND(index >= 0 && index < int(m_files.size()));
 		internal_file_entry const& fe = m_files[index];
 		TORRENT_ASSERT(fe.symlink_index < int(m_symlinks.size()));
 		return m_symlinks[fe.symlink_index];
@@ -442,15 +547,9 @@ namespace libtorrent
 		return m_mtime[index];
 	}
 
-	int file_storage::file_index(int index) const
-	{
-		TORRENT_ASSERT(index >= 0 && index < int(m_files.size()));
-		return index;
-	}
-
 	void file_storage::set_file_base(int index, size_type off)
 	{
-		TORRENT_ASSERT(index >= 0 && index < int(m_files.size()));
+		TORRENT_ASSERT_PRECOND(index >= 0 && index < int(m_files.size()));
 		if (int(m_file_base.size()) <= index) m_file_base.resize(index + 1, 0);
 		m_file_base[index] = off;
 	}
@@ -461,27 +560,63 @@ namespace libtorrent
 		return m_file_base[index];
 	}
 
-	std::string file_storage::file_path(int index) const
+	std::string file_storage::file_path(int index, std::string const& save_path) const
 	{
-		TORRENT_ASSERT(index >= 0 && index < int(m_files.size()));
+		TORRENT_ASSERT_PRECOND(index >= 0 && index < int(m_files.size()));
 		internal_file_entry const& fe = m_files[index];
-		return file_path(fe);
+
+		// -2 means this is an absolute path filename
+		if (fe.path_index == -2) return fe.filename();
+
+		// -1 means no path
+		if (fe.path_index == -1) return combine_path(save_path, fe.filename());
+
+		if (fe.no_root_dir)
+			return combine_path(save_path
+				, combine_path(m_paths[fe.path_index]
+				, fe.filename()));
+
+		return combine_path(save_path
+			, combine_path(m_name
+			, combine_path(m_paths[fe.path_index]
+			, fe.filename())));
 	}
 
 	std::string file_storage::file_name(int index) const
 	{
-		TORRENT_ASSERT(index >= 0 && index < int(m_files.size()));
+		TORRENT_ASSERT_PRECOND(index >= 0 && index < int(m_files.size()));
 		internal_file_entry const& fe = m_files[index];
 		return fe.filename();
 	}
 
-
 	size_type file_storage::file_size(int index) const
 	{
-		TORRENT_ASSERT(index >= 0 && index < int(m_files.size()));
+		TORRENT_ASSERT_PRECOND(index >= 0 && index < int(m_files.size()));
 		return m_files[index].size;
 	}
 
+	bool file_storage::pad_file_at(int index) const
+	{
+		TORRENT_ASSERT_PRECOND(index >= 0 && index < int(m_files.size()));
+		return m_files[index].pad_file;
+	}
+
+	size_type file_storage::file_offset(int index) const
+	{
+		TORRENT_ASSERT_PRECOND(index >= 0 && index < int(m_files.size()));
+		return m_files[index].offset;
+	}
+
+	int file_storage::file_flags(int index) const
+	{
+		internal_file_entry const& fe = m_files[index];
+		return (fe.pad_file ? flag_pad_file : 0)
+			| (fe.hidden_attribute ? flag_hidden : 0)
+			| (fe.executable_attribute ? flag_executable : 0)
+			| (fe.symlink_attribute ? flag_symlink : 0);
+	}
+
+#ifndef TORRENT_NO_DEPRECATE
 	sha1_hash file_storage::hash(internal_file_entry const& fe) const
 	{
 		int index = &fe - &m_files[0];
@@ -491,7 +626,7 @@ namespace libtorrent
 	
 	std::string const& file_storage::symlink(internal_file_entry const& fe) const
 	{
-		TORRENT_ASSERT(fe.symlink_index < int(m_symlinks.size()));
+		TORRENT_ASSERT_PRECOND(fe.symlink_index < int(m_symlinks.size()));
 		return m_symlinks[fe.symlink_index];
 	}
 
@@ -505,14 +640,14 @@ namespace libtorrent
 	int file_storage::file_index(internal_file_entry const& fe) const
 	{
 		int index = &fe - &m_files[0];
-		TORRENT_ASSERT(index >= 0 && index < int(m_files.size()));
+		TORRENT_ASSERT_PRECOND(index >= 0 && index < int(m_files.size()));
 		return index;
 	}
 
 	void file_storage::set_file_base(internal_file_entry const& fe, size_type off)
 	{
 		int index = &fe - &m_files[0];
-		TORRENT_ASSERT(index >= 0 && index < int(m_files.size()));
+		TORRENT_ASSERT_PRECOND(index >= 0 && index < int(m_files.size()));
 		if (int(m_file_base.size()) <= index) m_file_base.resize(index + 1, 0);
 		m_file_base[index] = off;
 	}
@@ -524,20 +659,16 @@ namespace libtorrent
 		return m_file_base[index];
 	}
 
-	std::string file_storage::file_path(internal_file_entry const& fe) const
+	std::string file_storage::file_path(internal_file_entry const& fe
+		, std::string const& save_path) const
 	{
-		TORRENT_ASSERT(fe.path_index >= -1 && fe.path_index < int(m_paths.size()));
+		int index = &fe - &m_files[0];
+		return file_path(index);
+	}
 
-		// -1 means no path
-		if (fe.path_index == -1) return fe.filename();
-
-		if (fe.no_root_dir)
-			return combine_path(m_paths[fe.path_index]
-				, fe.filename());
-
-		return combine_path(m_name
-			, combine_path(m_paths[fe.path_index]
-			, fe.filename()));
+	std::string file_storage::file_name(internal_file_entry const& fe) const
+	{
+		return fe.filename();
 	}
 
 	size_type file_storage::file_size(internal_file_entry const& fe) const
@@ -545,6 +676,20 @@ namespace libtorrent
 		return fe.size;
 	}
 
+	bool file_storage::pad_file_at(internal_file_entry const& fe) const
+	{
+		return fe.pad_file;
+	}
+
+	size_type file_storage::file_offset(internal_file_entry const& fe) const
+	{
+		return fe.offset;
+	}
+
+	file_entry file_storage::at(file_storage::iterator i) const
+	{ return at(i - m_files.begin()); }
+#endif // TORRENT_NO_DEPRECATE
+
 	bool compare_file_entry_size(internal_file_entry const& fe1, internal_file_entry const& fe2)
 	{ return fe1.size < fe2.size; }
 
@@ -554,63 +699,38 @@ namespace libtorrent
 		TORRENT_ASSERT(dst < int(m_files.size()));
 		TORRENT_ASSERT(dst < index);
 
-		internal_file_entry e = m_files[index];
-		m_files.erase(m_files.begin() + index);
-		m_files.insert(m_files.begin() + dst, e);
+		std::iter_swap(m_files.begin() + index, m_files.begin() + dst);
 		if (!m_mtime.empty())
 		{
 			TORRENT_ASSERT(m_mtime.size() == m_files.size());
-			time_t mtime = 0;
-			if (int(m_mtime.size()) > index)
-			{
-				mtime = m_mtime[index];
-				m_mtime.erase(m_mtime.begin() + index);
-			}
-			if (dst > int(m_mtime.size())) m_mtime.resize(dst, 0);
-			m_mtime.insert(m_mtime.begin() + dst, mtime);
+			if (int(m_mtime.size()) < index) m_mtime.resize(index+1, 0);
+			std::iter_swap(m_mtime.begin() + dst, m_mtime.begin() + index);
 		}
 		if (!m_file_hashes.empty())
 		{
 			TORRENT_ASSERT(m_file_hashes.size() == m_files.size());
-			char const* fh = 0;
-			if (int(m_file_hashes.size()) > index)
-			{
-				fh = m_file_hashes[index];
-				m_file_hashes.erase(m_file_hashes.begin() + index);
-			}
-			if (int(m_file_hashes.size()) < dst) m_file_hashes.resize(dst, NULL);
-			m_file_hashes.insert(m_file_hashes.begin() + dst, fh);
+			if (int(m_file_hashes.size()) < index) m_file_hashes.resize(index + 1, NULL);
+			std::iter_swap(m_file_hashes.begin() + dst, m_file_hashes.begin() + index);
 		}
 		if (!m_file_base.empty())
 		{
 			TORRENT_ASSERT(m_file_base.size() == m_files.size());
-			size_type base = 0;
-			if (int(m_file_base.size()) > index)
-			{
-				base = m_file_base[index];
-				m_file_base.erase(m_file_base.begin() + index);
-			}
-			m_file_base.insert(m_file_base.begin() + dst, base);
+			if (int(m_file_base.size()) < index) m_file_base.resize(index + 1, 0);
+			std::iter_swap(m_file_base.begin() + dst, m_file_base.begin() + index);
 		}
 	}
 
-	void file_storage::optimize(int pad_file_limit)
+	void file_storage::optimize(int pad_file_limit, int alignment)
 	{
-		// the main purpuse of padding is to optimize disk
-		// I/O. This is a conservative memory page size assumption
-		int alignment = 16*1024;
-
-		// it doesn't make any sense to pad files that
-		// are smaller than the alignment
-		if (pad_file_limit >= 0 && pad_file_limit < alignment)
-			pad_file_limit = alignment;
+		if (alignment == -1)
+			alignment = m_piece_length;
 
 		size_type off = 0;
 		int padding_file = 0;
 		for (std::vector<internal_file_entry>::iterator i = m_files.begin();
 			i != m_files.end(); ++i)
 		{
-			if ((off & (alignment-1)) == 0)
+			if ((off % alignment) == 0)
 			{
 				// this file position is aligned, pick the largest
 				// available file to put here
@@ -620,46 +740,52 @@ namespace libtorrent
 
 				if (best_match != i)
 				{
-					int index = file_index(*best_match);
-					int cur_index = file_index(*i);
+					int index = best_match - m_files.begin();
+					int cur_index = i - m_files.begin();
 					reorder_file(index, cur_index);
 					i = m_files.begin() + cur_index;
 				}
 			}
 			else if (pad_file_limit >= 0
-				&& i->size > pad_file_limit
+				&& i->size > boost::uint32_t(pad_file_limit)
 				&& i->pad_file == false)
 			{
 				// if we have pad files enabled, and this file is
 				// not piece-aligned and the file size exceeds the
 				// limit, and it's not a padding file itself.
 				// so add a padding file in front of it
-				int pad_size = alignment - (off & (alignment-1));
+				int pad_size = alignment - (off % alignment);
 				
 				// find the largest file that fits in pad_size
 				std::vector<internal_file_entry>::iterator best_match = m_files.end();
-				for (std::vector<internal_file_entry>::iterator j = i+1; j < m_files.end(); ++j)
-				{
-					if (j->size > pad_size) continue;
-					if (best_match == m_files.end() || j->size > best_match->size)
-						best_match = j;
-				}
 
-				if (best_match != m_files.end())
+				// if pad_file_limit is 0, it means all files are padded, there's
+				// no point in trying to find smaller files to use as filling
+				if (pad_file_limit > 0)
 				{
-					// we found one
-					// We cannot have found i, because i->size > pad_file_limit
-					// which is forced to be no less than alignment. We only
-					// look for files <= pad_size, which never is greater than
-					// alignment
-					TORRENT_ASSERT(best_match != i);
-					int index = file_index(*best_match);
-					int cur_index = file_index(*i);
-					reorder_file(index, cur_index);
-					i = m_files.begin() + cur_index;
-					i->offset = off;
-					off += i->size;
-					continue;
+					for (std::vector<internal_file_entry>::iterator j = i+1; j < m_files.end(); ++j)
+					{
+						if (j->size > boost::uint32_t(pad_size)) continue;
+						if (best_match == m_files.end() || j->size > best_match->size)
+							best_match = j;
+					}
+
+					if (best_match != m_files.end())
+					{
+						// we found one
+						// We cannot have found i, because i->size > pad_file_limit
+						// which is forced to be no less than alignment. We only
+						// look for files <= pad_size, which never is greater than
+						// alignment
+						TORRENT_ASSERT(best_match != i);
+						int index = best_match - m_files.begin();
+						int cur_index = i - m_files.begin();
+						reorder_file(index, cur_index);
+						i = m_files.begin() + cur_index;
+						i->offset = off;
+						off += i->size;
+						continue;
+					}
 				}
 
 				// we could not find a file that fits in pad_size
@@ -667,32 +793,34 @@ namespace libtorrent
 				// note that i will be set to point to the
 				// new pad file. Once we're done adding it, we need
 				// to increment i to point to the current file again
-				internal_file_entry e;
-				int cur_index = file_index(*i);
-				i = m_files.insert(i, e);
-				i->size = pad_size;
-				i->offset = off;
+				// first add the pad file to the end of the file list
+				// then swap it in place. This minimizes the amount
+				// of copying of internal_file_entry, which is somewhat
+				// expensive (until we have move semantics)
+				int cur_index = i - m_files.begin();
+				int index = m_files.size();
+				m_files.push_back(internal_file_entry());
+				internal_file_entry& e = m_files.back();
+				// i may have been invalidated, refresh it
+				i = m_files.begin() + cur_index;
+				e.size = pad_size;
+				e.offset = off;
 				char name[30];
 				snprintf(name, sizeof(name), ".____padding_file/%d", padding_file);
 				std::string path = combine_path(m_name, name);
-				i->set_name(path.c_str());
-				i->pad_file = true;
+				e.set_name(path.c_str());
+				e.pad_file = true;
 				off += pad_size;
 				++padding_file;
 
-				if (int(m_mtime.size()) > cur_index)
-					m_mtime.insert(m_mtime.begin() + cur_index, 0);
+				if (!m_mtime.empty()) m_mtime.resize(index + 1, 0);
+				if (!m_file_hashes.empty()) m_file_hashes.resize(index + 1, NULL);
+				if (!m_file_base.empty()) m_file_base.resize(index + 1, 0);
 
-				if (int(m_file_hashes.size()) > cur_index)
-					m_file_hashes.insert(m_file_hashes.begin()
-						+ cur_index, (char const*)NULL);
+				reorder_file(index, cur_index);
 
-				if (int(m_file_base.size()) > cur_index)
-					m_file_base.insert(m_file_base.begin() + cur_index, 0);
-
-				// skip the pad file we just added and point
-				// at the current file again
-				++i;
+				TORRENT_ASSERT((off % alignment) == 0);
+				continue;
 			}
 			i->offset = off;
 			off += i->size;
