@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2009, Arvid Norberg
+Copyright (c) 2009-2014, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -30,50 +30,69 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
-#include "libtorrent/pch.hpp"
-
+#include "libtorrent/config.hpp"
 #include "libtorrent/i2p_stream.hpp"
 #include "libtorrent/assert.hpp"
 #include "libtorrent/error_code.hpp"
+#include "libtorrent/string_util.hpp"
+
+#if TORRENT_USE_I2P
 
 #include <boost/bind.hpp>
 
-#if TORRENT_USE_I2P
+#if defined TORRENT_ASIO_DEBUGGING
+#include "libtorrent/debug.hpp"
+#endif
 
 namespace libtorrent
 {
 
-	i2p_error_category i2p_category;
-
-	const char* i2p_error_category::name() const BOOST_SYSTEM_NOEXCEPT
+	struct i2p_error_category : boost::system::error_category
 	{
-		return "i2p error";
+		virtual const char* name() const BOOST_SYSTEM_NOEXCEPT
+		{ return "i2p error"; }
+		virtual std::string message(int ev) const BOOST_SYSTEM_NOEXCEPT
+		{
+			static char const* messages[] =
+			{
+				"no error",
+				"parse failed",
+				"cannot reach peer",
+				"i2p error",
+				"invalid key",
+				"invalid id",
+				"timeout",
+				"key not found",
+				"duplicated id"
+			};
+
+			if (ev < 0 || ev >= i2p_error::num_errors) return "unknown error";
+			return messages[ev];
+		}
+		virtual boost::system::error_condition default_error_condition(
+			int ev) const BOOST_SYSTEM_NOEXCEPT
+		{ return boost::system::error_condition(ev, *this); }
+	};
+
+
+	boost::system::error_category& get_i2p_category()
+	{
+		static i2p_error_category i2p_category;
+		return i2p_category;
 	}
 
-	std::string i2p_error_category::message(int ev) const BOOST_SYSTEM_NOEXCEPT
+	namespace i2p_error
 	{
-		static char const* messages[] =
+		boost::system::error_code make_error_code(i2p_error_code e)
 		{
-			"no error",
-			"parse failed",
-			"cannot reach peer",
-			"i2p error",
-			"invalid key",
-			"invalid id",
-			"timeout",
-			"key not found",
-			"duplicated id"
-		};
-
-		if (ev < 0 || ev >= i2p_error::num_errors) return "unknown error";
-		return messages[ev];
+			return error_code(e, get_i2p_category());
+		}
 	}
 
 	i2p_connection::i2p_connection(io_service& ios)
 		: m_state(sam_idle)
 		, m_io_service(ios)
-	{
-	}
+	{}
 
 	i2p_connection::~i2p_connection()
 	{}
@@ -108,12 +127,18 @@ namespace libtorrent
 		m_sam_socket->set_command(i2p_stream::cmd_create_session);
 		m_sam_socket->set_session_id(m_session_id.c_str());
 
+#if defined TORRENT_ASIO_DEBUGGING
+		add_outstanding_async("i2p_stream::on_sam_connect");
+#endif
 		m_sam_socket->async_connect(tcp::endpoint()
 			, boost::bind(&i2p_connection::on_sam_connect, this, _1, handler, m_sam_socket));
 	}
 
 	void i2p_connection::on_sam_connect(error_code const& ec, i2p_stream::handler_type const& h, boost::shared_ptr<i2p_stream>)
 	{
+#if defined TORRENT_ASIO_DEBUGGING
+		complete_async("i2p_stream::on_sam_connect");
+#endif
 		m_state = sam_idle;
 	
 		if (ec)
@@ -184,14 +209,14 @@ namespace libtorrent
 		, m_command(cmd_create_session)
 		, m_state(0)
 	{
-#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+#if TORRENT_USE_ASSERTS
 		m_magic = 0x1337;
 #endif
 	}
 
 	i2p_stream::~i2p_stream()
 	{
-#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+#if TORRENT_USE_ASSERTS
 		TORRENT_ASSERT(m_magic == 0x1337);
 		m_magic = 0;
 #endif
@@ -219,6 +244,9 @@ namespace libtorrent
 			return;
 		}
 
+#if defined TORRENT_ASIO_DEBUGGING
+		add_outstanding_async("i2p_stream::connected");
+#endif
 		m_sock.async_connect(i->endpoint(), boost::bind(
 			&i2p_stream::connected, this, _1, h));
 	}
@@ -226,11 +254,18 @@ namespace libtorrent
 	void i2p_stream::connected(error_code const& e, boost::shared_ptr<handler_type> h)
 	{
 		TORRENT_ASSERT(m_magic == 0x1337);
+#if defined TORRENT_ASIO_DEBUGGING
+		complete_async("i2p_stream::connected");
+#endif
 		if (handle_error(e, h)) return;
 
 		// send hello command
 		m_state = read_hello_response;
 		static const char cmd[] = "HELLO VERSION MIN=3.0 MAX=3.0\n";
+
+#if defined TORRENT_ASIO_DEBUGGING
+		add_outstanding_async("i2p_stream::start_read_line");
+#endif
 		async_write(m_sock, asio::buffer(cmd, sizeof(cmd) - 1)
 			, boost::bind(&i2p_stream::start_read_line, this, _1, h));
 //		fprintf(stderr, ">>> %s", cmd);
@@ -239,37 +274,25 @@ namespace libtorrent
 	void i2p_stream::start_read_line(error_code const& e, boost::shared_ptr<handler_type> h)
 	{
 		TORRENT_ASSERT(m_magic == 0x1337);
+#if defined TORRENT_ASIO_DEBUGGING
+		complete_async("i2p_stream::start_read_line");
+#endif
 		if (handle_error(e, h)) return;
 
+#if defined TORRENT_ASIO_DEBUGGING
+		add_outstanding_async("i2p_stream::read_line");
+#endif
 		m_buffer.resize(1);
 		async_read(m_sock, asio::buffer(m_buffer)
 			, boost::bind(&i2p_stream::read_line, this, _1, h));
 	}
 
-	char* string_tokenize(char* last, char sep, char** next)
-	{
-		if (last == 0) return 0;
-		if (last[0] == '"')
-		{
-			*next = strchr(last + 1, '"');
-			// consume the actual separator as well.
-			if (*next != NULL)
-				*next = strchr(*next, sep);
-		}
-		else
-		{
-			*next = strchr(last, sep);
-		}
-		if (*next == 0) return last;
-		**next = 0;
-		++(*next);
-		while (**next == sep && **next) ++(*next);
-		return last;
-	}
-
 	void i2p_stream::read_line(error_code const& e, boost::shared_ptr<handler_type> h)
 	{
 		TORRENT_ASSERT(m_magic == 0x1337);
+#if defined TORRENT_ASIO_DEBUGGING
+		complete_async("i2p_stream::read_line");
+#endif
 		if (handle_error(e, h)) return;
 
 		int read_pos = m_buffer.size();
@@ -277,6 +300,9 @@ namespace libtorrent
 		// look for \n which means end of the response
 		if (m_buffer[read_pos - 1] != '\n')
 		{
+#if defined TORRENT_ASIO_DEBUGGING
+			add_outstanding_async("i2p_stream::read_line");
+#endif
 			// read another byte from the socket
 			m_buffer.resize(read_pos + 1);
 			async_read(m_sock, asio::buffer(&m_buffer[read_pos], 1)
@@ -296,7 +322,7 @@ namespace libtorrent
 		}
 
 		error_code invalid_response(i2p_error::parse_failed
-			, i2p_category);
+			, get_i2p_category());
 
 		// null-terminate the string and parse it
 		m_buffer.push_back(0);
@@ -334,8 +360,8 @@ namespace libtorrent
 		if (ptr == 0 || expect2 == 0 || strcmp(expect2, ptr)) { handle_error(invalid_response, h); return; }
 
 		int result = 0;
-		char const* message = 0;
-		float version = 3.0f;
+//		char const* message = 0;
+//		float version = 3.0f;
 
 		for(;;)
 		{
@@ -369,11 +395,11 @@ namespace libtorrent
 			}
 			else if (strcmp("MESSAGE", name) == 0)
 			{
-				message = ptr;
+//				message = ptr;
 			}
 			else if (strcmp("VERSION", name) == 0)
 			{
-				version = float(atof(ptr));
+//				version = float(atof(ptr));
 			}
 			else if (strcmp("VALUE", name) == 0)
 			{
@@ -387,7 +413,7 @@ namespace libtorrent
 
 		if (result != i2p_error::no_error)
 		{
-			error_code ec(result, i2p_category);
+			error_code ec(result, get_i2p_category());
 			handle_error(ec, h);
 			return;
 		}
@@ -424,6 +450,9 @@ namespace libtorrent
 			// the destination of the remote peer
 			m_command = cmd_incoming;
 			m_buffer.resize(1);
+#if defined TORRENT_ASIO_DEBUGGING
+			add_outstanding_async("i2p_stream::read_line");
+#endif
 			async_read(m_sock, asio::buffer(m_buffer)
 				, boost::bind(&i2p_stream::read_line, this, _1, h));
 			break;
@@ -440,6 +469,9 @@ namespace libtorrent
 		int size = snprintf(cmd, sizeof(cmd), "STREAM CONNECT ID=%s DESTINATION=%s\n"
 			, m_id, m_dest.c_str());
 //		fprintf(stderr, ">>> %s", cmd);
+#if defined TORRENT_ASIO_DEBUGGING
+		add_outstanding_async("i2p_stream::start_read_line");
+#endif
 		async_write(m_sock, asio::buffer(cmd, size)
 			, boost::bind(&i2p_stream::start_read_line, this, _1, h));
 	}
@@ -451,6 +483,9 @@ namespace libtorrent
 		char cmd[400];
 		int size = snprintf(cmd, sizeof(cmd), "STREAM ACCEPT ID=%s\n", m_id);
 //		fprintf(stderr, ">>> %s", cmd);
+#if defined TORRENT_ASIO_DEBUGGING
+		add_outstanding_async("i2p_stream::start_read_line");
+#endif
 		async_write(m_sock, asio::buffer(cmd, size)
 			, boost::bind(&i2p_stream::start_read_line, this, _1, h));
 	}
@@ -463,6 +498,9 @@ namespace libtorrent
 		int size = snprintf(cmd, sizeof(cmd), "SESSION CREATE STYLE=STREAM ID=%s DESTINATION=TRANSIENT\n"
 			, m_id);
 //		fprintf(stderr, ">>> %s", cmd);
+#if defined TORRENT_ASIO_DEBUGGING
+		add_outstanding_async("i2p_stream::start_read_line");
+#endif
 		async_write(m_sock, asio::buffer(cmd, size)
 			, boost::bind(&i2p_stream::start_read_line, this, _1, h));
 	}
@@ -474,6 +512,9 @@ namespace libtorrent
 		char cmd[1024];
 		int size = snprintf(cmd, sizeof(cmd), "NAMING LOOKUP NAME=%s\n", m_name_lookup.c_str());
 //		fprintf(stderr, ">>> %s", cmd);
+#if defined TORRENT_ASIO_DEBUGGING
+		add_outstanding_async("i2p_stream::start_read_line");
+#endif
 		async_write(m_sock, asio::buffer(cmd, size)
 			, boost::bind(&i2p_stream::start_read_line, this, _1, h));
 	}
