@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2006, Arvid Norberg & Daniel Wallin
+Copyright (c) 2006-2014, Arvid Norberg & Daniel Wallin
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -30,8 +30,6 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
-#include "libtorrent/pch.hpp"
-
 #include <libtorrent/kademlia/find_data.hpp>
 #include <libtorrent/kademlia/routing_table.hpp>
 #include <libtorrent/kademlia/rpc_manager.hpp>
@@ -56,11 +54,6 @@ using detail::read_v6_endpoint;
 
 void find_data_observer::reply(msg const& m)
 {
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-	std::stringstream log_line;
-	log_line << "[" << m_algorithm.get() << "] incoming get_peer response [ ";
-#endif
-
 	lazy_entry const* r = m.message.dict_find_dict("r");
 	if (!r)
 	{
@@ -78,94 +71,14 @@ void find_data_observer::reply(msg const& m)
 #endif
 		return;
 	}
-
 	lazy_entry const* token = r->dict_find_string("token");
 	if (token)
 	{
 		static_cast<find_data*>(m_algorithm.get())->got_write_token(
 			node_id(id->string_ptr()), token->string_value());
-
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-		log_line << " token: " << to_hex(token->string_value());
-#endif
 	}
 
-	// look for peers
-	lazy_entry const* n = r->dict_find_list("values");
-	if (n)
-	{
-		std::vector<tcp::endpoint> peer_list;
-		if (n->list_size() == 1 && n->list_at(0)->type() == lazy_entry::string_t)
-		{
-			// assume it's mainline format
-			char const* peers = n->list_at(0)->string_ptr();
-			char const* end = peers + n->list_at(0)->string_length();
-
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-			log_line << " p: " << ((end - peers) / 6);
-#endif
-			while (end - peers >= 6)
-				peer_list.push_back(read_v4_endpoint<tcp::endpoint>(peers));
-		}
-		else
-		{
-			// assume it's uTorrent/libtorrent format
-			read_endpoint_list<tcp::endpoint>(n, peer_list);
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-			log_line << " p: " << n->list_size();
-#endif
-		}
-		static_cast<find_data*>(m_algorithm.get())->got_peers(peer_list);
-	}
-
-	// look for nodes
-	n = r->dict_find_string("nodes");
-	if (n)
-	{
-		std::vector<node_entry> node_list;
-		char const* nodes = n->string_ptr();
-		char const* end = nodes + n->string_length();
-
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-		log_line << " nodes: " << ((end - nodes) / 26);
-#endif
-		while (end - nodes >= 26)
-		{
-			node_id id;
-			std::copy(nodes, nodes + 20, id.begin());
-			nodes += 20;
-			m_algorithm->traverse(id, read_v4_endpoint<udp::endpoint>(nodes));
-		}
-	}
-
-	n = r->dict_find_list("nodes2");
-	if (n)
-	{
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-		log_line << " nodes2: " << n->list_size();
-#endif
-		for (int i = 0; i < n->list_size(); ++i)
-		{
-			lazy_entry const* p = n->list_at(0);
-			if (p->type() != lazy_entry::string_t) continue;
-			if (p->string_length() < 6 + 20) continue;
-			char const* in = p->string_ptr();
-
-			node_id id;
-			std::copy(in, in + 20, id.begin());
-			in += 20;
-			if (p->string_length() == 6 + 20)
-				m_algorithm->traverse(id, read_v4_endpoint<udp::endpoint>(in));
-#if TORRENT_USE_IPV6
-			else if (p->string_length() == 18 + 20)
-				m_algorithm->traverse(id, read_v6_endpoint<udp::endpoint>(in));
-#endif
-		}
-	}
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-	log_line << " ]";
-	TORRENT_LOG(traversal) << log_line.str();
-#endif
+	traversal_observer::reply(m);
 	done();
 }
 
@@ -178,18 +91,21 @@ void add_entry_fun(void* userdata, node_entry const& e)
 find_data::find_data(
 	node_impl& node
 	, node_id target
-	, data_callback const& dcallback
-	, nodes_callback const& ncallback
-	, bool noseeds)
+	, nodes_callback const& ncallback)
 	: traversal_algorithm(node, target)
-	, m_data_callback(dcallback)
 	, m_nodes_callback(ncallback)
-	, m_target(target)
 	, m_done(false)
-	, m_got_peers(false)
-	, m_noseeds(noseeds)
 {
-	node.m_table.for_each_node(&add_entry_fun, 0, (traversal_algorithm*)this);
+}
+
+void find_data::start()
+{
+	// if the user didn't add seed-nodes manually, grab a bunch of nodes from the
+	// routing table
+	if (m_results.empty())
+		m_node.m_table.for_each_node(&add_entry_fun, 0, (traversal_algorithm*)this);
+
+	traversal_algorithm::start();
 }
 
 void find_data::got_write_token(node_id const& n, std::string const& write_token)
@@ -211,28 +127,7 @@ observer_ptr find_data::new_observer(void* ptr
 	return o;
 }
 
-bool find_data::invoke(observer_ptr o)
-{
-	if (m_done)
-	{
-		m_invoke_count = -1;
-		return false;
-	}
-
-	entry e;
-	e["y"] = "q";
-	e["q"] = "get_peers";
-	entry& a = e["a"];
-	a["info_hash"] = m_target.to_string();
-	if (m_noseeds) a["noseed"] = 1;
-	return m_node.m_rpc.invoke(e, o->target_ep(), o);
-}
-
-void find_data::got_peers(std::vector<tcp::endpoint> const& peers)
-{
-	if (!peers.empty()) m_got_peers = true;
-	m_data_callback(peers);
-}
+char const* find_data::name() const { return "find_data"; }
 
 void find_data::done()
 {
@@ -241,7 +136,7 @@ void find_data::done()
 	m_done = true;
 
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
-	TORRENT_LOG(traversal) << time_now_string() << "[" << this << "] " << name() << " DONE";
+	TORRENT_LOG(traversal) << "[" << this << "] " << name() << " DONE";
 #endif
 
 	std::vector<std::pair<node_entry, std::string> > results;
@@ -250,7 +145,6 @@ void find_data::done()
 		, end(m_results.end()); i != end && num_results > 0; ++i)
 	{
 		observer_ptr const& o = *i;
- 		if (o->flags & observer::flag_no_id) continue;
 		if ((o->flags & observer::flag_alive) == 0)
 		{
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
@@ -275,7 +169,8 @@ void find_data::done()
 #endif
 		--num_results;
 	}
-	m_nodes_callback(results, m_got_peers);
+
+	if (m_nodes_callback) m_nodes_callback(results);
 
 	traversal_algorithm::done();
 }
