@@ -280,7 +280,7 @@ void routing_table::print_state(std::ostream& os) const
 			id_shift = bucket_index;
 		else
 			id_shift = bucket_index + 1;
-		
+
 		for (bucket_t::const_iterator j = i->live_nodes.begin()
 			, end(i->live_nodes.end()); j != end; ++j)
 		{
@@ -389,7 +389,7 @@ bool compare_ip_cidr(node_entry const& lhs, node_entry const& rhs)
 }
 
 node_entry* routing_table::find_node(udp::endpoint const& ep
-	, routing_table::table_t::iterator* bucket) 
+	, routing_table::table_t::iterator* bucket)
 {
 	for (table_t::iterator i = m_buckets.begin()
 		, end(m_buckets.end()); i != end; ++i)
@@ -416,7 +416,7 @@ node_entry* routing_table::find_node(udp::endpoint const& ep
 }
 
 void routing_table::remove_node(node_entry* n
-	, routing_table::table_t::iterator bucket) 
+	, routing_table::table_t::iterator bucket)
 {
 	INVARIANT_CHECK;
 
@@ -535,7 +535,7 @@ routing_table::add_node_status_t routing_table::add_node_impl(node_entry e)
 			remove_node(existing, existing_bucket);
 		}
 	}
-	
+
 	table_t::iterator i = find_bucket(e.id);
 	bucket_t& b = i->live_nodes;
 	bucket_t& rb = i->replacements;
@@ -636,8 +636,10 @@ routing_table::add_node_status_t routing_table::add_node_impl(node_entry e)
 	// can we split the bucket?
 	// only nodes that haven't failed can split the bucket, and we can only
 	// split the last bucket
-	bool can_split = (boost::next(i) == m_buckets.end() && m_buckets.size() < 159)
-		&& e.fail_count() == 0;
+	const bool can_split = (boost::next(i) == m_buckets.end()
+		&& m_buckets.size() < 159)
+		&& e.fail_count() == 0
+		&& (i == m_buckets.begin() || boost::prior(i)->live_nodes.size() > 1);
 
 	if (e.pinged() && e.fail_count() == 0)
 	{
@@ -678,7 +680,7 @@ routing_table::add_node_status_t routing_table::add_node_impl(node_entry e)
 //			TORRENT_LOG(table) << "replacing stale node: " << e.id << " " << e.addr();
 			return node_added;
 		}
-		
+
 		// in order to provide as few lookups as possible before finding
 		// the data someone is looking for, make sure there is an affinity
 		// towards having a good spread of node IDs in each bucket
@@ -968,7 +970,7 @@ void routing_table::node_failed(node_id const& nid, udp::endpoint const& ep)
 	// claiming the same ID. The node we have in our routing
 	// table is not necessarily stale
 	if (j->ep() != ep) return;
-	
+
 	if (rb.empty())
 	{
 		j->timed_out();
@@ -1088,7 +1090,7 @@ void routing_table::find_node(node_id const& target
 	{
 		--j;
 		bucket_t& b = j->live_nodes;
-	
+
 		if (options & include_failed)
 		{
 			std::copy(b.begin(), b.end(), std::back_inserter(l));
