@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2007, Arvid Norberg
+Copyright (c) 2007-2014, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -29,8 +29,10 @@ ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 
 */
-
-#include "libtorrent/pch.hpp"
+#include "libtorrent/config.hpp"
+#if defined TORRENT_OS2
+#include <pthread.h>
+#endif
 
 #include <boost/version.hpp>
 #include <boost/bind.hpp>
@@ -236,7 +238,7 @@ int natpmp::add_mapping(protocol_type p, int external_port, int local_port)
 	for (std::vector<mapping_t>::iterator m = m_mappings.begin()
 		, end(m_mappings.end()); m != end; ++m)
 	{
-	std::cout << " ADD MAPPING: " << mapping_index << " [ "
+		std::cout << " ADD MAPPING: " << mapping_index << " [ "
 			"proto: " << (i->protocol == none ? "none" : i->protocol == tcp ? "tcp" : "udp")
 			<< " port: " << i->external_port
 			<< " local-port: " << i->local_port
@@ -257,7 +259,7 @@ void natpmp::try_next_mapping(int i, mutex::scoped_lock& l)
 	for (std::vector<mapping_t>::iterator m = m_mappings.begin()
 		, end(m_mappings.end()); m != end; ++m)
 	{
-	std::cout << "     " << (m - m_mappings.begin()) << " [ "
+		std::cout << "     " << (m - m_mappings.begin()) << " [ "
 			"proto: " << (m->protocol == none ? "none" : m->protocol == tcp ? "tcp" : "udp")
 			<< " port: " << m->external_port
 			<< " local-port: " << m->local_port
