@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2007, Arvid Norberg
+Copyright (c) 2007-2014, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -30,8 +30,6 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
-#include "libtorrent/pch.hpp"
-
 #include "libtorrent/socks5_stream.hpp"
 #include "libtorrent/assert.hpp"
 #include "libtorrent/socket_io.hpp"
@@ -39,38 +37,54 @@ POSSIBILITY OF SUCH DAMAGE.
 namespace libtorrent
 {
 
-	socks_error_category socks_category;
-
-#if BOOST_VERSION >= 103500
-	const char* socks_error_category::name() const BOOST_SYSTEM_NOEXCEPT
+	namespace socks_error
 	{
-		return "socks error";
-	}
-
-	std::string socks_error_category::message(int ev) const BOOST_SYSTEM_NOEXCEPT
-	{
-		static char const* messages[] =
+		boost::system::error_code make_error_code(socks_error_code e)
 		{
-			"SOCKS no error",
-			"SOCKS unsupported version",
-			"SOCKS unsupported authentication method",
-			"SOCKS unsupported authentication version",
-			"SOCKS authentication error",
-			"SOCKS username required",
-			"SOCKS general failure",
-			"SOCKS command not supported",
-			"SOCKS no identd running",
-			"SOCKS identd could not identify username"
-		};
+			return error_code(e, get_socks_category());
+		}
+	}
 
-		if (ev < 0 || ev >= socks_error::num_errors) return "unknown error";
-		return messages[ev];
+	struct socks_error_category : boost::system::error_category
+	{
+		virtual const char* name() const BOOST_SYSTEM_NOEXCEPT
+		{ return "socks error"; }
+		virtual std::string message(int ev) const BOOST_SYSTEM_NOEXCEPT
+		{
+			static char const* messages[] =
+			{
+				"SOCKS no error",
+				"SOCKS unsupported version",
+				"SOCKS unsupported authentication method",
+				"SOCKS unsupported authentication version",
+				"SOCKS authentication error",
+				"SOCKS username required",
+				"SOCKS general failure",
+				"SOCKS command not supported",
+				"SOCKS no identd running",
+				"SOCKS identd could not identify username"
+			};
+
+			if (ev < 0 || ev >= socks_error::num_errors) return "unknown error";
+			return messages[ev];
+		}
+		virtual boost::system::error_condition default_error_condition(
+			int ev) const BOOST_SYSTEM_NOEXCEPT
+		{ return boost::system::error_condition(ev, *this); }
+	};
+
+	TORRENT_EXPORT boost::system::error_category& get_socks_category()
+	{
+		static socks_error_category socks_category;
+		return socks_category;
 	}
-#endif
 
 	void socks5_stream::name_lookup(error_code const& e, tcp::resolver::iterator i
 		, boost::shared_ptr<handler_type> h)
 	{
+#if defined TORRENT_ASIO_DEBUGGING
+		complete_async("socks5_stream::name_lookup");
+#endif
 		if (e || i == tcp::resolver::iterator())
 		{
 			(*h)(e);
@@ -93,12 +107,18 @@ namespace libtorrent
 
 		// TOOD: we could bind the socket here, since we know what the
 		// target endpoint is of the proxy
+#if defined TORRENT_ASIO_DEBUGGING
+		add_outstanding_async("socks5_stream::connected");
+#endif
 		m_sock.async_connect(i->endpoint(), boost::bind(
 			&socks5_stream::connected, this, _1, h));
 	}
 
 	void socks5_stream::connected(error_code const& e, boost::shared_ptr<handler_type> h)
 	{
+#if defined TORRENT_ASIO_DEBUGGING
+		complete_async("socks5_stream::connected");
+#endif
 		if (e)
 		{
 			(*h)(e);
@@ -125,6 +145,9 @@ namespace libtorrent
 				write_uint8(0, p); // no authentication
 				write_uint8(2, p); // username/password
 			}
+#if defined TORRENT_ASIO_DEBUGGING
+			add_outstanding_async("socks5_stream::handshake1");
+#endif
 			async_write(m_sock, asio::buffer(m_buffer)
 				, boost::bind(&socks5_stream::handshake1, this, _1, h));
 		}
@@ -134,7 +157,7 @@ namespace libtorrent
 		}
 		else
 		{
-			(*h)(error_code(socks_error::unsupported_version, socks_category));
+			(*h)(socks_error::unsupported_version);
 			error_code ec;
 			close(ec);
 		}
@@ -142,6 +165,9 @@ namespace libtorrent
 
 	void socks5_stream::handshake1(error_code const& e, boost::shared_ptr<handler_type> h)
 	{
+#if defined TORRENT_ASIO_DEBUGGING
+		complete_async("socks5_stream::handshake1");
+#endif
 		if (e)
 		{
 			(*h)(e);
@@ -150,6 +176,9 @@ namespace libtorrent
 			return;
 		}
 
+#if defined TORRENT_ASIO_DEBUGGING
+		add_outstanding_async("socks5_stream::handshake2");
+#endif
 		m_buffer.resize(2);
 		async_read(m_sock, asio::buffer(m_buffer)
 			, boost::bind(&socks5_stream::handshake2, this, _1, h));
@@ -157,6 +186,9 @@ namespace libtorrent
 
 	void socks5_stream::handshake2(error_code const& e, boost::shared_ptr<handler_type> h)
 	{
+#if defined TORRENT_ASIO_DEBUGGING
+		complete_async("socks5_stream::handshake2");
+#endif
 		if (e)
 		{
 			(*h)(e);
@@ -173,7 +205,7 @@ namespace libtorrent
 
 		if (version < m_version)
 		{
-			(*h)(error_code(socks_error::unsupported_version, socks_category));
+			(*h)(socks_error::unsupported_version);
 			error_code ec;
 			close(ec);
 			return;
@@ -187,7 +219,7 @@ namespace libtorrent
 		{
 			if (m_user.empty())
 			{
-				(*h)(error_code(socks_error::username_required, socks_category));
+				(*h)(socks_error::username_required);
 				error_code ec;
 				close(ec);
 				return;
@@ -201,12 +233,16 @@ namespace libtorrent
 			write_string(m_user, p);
 			write_uint8(m_password.size(), p);
 			write_string(m_password, p);
+
+#if defined TORRENT_ASIO_DEBUGGING
+			add_outstanding_async("socks5_stream::handshake3");
+#endif
 			async_write(m_sock, asio::buffer(m_buffer)
 				, boost::bind(&socks5_stream::handshake3, this, _1, h));
 		}
 		else
 		{
-			(*h)(error_code(socks_error::unsupported_authentication_method, socks_category));
+			(*h)(socks_error::unsupported_authentication_method);
 			error_code ec;
 			close(ec);
 			return;
@@ -216,6 +252,9 @@ namespace libtorrent
 	void socks5_stream::handshake3(error_code const& e
 		, boost::shared_ptr<handler_type> h)
 	{
+#if defined TORRENT_ASIO_DEBUGGING
+		complete_async("socks5_stream::handshake3");
+#endif
 		if (e)
 		{
 			(*h)(e);
@@ -224,6 +263,9 @@ namespace libtorrent
 			return;
 		}
 
+#if defined TORRENT_ASIO_DEBUGGING
+		add_outstanding_async("socks5_stream::handshake4");
+#endif
 		m_buffer.resize(2);
 		async_read(m_sock, asio::buffer(m_buffer)
 			, boost::bind(&socks5_stream::handshake4, this, _1, h));
@@ -232,6 +274,9 @@ namespace libtorrent
 	void socks5_stream::handshake4(error_code const& e
 		, boost::shared_ptr<handler_type> h)
 	{
+#if defined TORRENT_ASIO_DEBUGGING
+		complete_async("socks5_stream::handshake4");
+#endif
 		if (e)
 		{
 			(*h)(e);
@@ -248,7 +293,7 @@ namespace libtorrent
 
 		if (version != 1)
 		{
-			(*h)(error_code(socks_error::unsupported_authentication_version, socks_category));
+			(*h)(socks_error::unsupported_authentication_version);
 			error_code ec;
 			close(ec);
 			return;
@@ -256,7 +301,7 @@ namespace libtorrent
 
 		if (status != 0)
 		{
-			(*h)(error_code(socks_error::authentication_error, socks_category));
+			(*h)(socks_error::authentication_error);
 			error_code ec;
 			close(ec);
 			return;
@@ -300,7 +345,7 @@ namespace libtorrent
 			// SOCKS4 only supports IPv4
 			if (!m_remote_endpoint.address().is_v4())
 			{
-				(*h)(error_code(boost::asio::error::address_family_not_supported));
+				(*h)(boost::asio::error::address_family_not_supported);
 				error_code ec;
 				close(ec);
 				return;
@@ -317,18 +362,24 @@ namespace libtorrent
 		}
 		else
 		{
-			(*h)(error_code(socks_error::unsupported_version, socks_category));
+			(*h)(socks_error::unsupported_version);
 			error_code ec;
 			close(ec);
 			return;
 		}
 
+#if defined TORRENT_ASIO_DEBUGGING
+		add_outstanding_async("socks5_stream::connect1");
+#endif
 		async_write(m_sock, asio::buffer(m_buffer)
 			, boost::bind(&socks5_stream::connect1, this, _1, h));
 	}
 
 	void socks5_stream::connect1(error_code const& e, boost::shared_ptr<handler_type> h)
 	{
+#if defined TORRENT_ASIO_DEBUGGING
+		complete_async("socks5_stream::connect1");
+#endif
 		if (e)
 		{
 			(*h)(e);
@@ -341,12 +392,19 @@ namespace libtorrent
 			m_buffer.resize(6 + 4); // assume an IPv4 address
 		else if (m_version == 4)
 			m_buffer.resize(8);
+
+#if defined TORRENT_ASIO_DEBUGGING
+		add_outstanding_async("socks5_stream::connect2");
+#endif
 		async_read(m_sock, asio::buffer(m_buffer)
 			, boost::bind(&socks5_stream::connect2, this, _1, h));
 	}
 
 	void socks5_stream::connect2(error_code const& e, boost::shared_ptr<handler_type> h)
 	{
+#if defined TORRENT_ASIO_DEBUGGING
+		complete_async("socks5_stream::connect2");
+#endif
 		if (e)
 		{
 			(*h)(e);
@@ -366,14 +424,14 @@ namespace libtorrent
 		{
 			if (version < m_version)
 			{
-				(*h)(error_code(socks_error::unsupported_version, socks_category));
+				(*h)(socks_error::unsupported_version);
 				error_code ec;
 				close(ec);
 				return;
 			}
 			if (response != 0)
 			{
-				error_code ec(socks_error::general_failure, socks_category);
+				error_code ec(socks_error::general_failure, get_socks_category());
 				switch (response)
 				{
 					case 2: ec = asio::error::no_permission; break;
@@ -381,7 +439,7 @@ namespace libtorrent
 					case 4: ec = asio::error::host_unreachable; break;
 					case 5: ec = asio::error::connection_refused; break;
 					case 6: ec = asio::error::timed_out; break;
-					case 7: ec = error_code(socks_error::command_not_supported, socks_category); break;
+					case 7: ec = socks_error::command_not_supported; break;
 					case 8: ec = asio::error::address_family_not_supported; break;
 				}
 				(*h)(ec);
@@ -397,6 +455,9 @@ namespace libtorrent
 				{
 					if (m_listen == 0)
 					{
+#if defined TORRENT_ASIO_DEBUGGING
+						add_outstanding_async("socks5_stream::connect1");
+#endif
 						m_listen = 1;
 						connect1(e, h);
 						return;
@@ -431,6 +492,9 @@ namespace libtorrent
 			}
 			m_buffer.resize(m_buffer.size() + extra_bytes);
 
+#if defined TORRENT_ASIO_DEBUGGING
+			add_outstanding_async("socks5_stream::connect3");
+#endif
 			TORRENT_ASSERT(extra_bytes > 0);
 			async_read(m_sock, asio::buffer(&m_buffer[m_buffer.size() - extra_bytes], extra_bytes)
 				, boost::bind(&socks5_stream::connect3, this, _1, h));
@@ -439,7 +503,7 @@ namespace libtorrent
 		{
 			if (version != 0)
 			{
-				(*h)(error_code(socks_error::general_failure, socks_category));
+				(*h)(socks_error::general_failure);
 				error_code ec;
 				close(ec);
 				return;
@@ -452,6 +516,9 @@ namespace libtorrent
 				{
 					if (m_listen == 0)
 					{
+#if defined TORRENT_ASIO_DEBUGGING
+						add_outstanding_async("socks5_stream::connect1");
+#endif
 						m_listen = 1;
 						connect1(e, h);
 						return;
@@ -476,7 +543,7 @@ namespace libtorrent
 				case 92: code = socks_error::no_identd; break;
 				case 93: code = socks_error::identd_error; break;
 			}
-			error_code ec(code, socks_category);
+			error_code ec(code, get_socks_category());
 			(*h)(ec);
 			close(ec);
 		}
@@ -484,6 +551,9 @@ namespace libtorrent
 
 	void socks5_stream::connect3(error_code const& e, boost::shared_ptr<handler_type> h)
 	{
+#if defined TORRENT_ASIO_DEBUGGING
+		complete_async("socks5_stream::connect3");
+#endif
 		using namespace libtorrent::detail;
 
 		if (e)
@@ -498,6 +568,9 @@ namespace libtorrent
 		{
 			if (m_listen == 0)
 			{
+#if defined TORRENT_ASIO_DEBUGGING
+				add_outstanding_async("socks5_stream::connect1");
+#endif
 				m_listen = 1;
 				connect1(e, h);
 				return;
