@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003, Arvid Norberg, Daniel Wallin
+Copyright (c) 2003-2014, Arvid Norberg, Daniel Wallin
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -30,8 +30,6 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
-#include "libtorrent/pch.hpp"
-
 #include <ctime>
 #include <algorithm>
 #include <set>
@@ -71,12 +69,6 @@ POSSIBILITY OF SUCH DAMAGE.
 
 //#define TORRENT_PARTIAL_HASH_LOG
 
-#if TORRENT_USE_IOSTREAM
-#include <ios>
-#include <iostream>
-#include <iomanip>
-#endif
-
 #if defined(__APPLE__)
 // for getattrlist()
 #include <sys/attr.h>
@@ -99,20 +91,6 @@ POSSIBILITY OF SUCH DAMAGE.
 // for convert_to_wstring and convert_to_native
 #include "libtorrent/escape_string.hpp"
 
-#if defined TORRENT_DEBUG && defined TORRENT_STORAGE_DEBUG && TORRENT_USE_IOSTREAM
-namespace
-{
-	using namespace libtorrent;
-
-	void print_to_log(std::string const& s)
-	{
-		static std::ofstream log("log.txt");
-		log << s;
-		log.flush();
-	}
-}
-#endif
-
 namespace libtorrent
 {
 	std::vector<std::pair<size_type, std::time_t> > get_filesizes(
@@ -120,17 +98,16 @@ namespace libtorrent
 	{
 		std::string save_path = complete(p);
 		std::vector<std::pair<size_type, std::time_t> > sizes;
-		for (file_storage::iterator i = storage.begin()
-			, end(storage.end()); i != end; ++i)
+		for (int i = 0; i < storage.num_files(); ++i)
 		{
 			size_type size = 0;
 			std::time_t time = 0;
 
-			if (!i->pad_file)
+			if (!storage.pad_file_at(i))
 			{
 				file_status s;
 				error_code ec;
-				stat_file(combine_path(save_path, storage.file_path(*i)), &s, ec);
+				stat_file(storage.file_path(i, save_path), &s, ec);
 
 				if (!ec)
 				{
@@ -171,16 +148,15 @@ namespace libtorrent
 
 		std::vector<std::pair<size_type, std::time_t> >::const_iterator size_iter
 			= sizes.begin();
-		for (file_storage::iterator i = fs.begin()
-			, end(fs.end());i != end; ++i, ++size_iter)
+		for (int i = 0; i < fs.num_files(); ++i, ++size_iter)
 		{
 			size_type size = 0;
 			std::time_t time = 0;
-			if (i->pad_file) continue;
+			if (fs.pad_file_at(i)) continue;
 
 			file_status s;
 			error_code ec;
-			stat_file(combine_path(p, fs.file_path(*i)), &s, ec);
+			stat_file(fs.file_path(i, p), &s, ec);
 
 			if (!ec)
 			{
@@ -299,7 +275,7 @@ namespace libtorrent
 			std::memset(i->iov_base, 0, i->iov_len);
 	}
 
-#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+#if TORRENT_USE_ASSERTS
 	int count_bufs(file::iovec_t const* bufs, int bytes)
 	{
 		int size = 0;
@@ -413,33 +389,49 @@ namespace libtorrent
 	{
 		if (mapped) m_mapped_files.reset(new file_storage(*mapped));
 
-		TORRENT_ASSERT(m_files.begin() != m_files.end());
+		TORRENT_ASSERT(m_files.num_files() > 0);
 		m_save_path = complete(path);
 	}
 
 	default_storage::~default_storage() { m_pool.release(this); }
 
+	void default_storage::set_file_priority(std::vector<boost::uint8_t> const& prio)
+	{
+		m_file_priority = prio;
+	}
+
 	bool default_storage::initialize(bool allocate_files)
 	{
 		m_allocate_files = allocate_files;
+
+#ifdef TORRENT_WINDOWS
+		// don't do full file allocations on network drives
+#if TORRENT_USE_WSTRING
+		std::wstring f = convert_to_wstring(m_save_path);
+		int drive_type = GetDriveTypeW(f.c_str());
+#else
+		int drive_type = GetDriveTypeA(m_save_path.c_str());
+#endif
+
+		if (drive_type == DRIVE_REMOTE)
+			m_allocate_files = false;
+#endif
+
 		error_code ec;
 		m_file_created.resize(files().num_files(), false);
 
 		// first, create all missing directories
 		std::string last_path;
-		for (file_storage::iterator file_iter = files().begin(),
-			end_iter = files().end(); file_iter != end_iter; ++file_iter)
+		for (int file_index = 0; file_index < files().num_files(); ++file_index)
 		{
-			int file_index = files().file_index(*file_iter);
-
 			// ignore files that have priority 0
 			if (int(m_file_priority.size()) > file_index
 				&& m_file_priority[file_index] == 0) continue;
 
 			// ignore pad files
-			if (file_iter->pad_file) continue;
+			if (files().pad_file_at(file_index)) continue;
 
-			std::string file_path = combine_path(m_save_path, files().file_path(*file_iter));
+			std::string file_path = files().file_path(file_index, m_save_path);
 
 			file_status s;
 			stat_file(file_path, &s, ec);
@@ -453,7 +445,7 @@ namespace libtorrent
 			// if the file already exists, but is larger than what
 			// it's supposed to be, truncate it
 			// if the file is empty, just create it either way.
-			if ((!ec && s.file_size > file_iter->size) || file_iter->size == 0)
+			if ((!ec && s.file_size > files().file_size(file_index)) || files().file_size(file_index) == 0)
 			{
 				std::string dir = parent_path(file_path);
 
@@ -470,11 +462,11 @@ namespace libtorrent
 				}
 				ec.clear();
 
-				boost::intrusive_ptr<file> f = open_file(file_iter, file::read_write | file::random_access, ec);
+				boost::intrusive_ptr<file> f = open_file(file_index, file::read_write | file::random_access, ec);
 				if (ec) set_error(file_path, ec);
 				else if (f)
 				{
-					f->set_size(file_iter->size, ec);
+					f->set_size(files().file_size(file_index), ec);
 					if (ec) set_error(file_path, ec);
 				}
 				if (ec) break;
@@ -494,16 +486,13 @@ namespace libtorrent
 
 	bool default_storage::has_any_file()
 	{
-		file_storage::iterator i = files().begin();
-		file_storage::iterator end = files().end();
-
-		for (; i != end; ++i)
+		for (int i = 0; i < files().num_files(); ++i)
 		{
 			error_code ec;
 			file_status s;
-			stat_file(combine_path(m_save_path, files().file_path(*i)), &s, ec);
+			stat_file(files().file_path(i, m_save_path), &s, ec);
 			if (ec) continue;
-			if (s.mode & file_status::regular_file && i->size > 0)
+			if (s.mode & file_status::regular_file && files().file_size(i) > 0)
 				return true;
 		}
 		return false;
@@ -512,11 +501,13 @@ namespace libtorrent
 	bool default_storage::rename_file(int index, std::string const& new_filename)
 	{
 		if (index < 0 || index >= files().num_files()) return true;
-		std::string old_name = combine_path(m_save_path, files().file_path(files().at(index)));
+		std::string old_name = files().file_path(index, m_save_path);
 		m_pool.release(this, index);
 
 		error_code ec;
-		std::string new_path = combine_path(m_save_path, new_filename);
+		std::string new_path;
+		if (is_complete(new_filename)) new_path = new_filename;
+		else new_path = combine_path(m_save_path, new_filename);
 		std::string new_dir = parent_path(new_path);
 
 		// create any missing directories that the new filename
@@ -571,18 +562,21 @@ namespace libtorrent
 		// delete the files from disk
 		std::set<std::string> directories;
 		typedef std::set<std::string>::iterator iter_t;
-		for (file_storage::iterator i = files().begin()
-			, end(files().end()); i != end; ++i)
+		for (int i = 0; i < files().num_files(); ++i)
 		{
-			std::string fp = files().file_path(*i);
-			std::string p = combine_path(m_save_path, fp);
-			std::string bp = parent_path(fp);
-			std::pair<iter_t, bool> ret;
-			ret.second = true;
-			while (ret.second && !bp.empty())
+			std::string fp = files().file_path(i);
+			bool complete = is_complete(fp);
+			std::string p = complete ? fp : combine_path(m_save_path, fp);
+			if (!complete)
 			{
-				ret = directories.insert(combine_path(m_save_path, bp));
-				bp = parent_path(bp);
+				std::string bp = parent_path(fp);
+				std::pair<iter_t, bool> ret;
+				ret.second = true;
+				while (ret.second && !bp.empty())
+				{
+					ret = directories.insert(combine_path(m_save_path, bp));
+					bp = parent_path(bp);
+				}
 			}
 			delete_one_file(p);
 		}
@@ -625,21 +619,21 @@ namespace libtorrent
 		TORRENT_ASSERT(slot >= 0);
 		TORRENT_ASSERT(slot < files().num_pieces());
 
-		size_type file_offset = (size_type)slot * m_files.piece_length();
-		file_storage::iterator file_iter;
+		size_type file_offset = (size_type)slot * files().piece_length();
+		int file_index = 0;
 
-		for (file_iter = files().begin();;)
+		for (;;)
 		{
-			if (file_offset < file_iter->size)
+			if (file_offset < files().file_size(file_index))
 				break;
 
-			file_offset -= file_iter->size;
-			++file_iter;
-			TORRENT_ASSERT(file_iter != files().end());
+			file_offset -= files().file_size(file_index);
+			++file_index;
+			TORRENT_ASSERT(file_index != files().num_files());
 		}
 	
 		error_code ec;
-		boost::intrusive_ptr<file> file_handle = open_file(file_iter, file::read_only, ec);
+		boost::intrusive_ptr<file> file_handle = open_file(file_index, file::read_only, ec);
 		if (!file_handle || ec) return slot;
 
 		size_type data_start = file_handle->sparse_end(file_offset);
@@ -750,10 +744,9 @@ namespace libtorrent
 				fs = file_sizes.begin();
 			// the resume data says we have the entire torrent
 			// make sure the file sizes are the right ones
-			for (file_storage::iterator i = files().begin()
-				, end(files().end()); i != end; ++i, ++fs)
+			for (int i = 0; i < files().num_files(); ++i, ++fs)
 			{
-				if (!i->pad_file && i->size != fs->first)
+				if (!files().pad_file_at(i) && files().file_size(i) != fs->first)
 				{
 					error = errors::mismatching_file_size;
 					return false;
@@ -768,31 +761,64 @@ namespace libtorrent
 	}
 
 	// returns true on success
-	bool default_storage::move_storage(std::string const& sp)
+	int default_storage::move_storage(std::string const& sp, int flags)
 	{
+		int ret = piece_manager::no_error;
 		std::string save_path = complete(sp);
 
+		// check to see if any of the files exist
 		error_code ec;
-		file_status s;
-		stat_file(save_path, &s, ec);
-		if (ec == boost::system::errc::no_such_file_or_directory)
-			create_directories(save_path, ec);
-		else if (ec)
-			return false;
-
-		m_pool.release(this);
-
-		bool ret = true;
-		std::set<std::string> to_move;
 		file_storage const& f = files();
 
-		for (file_storage::iterator i = f.begin()
-			, end(f.end()); i != end; ++i)
+		file_status s;
+		if (flags == fail_if_exist)
 		{
-			std::string split = split_path(f.file_path(*i));
+			stat_file(combine_path(save_path, f.name()), &s, ec);
+			if (ec != boost::system::errc::no_such_file_or_directory)
+			{
+				// the directory exists, check all the files
+				for (int i = 0; i < f.num_files(); ++i)
+				{
+					// files moved out to absolute paths are ignored
+					if (is_complete(f.file_path(i))) continue;
+
+					std::string new_path = f.file_path(i, save_path);
+					stat_file(new_path, &s, ec);
+					if (ec != boost::system::errc::no_such_file_or_directory)
+						return piece_manager::file_exist;
+				}
+			}
+		}
+
+		// collect all directories in to_move. This is because we
+		// try to move entire directories by default (instead of
+		// files independently).
+		std::set<std::string> to_move;
+		for (int i = 0; i < f.num_files(); ++i)
+		{
+			// files moved out to absolute paths are not moved
+			if (is_complete(f.file_path(i))) continue;
+
+			std::string split = split_path(f.file_path(i));
 			to_move.insert(to_move.begin(), split);
 		}
 
+		ec.clear();
+		stat_file(save_path, &s, ec);
+		if (ec == boost::system::errc::no_such_file_or_directory)
+		{
+			ec.clear();
+			create_directories(save_path, ec);
+		}
+
+		if (ec)
+		{
+			set_error(save_path, ec);
+			return piece_manager::fatal_disk_error;
+		}
+
+		m_pool.release(this);
+
 		for (std::set<std::string>::const_iterator i = to_move.begin()
 			, end(to_move.end()); i != end; ++i)
 		{
@@ -800,32 +826,42 @@ namespace libtorrent
 			std::string new_path = combine_path(save_path, *i);
 
 			rename(old_path, new_path, ec);
-			if (ec && ec != boost::system::errc::no_such_file_or_directory)
+			if (ec)
 			{
-				error_code ec;
-				recursive_copy(old_path, new_path, ec);
-				if (ec == boost::system::errc::no_such_file_or_directory)
+				if (flags == dont_replace && ec == boost::system::errc::file_exists)
 				{
-					// it's a bit weird that rename() would not return
-					// ENOENT, but the file still wouldn't exist. But,
-					// in case it does, we're done.
-					ec.clear();
+					if (ret == piece_manager::no_error) ret = piece_manager::need_full_check;
+					continue;
+				}
+
+				if (ec != boost::system::errc::no_such_file_or_directory)
+				{
+					error_code ec;
+					recursive_copy(old_path, new_path, ec);
+					if (ec == boost::system::errc::no_such_file_or_directory)
+					{
+						// it's a bit weird that rename() would not return
+						// ENOENT, but the file still wouldn't exist. But,
+						// in case it does, we're done.
+						ec.clear();
+						break;
+					}
+					if (ec)
+					{
+						set_error(old_path, ec);
+						ret = piece_manager::fatal_disk_error;
+					}
+					else
+					{
+						remove_all(old_path, ec);
+					}
 					break;
 				}
-				else if (ec)
-				{
-					set_error(old_path, ec);
-					ret = false;
-				}
-				else
-				{
-					remove_all(old_path, ec);
-				}
-				break;
 			}
 		}
 
-		if (ret) m_save_path = save_path;
+		if (ret == piece_manager::no_error || ret == piece_manager::need_full_check)
+			m_save_path = save_path;
 
 		return ret;
 	}
@@ -983,24 +1019,25 @@ ret:
 		// find the file and file
 		size_type tor_off = size_type(slot)
 			* files().piece_length() + offset;
-		file_storage::iterator file_iter = files().file_at_offset(tor_off);
-		while (file_iter->pad_file)
+		int file_index = files().file_index_at_offset(tor_off);
+		while (files().pad_file_at(file_index))
 		{
-			++file_iter;
-			if (file_iter == files().end())
+			++file_index;
+			if (file_index == files().num_files())
 				return size_type(slot) * files().piece_length() + offset;
 			// update offset as well, since we're moving it up ahead
-			tor_off = file_iter->offset;
-		}
-		TORRENT_ASSERT(!file_iter->pad_file);
+			tor_off = files().file_offset(file_index);
 
-		size_type file_offset = tor_off - file_iter->offset;
+		}
+		TORRENT_ASSERT(!files().pad_file_at(file_index));
+
+		size_type file_offset = tor_off - files().file_offset(file_index);
 		TORRENT_ASSERT(file_offset >= 0);
 
 		// open the file read only to avoid re-opening
 		// it in case it's already opened in read-only mode
 		error_code ec;
-		boost::intrusive_ptr<file> f = open_file(file_iter, file::read_only | file::random_access, ec);
+		boost::intrusive_ptr<file> f = open_file(file_index, file::read_only | file::random_access, ec);
 
 		size_type ret = 0;
 		if (f && !ec) ret = f->phys_offset(file_offset);
@@ -1019,19 +1056,10 @@ ret:
 		size_type start = slot * (size_type)m_files.piece_length() + offset;
 		TORRENT_ASSERT(start + size <= m_files.total_size());
 
-		size_type file_offset = start;
-		file_storage::iterator file_iter;
-
-		// TODO: use binary search!
-		for (file_iter = files().begin();;)
-		{
-			if (file_offset < file_iter->size)
-				break;
-
-			file_offset -= file_iter->size;
-			++file_iter;
-			TORRENT_ASSERT(file_iter != files().end());
-		}
+		int file_index = files().file_index_at_offset(start);
+		TORRENT_ASSERT(start >= files().file_offset(file_index));
+		TORRENT_ASSERT(start < files().file_offset(file_index) + files().file_size(file_index));
+		size_type file_offset = start - files().file_offset(file_index);
 
 		boost::intrusive_ptr<file> file_handle;
 		int bytes_left = size;
@@ -1043,20 +1071,20 @@ ret:
 		TORRENT_ASSERT(bytes_left >= 0);
 
 		int file_bytes_left;
-		for (;bytes_left > 0; ++file_iter, bytes_left -= file_bytes_left)
+		for (;bytes_left > 0; ++file_index, bytes_left -= file_bytes_left)
 		{
-			TORRENT_ASSERT(file_iter != files().end());
+			TORRENT_ASSERT(file_index < files().num_files());
 
 			file_bytes_left = bytes_left;
-			if (file_offset + file_bytes_left > file_iter->size)
-				file_bytes_left = (std::max)(static_cast<int>(file_iter->size - file_offset), 0);
+			if (file_offset + file_bytes_left > files().file_size(file_index))
+				file_bytes_left = (std::max)(static_cast<int>(files().file_size(file_index) - file_offset), 0);
 
 			if (file_bytes_left == 0) continue;
 
-			if (file_iter->pad_file) continue;
+			if (files().pad_file_at(file_index)) continue;
 
 			error_code ec;
-			file_handle = open_file(file_iter, file::read_only | file::random_access, ec);
+			file_handle = open_file(file_index, file::read_only | file::random_access, ec);
 
 			// failing to hint that we want to read is not a big deal
 			// just swollow the error and keep going
@@ -1115,7 +1143,7 @@ ret:
 		int size = bufs_size(bufs, num_bufs);
 		TORRENT_ASSERT(size > 0);
 
-#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+#if TORRENT_USE_ASSERTS
 		std::vector<file_slice> slices
 			= files().map_block(slot, offset, size);
 		TORRENT_ASSERT(!slices.empty());
@@ -1125,19 +1153,10 @@ ret:
 		TORRENT_ASSERT(start + size <= m_files.total_size());
 
 		// find the file iterator and file offset
-		size_type file_offset = start;
-		file_storage::iterator file_iter;
-
-		// TODO: use binary search!
-		for (file_iter = files().begin();;)
-		{
-			if (file_offset < file_iter->size)
-				break;
-
-			file_offset -= file_iter->size;
-			++file_iter;
-			TORRENT_ASSERT(file_iter != files().end());
-		}
+		int file_index = files().file_index_at_offset(start);
+		TORRENT_ASSERT(start >= files().file_offset(file_index));
+		TORRENT_ASSERT(start < files().file_offset(file_index) + files().file_size(file_index));
+		size_type file_offset = start - files().file_offset(file_index);
 
 		int buf_pos = 0;
 		error_code ec;
@@ -1151,7 +1170,7 @@ ret:
 
 		TORRENT_ASSERT(bytes_left >= 0);
 
-#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+#if TORRENT_USE_ASSERTS
 		int counter = 0;
 #endif
 
@@ -1160,28 +1179,27 @@ ret:
 		copy_bufs(bufs, size, current_buf);
 		TORRENT_ASSERT(count_bufs(current_buf, size) == num_bufs);
 		int file_bytes_left;
-		for (;bytes_left > 0; ++file_iter, bytes_left -= file_bytes_left
+		for (;bytes_left > 0; ++file_index, bytes_left -= file_bytes_left
 			, buf_pos += file_bytes_left)
 		{
-			TORRENT_ASSERT(file_iter != files().end());
+			TORRENT_ASSERT(file_index < files().num_files());
 			TORRENT_ASSERT(buf_pos >= 0);
 
 			file_bytes_left = bytes_left;
-			if (file_offset + file_bytes_left > file_iter->size)
-				file_bytes_left = (std::max)(static_cast<int>(file_iter->size - file_offset), 0);
+			if (file_offset + file_bytes_left > files().file_size(file_index))
+				file_bytes_left = (std::max)(static_cast<int>(files().file_size(file_index) - file_offset), 0);
 
 			if (file_bytes_left == 0) continue;
 
-#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+#if TORRENT_USE_ASSERTS
 			TORRENT_ASSERT(int(slices.size()) > counter);
 			size_type slice_size = slices[counter].size;
 			TORRENT_ASSERT(slice_size == file_bytes_left);
-			TORRENT_ASSERT((files().begin() + slices[counter].file_index)
-				== file_iter);
+			TORRENT_ASSERT(slices[counter].file_index == file_index);
 			++counter;
 #endif
 
-			if (file_iter->pad_file)
+			if (files().pad_file_at(file_index))
 			{
 				if ((op.mode & file::rw_mask) == file::read_only)
 				{
@@ -1197,42 +1215,40 @@ ret:
 			}
 
 			error_code ec;
-			file_handle = open_file(file_iter, op.mode, ec);
+			file_handle = open_file(file_index, op.mode, ec);
 			if (((op.mode & file::rw_mask) != file::read_only)
 				&& ec == boost::system::errc::no_such_file_or_directory)
 			{
 				// this means the directory the file is in doesn't exist.
 				// so create it
 				ec.clear();
-				std::string path = combine_path(m_save_path, files().file_path(*file_iter));
+				std::string path = files().file_path(file_index, m_save_path);
 				create_directories(parent_path(path), ec);
 				// if the directory creation failed, don't try to open the file again
 				// but actually just fail
-				if (!ec) file_handle = open_file(file_iter, op.mode, ec);
+				if (!ec) file_handle = open_file(file_index, op.mode, ec);
 			}
 
 			if (!file_handle || ec)
 			{
-				std::string path = combine_path(m_save_path, files().file_path(*file_iter));
+				std::string path = files().file_path(file_index, m_save_path);
 				TORRENT_ASSERT(ec);
 				set_error(path, ec);
 				return -1;
 			}
 
 			// if the file has priority 0, don't allocate it
-			int file_index = files().file_index(*file_iter);
 			if (m_allocate_files && (op.mode & file::rw_mask) != file::read_only
-				&& (m_file_priority.size() <= file_index || m_file_priority[file_index] > 0))
+				&& (int(m_file_priority.size()) <= file_index || m_file_priority[file_index] > 0))
 			{
-				TORRENT_ASSERT(m_file_created.size() == files().num_files());
+				TORRENT_ASSERT(int(m_file_created.size()) == files().num_files());
 				if (m_file_created[file_index] == false)
 				{
 					file_handle->set_size(files().file_size(file_index), ec);
 					m_file_created.set_bit(file_index);
 					if (ec)
 					{
-						set_error(combine_path(m_save_path
-							, files().file_path(*file_iter)), ec);
+						set_error(files().file_path(file_index, m_save_path), ec);
 						return -1;
 					}
 				}
@@ -1246,7 +1262,7 @@ ret:
 			// read is unaligned, we need to fall back on a slow
 			// special read that reads aligned buffers and copies
 			// it into the one supplied
-			size_type adjusted_offset = files().file_base(*file_iter) + file_offset;
+			size_type adjusted_offset = files().file_base(file_index) + file_offset;
 			if ((file_handle->open_mode() & file::no_buffer)
 				&& ((adjusted_offset & (file_handle->pos_alignment()-1)) != 0
 				|| (uintptr_t(tmp_bufs->iov_base) & (file_handle->buf_alignment()-1)) != 0))
@@ -1254,16 +1270,17 @@ ret:
 				bytes_transferred = (int)(this->*op.unaligned_op)(file_handle, adjusted_offset
 					, tmp_bufs, num_tmp_bufs, ec);
 				if ((op.mode & file::rw_mask) != file::read_only
-					&& adjusted_offset + bytes_transferred >= file_iter->size
+					&& adjusted_offset + bytes_transferred >= files().file_size(file_index)
 					&& (file_handle->pos_alignment() > 0 || file_handle->size_alignment() > 0))
 				{
 					// we were writing, and we just wrote the last block of the file
 					// we likely wrote a bit too much, since we're restricted to
 					// a specific alignment for writes. Make sure to truncate the size
 
-					// TODO: what if file_base is used to merge several virtual files
-					// into a single physical file?
-					file_handle->set_size(file_iter->size, ec);
+					// TODO: 0 what if file_base is used to merge several virtual files
+					// into a single physical file? We should probably disable this
+					// if file_base is used. This is not a widely used feature though
+					file_handle->set_size(files().file_size(file_index), ec);
 				}
 			}
 			else
@@ -1276,7 +1293,7 @@ ret:
 
 			if (ec)
 			{
-				set_error(combine_path(m_save_path, files().file_path(*file_iter)), ec);
+				set_error(files().file_path(file_index, m_save_path), ec);
 				return -1;
 			}
 
@@ -1409,26 +1426,25 @@ ret:
 		return readv(&b, slot, offset, 1);
 	}
 
-	boost::intrusive_ptr<file> default_storage::open_file(file_storage::iterator fe, int mode
+	boost::intrusive_ptr<file> default_storage::open_file(int file_index, int mode
 		, error_code& ec) const
 	{
 		int cache_setting = m_settings ? settings().disk_io_write_mode : 0;
 		if (cache_setting == session_settings::disable_os_cache
 			|| (cache_setting == session_settings::disable_os_cache_for_aligned_files
-			&& ((fe->offset + files().file_base(*fe)) & (m_page_size-1)) == 0))
+			&& ((files().file_offset(file_index) + files().file_base(file_index)) & (m_page_size-1)) == 0))
 			mode |= file::no_buffer;
 		bool lock_files = m_settings ? settings().lock_files : false;
 		if (lock_files) mode |= file::lock_file;
 		if (!m_allocate_files) mode |= file::sparse;
 
 		// files with priority 0 should always be sparse
-		int file_index = fe - files().begin();
-		if (m_file_priority.size() > file_index && m_file_priority[file_index] == 0)
+		if (int(m_file_priority.size()) > file_index && m_file_priority[file_index] == 0)
 			mode |= file::sparse;
 
 		if (m_settings && settings().no_atime_storage) mode |= file::no_atime;
 
-		return m_pool.open_file(const_cast<default_storage*>(this), m_save_path, fe, files(), mode, ec);
+		return m_pool.open_file(const_cast<default_storage*>(this), m_save_path, file_index, files(), mode, ec);
 	}
 
 	storage_interface* default_storage_constructor(file_storage const& fs
@@ -1524,6 +1540,19 @@ ret:
 	{
 	}
 
+	void piece_manager::async_set_file_priority(
+		std::vector<boost::uint8_t> const& prios
+		, boost::function<void(int, disk_io_job const&)> const& handler)
+	{
+		std::vector<boost::uint8_t>* p = new std::vector<boost::uint8_t>(prios);
+
+		disk_io_job j;
+		j.storage = this;
+		j.buffer = (char*)p;
+		j.action = disk_io_job::file_priority;
+		m_io_thread.add_job(j, handler);
+	}
+
 	void piece_manager::async_save_resume_data(
 		boost::function<void(int, disk_io_job const&)> const& handler)
 	{
@@ -1565,13 +1594,14 @@ ret:
 		m_io_thread.add_job(j, handler);
 	}
 
-	void piece_manager::async_move_storage(std::string const& p
+	void piece_manager::async_move_storage(std::string const& p, int flags
 		, boost::function<void(int, disk_io_job const&)> const& handler)
 	{
 		disk_io_job j;
 		j.storage = this;
 		j.action = disk_io_job::move_storage;
 		j.str = p;
+		j.piece = flags;
 		m_io_thread.add_job(j, handler);
 	}
 
@@ -1733,14 +1763,15 @@ ret:
 		return ph.h.final();
 	}
 
-	int piece_manager::move_storage_impl(std::string const& save_path)
+	int piece_manager::move_storage_impl(std::string const& save_path, int flags)
 	{
-		if (m_storage->move_storage(save_path))
+		int ret = m_storage->move_storage(save_path, flags);
+
+		if (ret == no_error || ret == need_full_check)
 		{
 			m_save_path = complete(save_path);
-			return 0;
 		}
-		return -1;
+		return ret;
 	}
 
 	void piece_manager::write_resume_data(entry& rd) const
@@ -1838,10 +1869,6 @@ ret:
 
 		if (m_storage->settings().disable_hash_checks) return ret;
 
-#if defined TORRENT_PARTIAL_HASH_LOG && TORRENT_USE_IOSTREAM
-		std::ofstream out("partial_hash.log", std::ios::app);
-#endif
-
 		if (offset == 0)
 		{
 			partial_hash& ph = m_piece_hasher[piece_index];
@@ -1851,15 +1878,6 @@ ret:
 			for (file::iovec_t* i = iov, *end(iov + num_bufs); i < end; ++i)
 				ph.h.update((char const*)i->iov_base, i->iov_len);
 
-#if defined TORRENT_PARTIAL_HASH_LOG && TORRENT_USE_IOSTREAM
-			out << time_now_string() << " NEW ["
-				" s: " << this
-				<< " p: " << piece_index
-				<< " off: " << offset
-				<< " size: " << size
-				<< " entries: " << m_piece_hasher.size()
-				<< " ]" << std::endl;
-#endif
 		}
 		else
 		{
@@ -2513,10 +2531,9 @@ ret:
 	{
 		size_type file_offset = 0;
 		size_type current_offset = size_type(m_current_slot) * m_files.piece_length();
-		for (file_storage::iterator i = m_files.begin()
-			, end(m_files.end()); i != end; ++i)
+		for (int i = 0; i < m_files.num_files(); ++i)
 		{
-			file_offset += i->size;
+			file_offset += m_files.file_size(i);
 			if (file_offset > current_offset) break;
 		}
 
@@ -2892,24 +2909,6 @@ ret:
 		if (slot_index != piece_index
 			&& m_slot_to_piece[piece_index] >= 0)
 		{
-
-#if defined TORRENT_DEBUG && defined TORRENT_STORAGE_DEBUG && TORRENT_USE_IOSTREAM
-			std::stringstream s;
-
-			s << "there is another piece at our slot, swapping..";
-
-			s << "\n   piece_index: ";
-			s << piece_index;
-			s << "\n   slot_index: ";
-			s << slot_index;
-			s << "\n   piece at our slot: ";
-			s << m_slot_to_piece[piece_index];
-			s << "\n";
-
-			print_to_log(s.str());
-			debug_log();
-#endif
-
 			int piece_at_our_slot = m_slot_to_piece[piece_index];
 			TORRENT_ASSERT(m_piece_to_slot[piece_at_our_slot] == piece_index);
 
@@ -3000,7 +2999,7 @@ ret:
 		return m_slot_to_piece[slot];
 	}
 		
-#ifdef TORRENT_DEBUG
+#if TORRENT_USE_INVARIANT_CHECKS
 	void piece_manager::check_invariant() const
 	{
 		TORRENT_ASSERT(m_current_slot <= m_files.num_pieces());
@@ -3135,24 +3134,6 @@ ret:
 		}
 	}
 
-#if defined(TORRENT_STORAGE_DEBUG) && TORRENT_USE_IOSTREAM
-	void piece_manager::debug_log() const
-	{
-		std::stringstream s;
-
-		s << "index\tslot\tpiece\n";
-
-		for (int i = 0; i < m_files.num_pieces(); ++i)
-		{
-			s << i << "\t" << m_slot_to_piece[i] << "\t";
-			s << m_piece_to_slot[i] << "\n";
-		}
-
-		s << "---------------------------------\n";
-
-		print_to_log(s.str());
-	}
-#endif
 #endif
 } // namespace libtorrent
 
