@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003, Arvid Norberg
+Copyright (c) 2003-2014, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -30,8 +30,6 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
-#include "libtorrent/pch.hpp"
-
 #include <ctime>
 #include <iterator>
 #include <algorithm>
@@ -77,21 +75,24 @@ namespace libtorrent
 {
 
 	torrent_status::torrent_status()
-		: state(checking_resume_data)
-		, paused(false)
-		, auto_managed(false)
-		, sequential_download(false)
-		, is_seeding(false)
-		, is_finished(false)
-		, has_metadata(false)
-		, progress(0.f)
-		, progress_ppm(0)
-		, total_download(0)
+		: total_download(0)
 		, total_upload(0)
 		, total_payload_download(0)
 		, total_payload_upload(0)
 		, total_failed_bytes(0)
 		, total_redundant_bytes(0)
+		, total_done(0)
+		, total_wanted_done(0)
+		, total_wanted(0)
+		, all_time_upload(0)
+		, all_time_download(0)
+		, added_time(0)
+		, completed_time(0)
+		, last_seen_complete(0)
+		, storage_mode(storage_mode_sparse)
+		, progress(0.f)
+		, progress_ppm(0)
+		, queue_position(0)
 		, download_rate(0)
 		, upload_rate(0)
 		, download_payload_rate(0)
@@ -104,9 +105,6 @@ namespace libtorrent
 		, list_peers(0)
 		, connect_candidates(0)
 		, num_pieces(0)
-		, total_done(0)
-		, total_wanted_done(0)
-		, total_wanted(0)
 		, distributed_full_copies(0)
 		, distributed_fraction(0)
 		, distributed_copies(0.f)
@@ -115,48 +113,48 @@ namespace libtorrent
 		, num_connections(0)
 		, uploads_limit(0)
 		, connections_limit(0)
-		, storage_mode(storage_mode_sparse)
 		, up_bandwidth_queue(0)
 		, down_bandwidth_queue(0)
-		, all_time_upload(0)
-		, all_time_download(0)
+		, time_since_upload(0)
+		, time_since_download(0)
 		, active_time(0)
 		, finished_time(0)
 		, seeding_time(0)
 		, seed_rank(0)
 		, last_scrape(0)
-		, has_incoming(false)
 		, sparse_regions(0)
-		, seed_mode(false)
+		, priority(0)
+		, state(checking_resume_data)
+		, need_save_resume(false)
+		, ip_filter_applies(true)
 		, upload_mode(false)
 		, share_mode(false)
 		, super_seeding(false)
-		, priority(0)
-		, added_time(0)
-		, completed_time(0)
-		, last_seen_complete(0)
-		, time_since_upload(0)
-		, time_since_download(0)
-		, queue_position(0)
-		, need_save_resume(false)
-		, ip_filter_applies(true)
+		, paused(false)
+		, auto_managed(false)
+		, sequential_download(false)
+		, is_seeding(false)
+		, is_finished(false)
+		, has_metadata(false)
+		, has_incoming(false)
+		, seed_mode(false)
+		, moving_storage(false)
 		, info_hash(0)
-		, listen_port(0)
 	{}
 
 	torrent_status::~torrent_status() {}
 
 	template <class R>
-	void fun_ret(R* ret, bool* done, condition* e, mutex* m, boost::function<R(void)> f)
+	void fun_ret(R* ret, bool* done, condition_variable* e, mutex* m, boost::function<R(void)> f)
 	{
 		*ret = f();
 		mutex::scoped_lock l(*m);
 		*done = true;
-		e->signal_all(l);
+		e->notify_all();
 	}
 
 	// defined in session.cpp
-	void fun_wrap(bool* done, condition* e, mutex* m, boost::function<void(void)> f);
+	void fun_wrap(bool* done, condition_variable* e, mutex* m, boost::function<void(void)> f);
 
 #define TORRENT_ASYNC_CALL(x) \
 	boost::shared_ptr<torrent> t = m_torrent.lock(); \
@@ -195,7 +193,7 @@ namespace libtorrent
 	session_impl& ses = t->session(); \
 	mutex::scoped_lock l(ses.mut); \
 	ses.m_io_service.dispatch(boost::bind(&fun_wrap, &done, &ses.cond, &ses.mut, boost::function<void(void)>(boost::bind(&torrent:: x, t)))); \
-	do { ses.cond.wait(l); } while(!done)
+	while (!done) { ses.cond.wait(l); }
 
 #define TORRENT_SYNC_CALL1(x, a1) \
 	boost::shared_ptr<torrent> t = m_torrent.lock(); \
@@ -205,7 +203,7 @@ namespace libtorrent
 	mutex::scoped_lock l(ses.mut); \
 	ses.m_io_service.dispatch(boost::bind(&fun_wrap, &done, &ses.cond, &ses.mut, boost::function<void(void)>(boost::bind(&torrent:: x, t, a1)))); \
 	t.reset(); \
-	do { ses.cond.wait(l); } while(!done); }
+	while (!done) { ses.cond.wait(l); }; }
 
 #define TORRENT_SYNC_CALL2(x, a1, a2) \
 	boost::shared_ptr<torrent> t = m_torrent.lock(); \
@@ -215,7 +213,7 @@ namespace libtorrent
 	mutex::scoped_lock l(ses.mut); \
 	ses.m_io_service.dispatch(boost::bind(&fun_wrap, &done, &ses.cond, &ses.mut, boost::function<void(void)>(boost::bind(&torrent:: x, t, a1, a2)))); \
 	t.reset(); \
-	do { ses.cond.wait(l); } while(!done); }
+	while (!done) { ses.cond.wait(l); }; }
 
 #define TORRENT_SYNC_CALL3(x, a1, a2, a3) \
 	boost::shared_ptr<torrent> t = m_torrent.lock(); \
@@ -225,7 +223,7 @@ namespace libtorrent
 	mutex::scoped_lock l(ses.mut); \
 	ses.m_io_service.dispatch(boost::bind(&fun_wrap, &done, &ses.cond, &ses.mut, boost::function<void(void)>(boost::bind(&torrent:: x, t, a1, a2, a3)))); \
 	t.reset(); \
-	do { ses.cond.wait(l); } while(!done); }
+	while (!done) { ses.cond.wait(l); }; }
 
 #define TORRENT_SYNC_CALL_RET(type, def, x) \
 	boost::shared_ptr<torrent> t = m_torrent.lock(); \
@@ -236,7 +234,7 @@ namespace libtorrent
 	mutex::scoped_lock l(ses.mut); \
 	ses.m_io_service.dispatch(boost::bind(&fun_ret<type >, &r, &done, &ses.cond, &ses.mut, boost::function<type(void)>(boost::bind(&torrent:: x, t)))); \
 	t.reset(); \
-	do { ses.cond.wait(l); } while(!done)
+	while (!done) { ses.cond.wait(l); }
 
 #define TORRENT_SYNC_CALL_RET1(type, def, x, a1) \
 	boost::shared_ptr<torrent> t = m_torrent.lock(); \
@@ -247,7 +245,7 @@ namespace libtorrent
 	mutex::scoped_lock l(ses.mut); \
 	ses.m_io_service.dispatch(boost::bind(&fun_ret<type >, &r, &done, &ses.cond, &ses.mut, boost::function<type(void)>(boost::bind(&torrent:: x, t, a1)))); \
 	t.reset(); \
-	do { ses.cond.wait(l); } while(!done)
+	while (!done) { ses.cond.wait(l); }
 
 #define TORRENT_SYNC_CALL_RET2(type, def, x, a1, a2) \
 	boost::shared_ptr<torrent> t = m_torrent.lock(); \
@@ -258,7 +256,7 @@ namespace libtorrent
 	mutex::scoped_lock l(ses.mut); \
 	ses.m_io_service.dispatch(boost::bind(&fun_ret<type >, &r, &done, &ses.cond, &ses.mut, boost::function<type(void)>(boost::bind(&torrent:: x, t, a1, a2)))); \
 	t.reset(); \
-	do { ses.cond.wait(l); } while(!done)
+	while (!done) { ses.cond.wait(l); }
 
 #ifndef BOOST_NO_EXCEPTIONS
 	void throw_invalid_handle()
@@ -267,104 +265,80 @@ namespace libtorrent
 	}
 #endif
 
-#ifdef TORRENT_DEBUG
-
-	void torrent_handle::check_invariant() const
-	{}
-
-#endif
-
 	sha1_hash torrent_handle::info_hash() const
 	{
-		INVARIANT_CHECK;
+		boost::shared_ptr<torrent> t = m_torrent.lock();
 		const static sha1_hash empty;
-		TORRENT_SYNC_CALL_RET(sha1_hash, empty, info_hash);
-		return r;
+		if (!t) return empty;
+		return t->info_hash();
 	}
 
 	int torrent_handle::max_uploads() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL_RET(int, 0, max_uploads);
 		return r;
 	}
 
 	void torrent_handle::set_max_uploads(int max_uploads) const
 	{
-		INVARIANT_CHECK;
-		TORRENT_ASSERT(max_uploads >= 2 || max_uploads == -1);
-		TORRENT_ASYNC_CALL1(set_max_uploads, max_uploads);
-	}
-
-	void torrent_handle::use_interface(const char* net_interface) const
-	{
-		INVARIANT_CHECK;
-		TORRENT_ASYNC_CALL1(use_interface, std::string(net_interface));
+		TORRENT_ASSERT_PRECOND(max_uploads >= 2 || max_uploads == -1);
+		TORRENT_ASYNC_CALL2(set_max_uploads, max_uploads, true);
 	}
 
 	int torrent_handle::max_connections() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL_RET(int, 0, max_connections);
 		return r;
 	}
 
 	void torrent_handle::set_max_connections(int max_connections) const
 	{
-		INVARIANT_CHECK;
-		TORRENT_ASSERT(max_connections >= 2 || max_connections == -1);
-		TORRENT_ASYNC_CALL1(set_max_connections, max_connections);
+		TORRENT_ASSERT_PRECOND(max_connections >= 2 || max_connections == -1);
+		TORRENT_ASYNC_CALL2(set_max_connections, max_connections, true);
 	}
 
 	void torrent_handle::set_upload_limit(int limit) const
 	{
-		INVARIANT_CHECK;
-		TORRENT_ASSERT(limit >= -1);
-		TORRENT_ASYNC_CALL1(set_upload_limit, limit);
+		TORRENT_ASSERT_PRECOND(limit >= -1);
+		TORRENT_ASYNC_CALL2(set_upload_limit, limit, true);
 	}
 
 	int torrent_handle::upload_limit() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL_RET(int, 0, upload_limit);
 		return r;
 	}
 
 	void torrent_handle::set_download_limit(int limit) const
 	{
-		INVARIANT_CHECK;
-		TORRENT_ASSERT(limit >= -1);
-		TORRENT_ASYNC_CALL1(set_download_limit, limit);
+		TORRENT_ASSERT_PRECOND(limit >= -1);
+		TORRENT_ASYNC_CALL2(set_download_limit, limit, true);
 	}
 
 	int torrent_handle::download_limit() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL_RET(int, 0, download_limit);
 		return r;
 	}
 
 	void torrent_handle::move_storage(
-		std::string const& save_path) const
+		std::string const& save_path, int flags) const
 	{
-		INVARIANT_CHECK;
-		TORRENT_ASYNC_CALL1(move_storage, save_path);
+		TORRENT_ASYNC_CALL2(move_storage, save_path, flags);
 	}
 
 #if TORRENT_USE_WSTRING
 #ifndef TORRENT_NO_DEPRECATE
 	void torrent_handle::move_storage(
-		std::wstring const& save_path) const
+		std::wstring const& save_path, int flags) const
 	{
-		INVARIANT_CHECK;
 		std::string utf8;
 		wchar_utf8(save_path, utf8);
-		TORRENT_ASYNC_CALL1(move_storage, utf8);
+		TORRENT_ASYNC_CALL2(move_storage, utf8, flags);
 	}
 
 	void torrent_handle::rename_file(int index, std::wstring const& new_name) const
 	{
-		INVARIANT_CHECK;
 		std::string utf8;
 		wchar_utf8(new_name, utf8);
 		TORRENT_ASYNC_CALL2(rename_file, index, utf8);
@@ -374,54 +348,46 @@ namespace libtorrent
 
 	void torrent_handle::rename_file(int index, std::string const& new_name) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL2(rename_file, index, new_name);
 	}
 
-#ifndef TORRENT_DISABLE_EXTENSIONS
 	void torrent_handle::add_extension(
 		boost::function<boost::shared_ptr<torrent_plugin>(torrent*, void*)> const& ext
 		, void* userdata)
 	{
-		INVARIANT_CHECK;
+#ifndef TORRENT_DISABLE_EXTENSIONS
 		TORRENT_ASYNC_CALL2(add_extension, ext, userdata);
-	}
 #endif
+	}
 
 	bool torrent_handle::set_metadata(char const* metadata, int size) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL_RET2(bool, false, set_metadata, metadata, size);
 		return r;
 	}
 
 	void torrent_handle::pause(int flags) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL1(pause, bool(flags & graceful_pause));
 	}
 
 	void torrent_handle::apply_ip_filter(bool b) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL1(set_apply_ip_filter, b);
 	}
 
 	void torrent_handle::set_share_mode(bool b) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL1(set_share_mode, b);
 	}
 
 	void torrent_handle::set_upload_mode(bool b) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL1(set_upload_mode, b);
 	}
 
 	void torrent_handle::flush_cache() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL(flush_cache);
 	}
 
@@ -436,106 +402,91 @@ namespace libtorrent
 #endif
 	}
 
+	void torrent_handle::set_ssl_certificate_buffer(
+		std::string const& certificate
+		, std::string const& private_key
+		, std::string const& dh_params)
+	{
+#ifdef TORRENT_USE_OPENSSL
+		TORRENT_ASYNC_CALL3(set_ssl_cert_buffer, certificate, private_key, dh_params);
+#endif
+	}
+
 	void torrent_handle::save_resume_data(int f) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL1(save_resume_data, f);
 	}
 
 	bool torrent_handle::need_save_resume_data() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL_RET(bool, false, need_save_resume_data);
 		return r;
 	}
 
 	void torrent_handle::force_recheck() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL(force_recheck);
 	}
 
 	void torrent_handle::resume() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL(resume);
 	}
 
 	void torrent_handle::auto_managed(bool m) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL1(auto_managed, m);
 	}
 
 	void torrent_handle::set_priority(int p) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL1(set_priority, p);
 	}
 
 	int torrent_handle::queue_position() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL_RET(int, -1, queue_position);
 		return r;
 	}
 
 	void torrent_handle::queue_position_up() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL(queue_up);
 	}
 
 	void torrent_handle::queue_position_down() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL(queue_down);
 	}
 
 	void torrent_handle::queue_position_top() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL1(set_queue_position, 0);
 	}
 
 	void torrent_handle::queue_position_bottom() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL1(set_queue_position, INT_MAX);
 	}
 
 	void torrent_handle::clear_error() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL(clear_error);
 	}
 
 	void torrent_handle::set_tracker_login(std::string const& name
 		, std::string const& password) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL2(set_tracker_login, name, password);
 	}
 
-#ifndef TORRENT_NO_DEPRECATE
-#if !TORRENT_NO_FPU
-	void torrent_handle::file_progress(std::vector<float>& progress) const
-	{
-		INVARIANT_CHECK;
-		TORRENT_SYNC_CALL1(file_progress, boost::ref(progress));
-	}
-#endif
-#endif
-
 	void torrent_handle::file_progress(std::vector<size_type>& progress, int flags) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL2(file_progress, boost::ref(progress), flags);
 	}
 
 	torrent_status torrent_handle::status(boost::uint32_t flags) const
 	{
-		INVARIANT_CHECK;
 		torrent_status st;
 		TORRENT_SYNC_CALL2(status, &st, flags);
 		return st;
@@ -543,45 +494,32 @@ namespace libtorrent
 
 	void torrent_handle::set_sequential_download(bool sd) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL1(set_sequential_download, sd);
 	}
 
-	std::string torrent_handle::name() const
-	{
-		INVARIANT_CHECK;
-		TORRENT_SYNC_CALL_RET(std::string, "", name);
-		return r;
-	}
-
 	void torrent_handle::piece_availability(std::vector<int>& avail) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL1(piece_availability, boost::ref(avail));
 	}
 
 	void torrent_handle::piece_priority(int index, int priority) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL2(set_piece_priority, index, priority);
 	}
 
 	int torrent_handle::piece_priority(int index) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL_RET1(int, 0, piece_priority, index);
 		return r;
 	}
 
 	void torrent_handle::prioritize_pieces(std::vector<int> const& pieces) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL1(prioritize_pieces, pieces);
 	}
 
 	std::vector<int> torrent_handle::piece_priorities() const
 	{
-		INVARIANT_CHECK;
 		std::vector<int> ret;
 		TORRENT_SYNC_CALL1(piece_priorities, &ret);
 		return ret;
@@ -589,136 +527,125 @@ namespace libtorrent
 
 	void torrent_handle::file_priority(int index, int priority) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL2(set_file_priority, index, priority);
 	}
 
 	int torrent_handle::file_priority(int index) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL_RET1(int, 0, file_priority, index);
 		return r;
 	}
 
 	void torrent_handle::prioritize_files(std::vector<int> const& files) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL1(prioritize_files, files);
 	}
 
 	std::vector<int> torrent_handle::file_priorities() const
 	{
-		INVARIANT_CHECK;
 		std::vector<int> ret;
 		TORRENT_SYNC_CALL1(file_priorities, &ret);
 		return ret;
 	}
 
+	void torrent_handle::use_interface(const char* net_interface) const
+	{
+		TORRENT_ASYNC_CALL1(use_interface, std::string(net_interface));
+	}
+
 #ifndef TORRENT_NO_DEPRECATE
 // ============ start deprecation ===============
 
+#if !TORRENT_NO_FPU
+	void torrent_handle::file_progress(std::vector<float>& progress) const
+	{
+		TORRENT_SYNC_CALL1(file_progress, boost::ref(progress));
+	}
+#endif
+
 	int torrent_handle::get_peer_upload_limit(tcp::endpoint ip) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL_RET1(int, -1, get_peer_upload_limit, ip);
 		return r;
 	}
 
 	int torrent_handle::get_peer_download_limit(tcp::endpoint ip) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL_RET1(int, -1, get_peer_download_limit, ip);
 		return r;
 	}
 
 	void torrent_handle::set_peer_upload_limit(tcp::endpoint ip, int limit) const
 	{
-		INVARIANT_CHECK;
-		TORRENT_ASSERT(limit >= -1);
+		TORRENT_ASSERT_PRECOND(limit >= -1);
 		TORRENT_ASYNC_CALL2(set_peer_upload_limit, ip, limit);
 	}
 
 	void torrent_handle::set_peer_download_limit(tcp::endpoint ip, int limit) const
 	{
-		INVARIANT_CHECK;
-		TORRENT_ASSERT(limit >= -1);
+		TORRENT_ASSERT_PRECOND(limit >= -1);
 		TORRENT_ASYNC_CALL2(set_peer_download_limit, ip, limit);
 	}
 
 	void torrent_handle::set_ratio(float ratio) const
 	{
-		INVARIANT_CHECK;
-		
-		TORRENT_ASSERT(ratio >= 0.f);
-		if (ratio < 1.f && ratio > 0.f)
-			ratio = 1.f;
-		TORRENT_ASYNC_CALL1(set_ratio, ratio);
+		TORRENT_ASSERT_PRECOND(ratio >= 0.f);
 	}
 
 	bool torrent_handle::is_seed() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL_RET(bool, false, is_seed);
 		return r;
 	}
 
 	bool torrent_handle::is_finished() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL_RET(bool, false, is_finished);
 		return r;
 	}
 
 	bool torrent_handle::is_paused() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL_RET(bool, false, is_torrent_paused);
 		return r;
 	}
 
 	bool torrent_handle::is_sequential_download() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL_RET(bool, false, is_sequential_download);
 		return r;
 	}
 
 	bool torrent_handle::is_auto_managed() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL_RET(bool, false, is_auto_managed);
 		return r;
 	}
 
 	bool torrent_handle::has_metadata() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL_RET(bool, false, valid_metadata);
 		return r;
 	}
 
 	void torrent_handle::filter_piece(int index, bool filter) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL2(filter_piece, index, filter);
 	}
 
 	void torrent_handle::filter_pieces(std::vector<bool> const& pieces) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL1(filter_pieces, pieces);
 	}
 
 	bool torrent_handle::is_piece_filtered(int index) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL_RET1(bool, false, is_piece_filtered, index);
 		return r;
 	}
 
 	std::vector<bool> torrent_handle::filtered_pieces() const
 	{
-		INVARIANT_CHECK;
 		std::vector<bool> ret;
 		TORRENT_SYNC_CALL1(filtered_pieces, ret);
 		return ret;
@@ -726,13 +653,11 @@ namespace libtorrent
 
 	void torrent_handle::filter_files(std::vector<bool> const& files) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL1(filter_files, files);
 	}
 
 	bool torrent_handle::super_seeding() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL_RET(bool, false, super_seeding);
 		return r;
 	}
@@ -742,7 +667,6 @@ namespace libtorrent
 
 	std::vector<announce_entry> torrent_handle::trackers() const
 	{
-		INVARIANT_CHECK;
 		const static std::vector<announce_entry> empty;
 		TORRENT_SYNC_CALL_RET(std::vector<announce_entry>, empty, trackers);
 		return r;
@@ -750,19 +674,16 @@ namespace libtorrent
 
 	void torrent_handle::add_url_seed(std::string const& url) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL2(add_web_seed, url, web_seed_entry::url_seed);
 	}
 
 	void torrent_handle::remove_url_seed(std::string const& url) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL2(remove_web_seed, url, web_seed_entry::url_seed);
 	}
 
 	std::set<std::string> torrent_handle::url_seeds() const
 	{
-		INVARIANT_CHECK;
 		const static std::set<std::string> empty;
 		TORRENT_SYNC_CALL_RET1(std::set<std::string>, empty, web_seeds, web_seed_entry::url_seed);
 		return r;
@@ -770,19 +691,16 @@ namespace libtorrent
 
 	void torrent_handle::add_http_seed(std::string const& url) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL2(add_web_seed, url, web_seed_entry::http_seed);
 	}
 
 	void torrent_handle::remove_http_seed(std::string const& url) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL2(remove_web_seed, url, web_seed_entry::http_seed);
 	}
 
 	std::set<std::string> torrent_handle::http_seeds() const
 	{
-		INVARIANT_CHECK;
 		const static std::set<std::string> empty;
 		TORRENT_SYNC_CALL_RET1(std::set<std::string>, empty, web_seeds, web_seed_entry::http_seed);
 		return r;
@@ -791,45 +709,54 @@ namespace libtorrent
 	void torrent_handle::replace_trackers(
 		std::vector<announce_entry> const& urls) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL1(replace_trackers, urls);
 	}
 
 	void torrent_handle::add_tracker(announce_entry const& url) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL1(add_tracker, url);
 	}
 
 	void torrent_handle::add_piece(int piece, char const* data, int flags) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL3(add_piece, piece, data, flags);
 	}
 
 	void torrent_handle::read_piece(int piece) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL1(read_piece, piece);
 	}
 
 	bool torrent_handle::have_piece(int piece) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL_RET1(bool, false, have_piece, piece);
 		return r;
 	}
 
 	storage_interface* torrent_handle::get_storage_impl() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL_RET(storage_interface*, 0, get_storage);
 		return r;
 	}
 
+	bool torrent_handle::is_valid() const
+	{
+		return !m_torrent.expired();
+	}
+
+	boost::intrusive_ptr<torrent_info const> torrent_handle::torrent_file() const
+	{
+		TORRENT_SYNC_CALL_RET(boost::intrusive_ptr<torrent_info const>
+			, boost::intrusive_ptr<torrent_info const>(), get_torrent_copy);
+		return r;
+	}
+
+#ifndef TORRENT_NO_DEPRECATE
+	// this function should either be removed, or return
+	// reference counted handle to the torrent_info which
+	// forces the torrent to stay loaded while the client holds it
 	torrent_info const& torrent_handle::get_torrent_info() const
 	{
-		INVARIANT_CHECK;
 #ifdef BOOST_NO_EXCEPTIONS
 		const static torrent_info empty(sha1_hash(0));
 #endif
@@ -840,7 +767,6 @@ namespace libtorrent
 #else
 			throw_invalid_handle();
 #endif
-//		mutex::scoped_lock l(t->session().m_mutex);
 		if (!t->valid_metadata())
 #ifdef BOOST_NO_EXCEPTIONS
 			return empty;
@@ -850,17 +776,8 @@ namespace libtorrent
 		return t->torrent_file();
 	}
 
-	bool torrent_handle::is_valid() const
-	{
-		INVARIANT_CHECK;
-		return !m_torrent.expired();
-	}
-
-#ifndef TORRENT_NO_DEPRECATE
 	entry torrent_handle::write_resume_data() const
 	{
-		INVARIANT_CHECK;
-
 		entry ret(entry::dictionary_t);
 		TORRENT_SYNC_CALL1(write_resume_data, boost::ref(ret));
 		t = m_torrent.lock();
@@ -873,104 +790,109 @@ namespace libtorrent
 				, &ses.mut, boost::function<void(void)>(boost::bind(
 					&piece_manager::write_resume_data, &t->filesystem(), boost::ref(ret)))));
 			t.reset();
-			do { ses.cond.wait(l); } while(!done);
+			while (!done) { ses.cond.wait(l); }
 		}
 
 		return ret;
 	}
-#endif
 
 	std::string torrent_handle::save_path() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL_RET(std::string, "", save_path);
 		return r;
 	}
 
+	std::string torrent_handle::name() const
+	{
+		TORRENT_SYNC_CALL_RET(std::string, "", name);
+		return r;
+	}
+
+#endif
+
 	void torrent_handle::connect_peer(tcp::endpoint const& adr, int source) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL2(add_peer, adr, source);
 	}
 
+#ifndef TORRENT_NO_DEPRECATE
 	void torrent_handle::force_reannounce(
 		boost::posix_time::time_duration duration) const
 	{
-		INVARIANT_CHECK;
-		TORRENT_ASYNC_CALL1(force_tracker_request, time_now() + seconds(duration.total_seconds()));
-	}
-
-#ifndef TORRENT_DISABLE_DHT
-	void torrent_handle::force_dht_announce() const
-	{
-		INVARIANT_CHECK;
-		TORRENT_ASYNC_CALL(dht_announce);
+		TORRENT_ASYNC_CALL2(force_tracker_request, time_now()
+			+ seconds(duration.total_seconds()), -1);
 	}
 #endif
 
-	void torrent_handle::force_reannounce() const
+	void torrent_handle::force_dht_announce() const
 	{
-		INVARIANT_CHECK;
-		TORRENT_ASYNC_CALL1(force_tracker_request, time_now());
+#ifndef TORRENT_DISABLE_DHT
+		TORRENT_ASYNC_CALL(dht_announce);
+#endif
+	}
+
+	void torrent_handle::force_reannounce(int s, int idx) const
+	{
+		TORRENT_ASYNC_CALL2(force_tracker_request, time_now() + seconds(s), idx);
 	}
 
 	void torrent_handle::scrape_tracker() const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL(scrape_tracker);
 	}
 
 	void torrent_handle::super_seeding(bool on) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL1(super_seeding, on);
 	}
 
-#ifndef TORRENT_DISABLE_RESOLVE_COUNTRIES
 	void torrent_handle::resolve_countries(bool r)
 	{
-		INVARIANT_CHECK;
+#ifndef TORRENT_DISABLE_RESOLVE_COUNTRIES
 		TORRENT_ASYNC_CALL1(resolve_countries, r);
+#endif
 	}
 
 	bool torrent_handle::resolve_countries() const
 	{
-		INVARIANT_CHECK;
+#ifndef TORRENT_DISABLE_RESOLVE_COUNTRIES
 		TORRENT_SYNC_CALL_RET(bool, false, resolving_countries);
 		return r;
-	}
+#else
+		return false;
 #endif
+	}
 
 	void torrent_handle::get_full_peer_list(std::vector<peer_list_entry>& v) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL1(get_full_peer_list, boost::ref(v));
 	}
 
 	void torrent_handle::get_peer_info(std::vector<peer_info>& v) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL1(get_peer_info, boost::ref(v));
 	}
 
 	void torrent_handle::get_download_queue(std::vector<partial_piece_info>& queue) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_SYNC_CALL1(get_download_queue, &queue);
 	}
 
 	void torrent_handle::set_piece_deadline(int index, int deadline, int flags) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL3(set_piece_deadline, index, deadline, flags);
 	}
 
 	void torrent_handle::reset_piece_deadline(int index) const
 	{
-		INVARIANT_CHECK;
 		TORRENT_ASYNC_CALL1(reset_piece_deadline, index);
 	}
 
+	void torrent_handle::clear_piece_deadlines() const
+	{
+		TORRENT_ASYNC_CALL(clear_time_critical);
+	}
+
 	boost::shared_ptr<torrent> torrent_handle::native_handle() const
 	{
 		return m_torrent.lock();
@@ -983,7 +905,9 @@ namespace libtorrent
 
 	std::size_t hash_value(torrent_handle const& th)
 	{
-		return std::size_t(th.m_torrent.lock().get());
+		// using the locked shared_ptr value as hash doesn't work
+		// for expired weak_ptrs. So, we're left with a hack
+		return std::size_t(*reinterpret_cast<void* const*>(&th.m_torrent));
 	}
 
 }
