@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003, Arvid Norberg
+Copyright (c) 2003-2014, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -30,8 +30,6 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
-#include "libtorrent/pch.hpp"
-
 #include <vector>
 #include <boost/limits.hpp>
 #include <boost/bind.hpp>
@@ -65,17 +63,16 @@ namespace libtorrent
 		, boost::weak_ptr<torrent> t
 		, boost::shared_ptr<socket_type> s
 		, tcp::endpoint const& remote
-		, std::string const& url
-		, policy::peer* peerinfo
-		, std::string const& auth
-		, web_seed_entry::headers_t const& extra_headers)
-		: web_connection_base(ses, t, s, remote, url, peerinfo, auth, extra_headers)
-		, m_url(url)
+		, web_seed_entry& web)
+		: web_connection_base(ses, t, s, remote, web)
+		, m_url(web.url)
+		, m_web(&web)
 		, m_received_body(0)
 		, m_range_pos(0)
 		, m_block_pos(0)
 		, m_chunk_pos(0)
 		, m_partial_chunk_header(0)
+		, m_num_responses(0)
 	{
 		INVARIANT_CHECK;
 
@@ -88,7 +85,13 @@ namespace libtorrent
 		// we always prefer downloading 1 MiB chunks
 		// from web seeds, or whole pieces if pieces
 		// are larger than a MiB
-		prefer_whole_pieces((std::max)((1024 * 1024) / tor->torrent_file().piece_length(), 1));
+		int preferred_size = 1024 * 1024;
+
+		// if the web server is known not to support keep-alive.
+		// request even larger blocks at a time
+		if (!web.supports_keepalive) preferred_size *= 4;
+
+		prefer_whole_pieces((std::max)(preferred_size / tor->torrent_file().piece_length(), 1));
 		
 		// we want large blocks as well, so
 		// we can request more bytes at once
@@ -97,13 +100,61 @@ namespace libtorrent
 		request_large_blocks(true);
 
 #ifdef TORRENT_VERBOSE_LOGGING
-		peer_log("*** web_peer_connection %s", url.c_str());
+		peer_log("*** web_peer_connection %s", web.url.c_str());
 #endif
 	}
 
+	void web_peer_connection::on_connected()
+	{
+		incoming_have_all();
+		if (m_web->restart_request.piece != -1)
+		{
+			// increase the chances of requesting the block
+			// we have partial data for already, to finish it
+			incoming_suggest(m_web->restart_request.piece);
+		}
+		web_connection_base::on_connected();
+	}
+
 	void web_peer_connection::disconnect(error_code const& ec, int error)
 	{
+		if (is_disconnecting()) return;
+
 		boost::shared_ptr<torrent> t = associated_torrent().lock();
+
+		if (!m_requests.empty() && !m_file_requests.empty()
+			&& !m_piece.empty() && m_web)
+		{
+#if 0
+			std::cerr << this << " SAVE-RESTART-DATA: data: " << m_piece.size()
+				<< " req: " << m_requests.front().piece
+				<< " off: " << m_requests.front().start
+				<< std::endl;
+#endif
+			m_web->restart_request = m_requests.front();
+			if (!m_web->restart_piece.empty())
+			{
+				// we're about to replace a different restart piece
+				// buffer. So it was wasted download
+				if (t) t->add_redundant_bytes(m_web->restart_piece.size()
+					, torrent::piece_closing);
+			}
+			m_web->restart_piece.swap(m_piece);
+
+			// we have to do this to not count this data as redundant. The
+			// upper layer will call downloading_piece_progress and assume
+			// it's all wasted download. Since we're saving it here, it isn't.
+			m_requests.clear();
+			m_block_pos = 0;
+		}
+
+		if (m_web && !m_web->supports_keepalive && error == 0)
+		{
+			// if the web server doesn't support keepalive and we were
+			// disconnected as a graceful EOF, reconnect right away
+			if (t) t->session().m_io_service.post(
+				boost::bind(&torrent::maybe_connect_web_seeds, t));
+		}
 		peer_connection::disconnect(ec, error);
 		if (t) t->disconnect_web_seed(this);
 	}
@@ -178,6 +229,7 @@ namespace libtorrent
 		}
 
 		torrent_info const& info = t->torrent_file();
+		peer_request req = r;
 		
 		std::string request;
 		request.reserve(400);
@@ -194,6 +246,32 @@ namespace libtorrent
 			pr.piece = r.piece + request_offset / piece_size;
 			m_requests.push_back(pr);
 			size -= pr.length;
+			if (m_web->restart_request == m_requests.front())
+			{
+				m_piece.swap(m_web->restart_piece);
+				m_block_pos += m_piece.size();
+				peer_request& front = m_requests.front();
+				TORRENT_ASSERT(front.length > int(m_piece.size()));
+
+#if 0
+				std::cerr << this << " RESTART-DATA: data: " << m_piece.size()
+					<< " req: ( " << front.piece << ", " << front.start
+					<< ", " << (front.start + front.length - 1) << ")"
+					<< std::endl;
+#endif
+
+				req.start += m_piece.size();
+				req.length -= m_piece.size();
+
+				// just to keep the accounting straight for the upper layer.
+				// it doesn't know we just re-wrote the request
+				incoming_piece_fragment(m_piece.size());
+				m_web->restart_request.piece = -1;
+			}
+
+#if 0
+			std::cerr << this << " REQ: p: " << pr.piece << " " << pr.start << std::endl;
+#endif
 		}
 
 		proxy_settings const& ps = m_ses.proxy();
@@ -209,35 +287,36 @@ namespace libtorrent
 			request += " HTTP/1.1\r\n";
 			add_headers(request, ps, using_proxy);
 			request += "\r\nRange: bytes=";
-			request += to_string(size_type(r.piece) * info.piece_length() + r.start).elems;
+			request += to_string(size_type(req.piece) * info.piece_length()
+				+ req.start).elems;
 			request += "-";
-			request += to_string(size_type(r.piece) * info.piece_length() + r.start + r.length - 1).elems;
+			request += to_string(size_type(req.piece) * info.piece_length()
+				+ req.start + req.length - 1).elems;
 			request += "\r\n\r\n";
 			m_first_request = false;
 			m_file_requests.push_back(0);
 		}
 		else
 		{
-			std::vector<file_slice> files = info.orig_files().map_block(r.piece, r.start
-				, r.length);
+			std::vector<file_slice> files = info.orig_files().map_block(
+				req.piece, req.start, req.length);
 
 			for (std::vector<file_slice>::iterator i = files.begin();
 				i != files.end(); ++i)
 			{
 				file_slice const& f = *i;
-				if (info.orig_files().internal_at(f.file_index).pad_file)
+				if (info.orig_files().pad_file_at(f.file_index))
 				{
 					m_file_requests.push_back(f.file_index);
 					continue;
 				}
-
 				request += "GET ";
 				if (using_proxy)
 				{
 					// m_url is already a properly escaped URL
 					// with the correct slashes. Don't encode it again
 					request += m_url;
-					std::string path = info.orig_files().file_path(info.orig_files().internal_at(f.file_index));
+					std::string path = info.orig_files().file_path(f.file_index);
 #ifdef TORRENT_WINDOWS
 					convert_path_to_posix(path);
 #endif
@@ -249,7 +328,7 @@ namespace libtorrent
 					// with the correct slashes. Don't encode it again
 					request += m_path;
 
-					std::string path = info.orig_files().file_path(info.orig_files().internal_at(f.file_index));
+					std::string path = info.orig_files().file_path(f.file_index);
 #ifdef TORRENT_WINDOWS
 					convert_path_to_posix(path);
 #endif
@@ -263,6 +342,12 @@ namespace libtorrent
 				request += to_string(f.offset + f.size - 1).elems;
 				request += "\r\n\r\n";
 				m_first_request = false;
+
+#if 0
+				std::cerr << this << " SEND-REQUEST: f: " << f.file_index
+					<< " s: " << f.offset
+					<< " e: " << (f.offset + f.size - 1) << std::endl;
+#endif
 				TORRENT_ASSERT(f.file_index >= 0);
 				m_file_requests.push_back(f.file_index);
 			}
@@ -272,6 +357,13 @@ namespace libtorrent
 		peer_log("==> %s", request.c_str());
 #endif
 
+		// in case the first file on this series of requests is a padfile
+		// we need to handle it right now, and pretend that we got a response
+		// with zeros.
+		buffer::const_interval recv_buffer = receive_buffer();
+		handle_padfile(recv_buffer);
+		if (associated_torrent().expired()) return;
+
 		send_buffer(request.c_str(), request.size(), message_type_request);
 	}
 
@@ -295,7 +387,7 @@ namespace libtorrent
 		peer_request const& front_request = m_requests.front();
 
 		if (int(m_piece.size()) < front_request.length) return false;
-		TORRENT_ASSERT(int(m_piece.size() == front_request.length));
+		TORRENT_ASSERT(int(m_piece.size()) == front_request.length);
 
 		// each call to incoming_piece() may result in us becoming
 		// a seed. If we become a seed, all seeds we're connected to
@@ -320,6 +412,51 @@ namespace libtorrent
 		return true;
 	}
 
+	bool web_peer_connection::received_invalid_data(int index, bool single_peer)
+	{
+		if (!single_peer) return peer_connection::received_invalid_data(index, single_peer);
+
+		// when a web seed fails a hash check, do the following:
+		// 1. if the whole piece only overlaps a single file, mark that file as not
+		//    have for this peer
+		// 2. if the piece overlaps more than one file, mark the piece as not have
+		//    for this peer
+		// 3. if it's a single file torrent, just ban it right away
+		// this handles the case where web seeds may have some files updated but not other
+
+		boost::shared_ptr<torrent> t = associated_torrent().lock();
+		file_storage const& fs = t->torrent_file().files();
+
+		// single file torrent
+		if (fs.num_files() == 1) return peer_connection::received_invalid_data(index, single_peer);
+
+		std::vector<file_slice> files = fs.map_block(index, 0, fs.piece_size(index));
+
+		if (files.size() == 1)
+		{
+			// assume the web seed has a different copy of this specific file
+			// than what we expect, and pretend not to have it.
+			int fi = files[0].file_index;
+			int first_piece = int(fs.file_offset(fi) / fs.piece_length());
+			// one past last piece
+			int end_piece = int((fs.file_offset(fi) + fs.file_size(fi) + 1) / fs.piece_length());
+			for (int i = first_piece; i < end_piece; ++i)
+				incoming_dont_have(i);
+		}
+		else
+		{
+			incoming_dont_have(index);
+		}
+
+		peer_connection::received_invalid_data(index, single_peer);
+
+		// if we don't think we have any of the files, allow banning the web seed
+		if (num_have_pieces() == 0) return true;
+
+		// don't disconnect, we won't request anything from this file again
+		return false;
+	}
+
 	void web_peer_connection::on_receive(error_code const& error
 		, std::size_t bytes_transferred)
 	{
@@ -421,6 +558,15 @@ namespace libtorrent
 			// we just completed reading the header
 			if (!header_finished)
 			{
+				++m_num_responses;
+
+				if (m_parser.connection_close())
+				{
+					incoming_choke();
+					if (m_num_responses == 1)
+						m_web->supports_keepalive = false;
+				}
+
 #ifdef TORRENT_VERBOSE_LOGGING
 				peer_log("*** STATUS: %d %s", m_parser.status_code(), m_parser.message().c_str());
 				std::multimap<std::string, std::string> const& headers = m_parser.headers();
@@ -431,8 +577,11 @@ namespace libtorrent
 				// if the status code is not one of the accepted ones, abort
 				if (!is_ok_status(m_parser.status_code()))
 				{
+					// TODO: 3 just make this peer not have the pieces
+					// associated with the file we just requested. Only
+					// when it doesn't have any of the file do the following
 					int retry_time = atoi(m_parser.header("retry-after").c_str());
-					if (retry_time <= 0) retry_time = 5 * 60;
+					if (retry_time <= 0) retry_time = m_ses.settings().urlseed_wait_retry;
 					// temporarily unavailable, retry later
 					t->retry_web_seed(this, retry_time);
 					std::string error_msg = to_string(m_parser.status_code()).elems
@@ -462,6 +611,7 @@ namespace libtorrent
 					{
 						// we should not try this server again.
 						t->remove_web_seed(this);
+						m_web = NULL;
 						disconnect(errors::missing_location, 2);
 #ifdef TORRENT_DEBUG
 						TORRENT_ASSERT(m_statistics.last_payload_downloaded()
@@ -481,8 +631,11 @@ namespace libtorrent
 						TORRENT_ASSERT(!m_file_requests.empty());
 						int file_index = m_file_requests.front();
 
+// TODO: 2 create a mapping of file-index to redirection URLs. Use that to form
+// URLs instead. Support to reconnect to a new server without destructing this
+// peer_connection
 						torrent_info const& info = t->torrent_file();
-						std::string path = info.orig_files().file_path(info.orig_files().at(file_index));
+						std::string path = info.orig_files().file_path(file_index);
 #ifdef TORRENT_WINDOWS
 						convert_path_to_posix(path);
 #endif
@@ -491,6 +644,7 @@ namespace libtorrent
 						if (i == std::string::npos)
 						{
 							t->remove_web_seed(this);
+							m_web = NULL;
 							disconnect(errors::invalid_redirection, 2);
 #ifdef TORRENT_DEBUG
 							TORRENT_ASSERT(m_statistics.last_payload_downloaded()
@@ -501,8 +655,14 @@ namespace libtorrent
 						}
 						location.resize(i);
 					}
-					t->add_web_seed(location, web_seed_entry::url_seed, m_external_auth, m_extra_headers);
+					else
+					{
+						location = resolve_redirect_location(m_url, location);
+					}
+					t->add_web_seed(location, web_seed_entry::url_seed
+						, m_external_auth, m_extra_headers);
 					t->remove_web_seed(this);
+					m_web = NULL;
 					disconnect(errors::redirecting, 2);
 #ifdef TORRENT_DEBUG
 					TORRENT_ASSERT(m_statistics.last_payload_downloaded()
@@ -550,6 +710,7 @@ namespace libtorrent
 					m_statistics.received_bytes(0, bytes_transferred);
 					// we should not try this server again.
 					t->remove_web_seed(this);
+					m_web = NULL;
 					disconnect(errors::invalid_range);
 #ifdef TORRENT_DEBUG
 					TORRENT_ASSERT(m_statistics.last_payload_downloaded()
@@ -570,6 +731,7 @@ namespace libtorrent
 					m_statistics.received_bytes(0, bytes_transferred);
 					// we should not try this server again.
 					t->remove_web_seed(this);
+					m_web = NULL;
 					disconnect(errors::no_content_length, 2);
 #ifdef TORRENT_DEBUG
 					TORRENT_ASSERT(m_statistics.last_payload_downloaded()
@@ -666,11 +828,6 @@ namespace libtorrent
 			m_block_pos += payload_transferred;
 			if (m_range_pos > range_end - range_start) m_range_pos = range_end - range_start;
 
-#if 0
-			std::cerr << "REQUESTS: m_requests: " << m_requests.size()
-				<< " file_requests: " << m_file_requests.size() << std::endl;
-#endif
-
 			int file_index = m_file_requests.front();
 			peer_request in_range = info.orig_files().map_file(file_index, range_start
 				, int(range_end - range_start));
@@ -681,13 +838,6 @@ namespace libtorrent
 			size_type re = rs + in_range.length;
 			// file start
 			size_type fs = size_type(front_request.piece) * info.piece_length() + front_request.start;
-#if 0
-			size_type fe = fs + front_request.length;
-
-			std::cerr << "RANGE: r = (" << rs << ", " << re << " ) "
-				"f = (" << fs << ", " << fe << ") "
-				"file_index = " << file_index << " received_body = " << m_received_body << std::endl;
-#endif
 
 			// the http response body consists of 3 parts
 			// 1. the middle of a block or the ending of a block
@@ -704,6 +854,7 @@ namespace libtorrent
 				m_statistics.received_bytes(0, bytes_transferred);
 				// this means the end of the incoming request ends _before_ the
 				// first expected byte (fs + m_piece.size())
+
 				disconnect(errors::invalid_range, 2);
 				return;
 			}
@@ -712,7 +863,8 @@ namespace libtorrent
 			// fits in the range) we should not start a partial piece, since we soon
 			// will receive enough to call incoming_piece() and pass the read buffer
 			// directly (in the next loop below).
-			if (range_overlaps_request && !range_contains(in_range, front_request, info.piece_length()))
+			if (range_overlaps_request
+				&& !range_contains(in_range, front_request, info.piece_length()))
 			{
 				// the start of the next block to receive is stored
 				// in m_piece. We need to append the rest of that
@@ -726,7 +878,7 @@ namespace libtorrent
 				if (copy_size > m_chunk_pos && m_chunk_pos > 0) copy_size = m_chunk_pos;
 				if (copy_size > 0)
 				{
-					TORRENT_ASSERT(m_piece.size() == m_received_in_piece);
+					TORRENT_ASSERT(int(m_piece.size()) == m_received_in_piece);
 					m_piece.resize(piece_size + copy_size);
 					std::memcpy(&m_piece[0] + piece_size, recv_buffer.begin, copy_size);
 					TORRENT_ASSERT(int(m_piece.size()) <= front_request.length);
@@ -741,7 +893,7 @@ namespace libtorrent
 					TORRENT_ASSERT(m_received_body <= range_end - range_start);
 					TORRENT_ASSERT(int(m_piece.size()) <= front_request.length);
 					incoming_piece_fragment(copy_size);
-					TORRENT_ASSERT(m_piece.size() == m_received_in_piece);
+					TORRENT_ASSERT(int(m_piece.size()) == m_received_in_piece);
 				}
 
 				if (maybe_harvest_block())
@@ -759,6 +911,11 @@ namespace libtorrent
 
 				incoming_piece_fragment(r.length);
 				incoming_piece(r, recv_buffer.begin);
+
+#ifdef TORRENT_VERBOSE_LOGGING
+				peer_log("<== POP REQUEST [ piece: %d start: %d len: %d ]"
+					, r.piece, r.start, r.length);
+#endif
 				m_requests.pop_front();
 				if (associated_torrent().expired()) return;
 				TORRENT_ASSERT(m_block_pos >= r.length);
@@ -778,9 +935,6 @@ namespace libtorrent
 
 			if (!m_requests.empty())
 			{
-//				range_overlaps_request = in_range.start + in_range.length
-//					> m_requests.front().start + int(m_piece.size());
-
 				if (in_range.start + in_range.length < m_requests.front().start + m_requests.front().length
 					&& (m_received_body + recv_buffer.left() >= range_end - range_start))
 				{
@@ -790,14 +944,14 @@ namespace libtorrent
 					TORRENT_ASSERT(copy_size >= 0);
 					if (copy_size > 0)
 					{
-						TORRENT_ASSERT(m_piece.size() == m_received_in_piece);
+						TORRENT_ASSERT(int(m_piece.size()) == m_received_in_piece);
 						m_piece.resize(piece_size + copy_size);
 						std::memcpy(&m_piece[0] + piece_size, recv_buffer.begin, copy_size);
 						recv_buffer.begin += copy_size;
 						m_received_body += copy_size;
 						m_body_start += copy_size;
 						incoming_piece_fragment(copy_size);
-						TORRENT_ASSERT(m_piece.size() == m_received_in_piece);
+						TORRENT_ASSERT(int(m_piece.size()) == m_received_in_piece);
 					}
 					TORRENT_ASSERT(m_received_body == range_end - range_start);
 				}
@@ -830,31 +984,11 @@ namespace libtorrent
 				m_chunk_pos = 0;
 				m_partial_chunk_header = 0;
 				
-				torrent_info const& info = t->torrent_file();
-				while (!m_file_requests.empty()
-					&& info.orig_files().internal_at(m_file_requests.front()).pad_file)
-				{
-					// the next file is a pad file. We didn't actually send
-					// a request for this since it most likely doesn't exist on
-					// the web server anyway. Just pretend that we received a
-					// bunch of zeroes here and pop it again
-					int file_index = m_file_requests.front();
-					m_file_requests.pop_front();
-					size_type file_size = info.orig_files().file_size(info.orig_files().internal_at(file_index));
-					TORRENT_ASSERT(m_block_pos < front_request.length);
-					int pad_size = (std::min)(file_size, size_type(front_request.length - m_block_pos));
-
-					// insert zeroes to represent the pad file
-					m_piece.resize(m_piece.size() + pad_size, 0);
-					m_block_pos += pad_size;
-					incoming_piece_fragment(pad_size);
-
-					if (maybe_harvest_block())
-						recv_buffer = receive_buffer();
-					if (associated_torrent().expired()) return;
-				}
+				handle_padfile(recv_buffer);
+				if (associated_torrent().expired()) return;
 				continue;
 			}
+
 			if (bytes_transferred == 0 || payload_transferred == 0)
 			{
 #ifdef TORRENT_DEBUG
@@ -880,5 +1014,37 @@ namespace libtorrent
 		p.connection_type = peer_info::web_seed;
 	}
 
+	void web_peer_connection::handle_padfile(buffer::const_interval& recv_buffer)
+	{
+		boost::shared_ptr<torrent> t = associated_torrent().lock();
+		TORRENT_ASSERT(t);
+		torrent_info const& info = t->torrent_file();
+
+		while (!m_file_requests.empty()
+			&& info.orig_files().pad_file_at(m_file_requests.front()))
+		{
+			// the next file is a pad file. We didn't actually send
+			// a request for this since it most likely doesn't exist on
+			// the web server anyway. Just pretend that we received a
+			// bunch of zeroes here and pop it again
+			int file_index = m_file_requests.front();
+			m_file_requests.pop_front();
+			size_type file_size = info.orig_files().file_size(file_index);
+
+			peer_request front_request = m_requests.front();
+
+			TORRENT_ASSERT(m_block_pos < front_request.length);
+			int pad_size = int((std::min)(file_size, size_type(front_request.length - m_block_pos)));
+
+			// insert zeroes to represent the pad file
+			m_piece.resize(m_piece.size() + size_t(pad_size), 0);
+			m_block_pos += pad_size;
+			incoming_piece_fragment(pad_size);
+
+			if (maybe_harvest_block())
+				recv_buffer = receive_buffer();
+			if (associated_torrent().expired()) return;
+		}
+	}
 }
 
