@@ -37,7 +37,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/io_service.hpp"
 #include "libtorrent/error_code.hpp"
 #include "libtorrent/socket.hpp"
-#include "libtorrent/time.hpp"
+#include "dht_server.hpp"
 
 #include <boost/detail/atomic_count.hpp>
 #include <boost/shared_ptr.hpp>
@@ -62,6 +62,7 @@ struct dht_server
 	dht_server()
 		: m_dht_requests(0)
 		, m_socket(m_ios)
+		, m_port(0)
 	{
 		error_code ec;
 		m_socket.open(udp::v4(), ec);
@@ -91,6 +92,7 @@ struct dht_server
 
 	~dht_server()
 	{
+		m_socket.cancel();
 		m_socket.close();
 		if (m_thread) m_thread->join();
 	}
@@ -99,6 +101,13 @@ struct dht_server
 
 	int num_hits() const { return m_dht_requests; }
 
+	static void incoming_packet(error_code const& ec, size_t bytes_transferred, size_t *ret, error_code* error, bool* done)
+	{
+		*ret = bytes_transferred;
+		*error = ec;
+		*done = true;
+	}
+
 	void thread_fun()
 	{
 		char buffer[2000];
@@ -107,8 +116,17 @@ struct dht_server
 		{
 			error_code ec;
 			udp::endpoint from;
-			int bytes_transferred = m_socket.receive_from(
-				asio::buffer(buffer, sizeof(buffer)), from, 0, ec);
+			size_t bytes_transferred;
+			bool done = false;
+			m_socket.async_receive_from(
+				asio::buffer(buffer, sizeof(buffer)), from, 0
+				, boost::bind(&incoming_packet, _1, _2, &bytes_transferred, &ec, &done));
+			while (!done)
+			{
+				m_ios.run_one();
+				m_ios.reset();
+			}
+
 			if (ec == boost::asio::error::operation_aborted
 				|| ec == boost::asio::error::bad_descriptor) return;
 
