@@ -35,52 +35,64 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/session.hpp"
 #include <boost/tuple/tuple.hpp>
+#include "test.hpp"
 
 namespace libtorrent
 {
 	class alert;
 	struct add_torrent_params;
+	struct session_status;
 }
 
-std::auto_ptr<libtorrent::alert> wait_for_alert(libtorrent::session& ses
-	, int type, char const* name);
+int EXPORT print_failures();
+unsigned char EXPORT random_byte();
 
-bool print_alerts(libtorrent::session& ses, char const* name
+int EXPORT load_file(std::string const& filename, std::vector<char>& v, libtorrent::error_code& ec, int limit = 8000000);
+void EXPORT save_file(char const* filename, char const* data, int size);
+
+void EXPORT report_failure(char const* err, char const* file, int line);
+
+std::auto_ptr<libtorrent::alert> EXPORT wait_for_alert(
+	libtorrent::session& ses, int type, char const* name = "");
+
+void EXPORT print_ses_rate(float time
+	, libtorrent::torrent_status const* st1
+	, libtorrent::torrent_status const* st2
+	, libtorrent::torrent_status const* st3 = NULL);
+
+bool EXPORT print_alerts(libtorrent::session& ses, char const* name
 	, bool allow_disconnects = false
 	, bool allow_no_torrents = false
 	, bool allow_failed_fastresume = false
 	, bool (*)(libtorrent::alert*) = 0
 	, bool no_output = false);
 
-void wait_for_listen(libtorrent::session& ses, char const* name);
-void test_sleep(int millisec);
+void EXPORT wait_for_listen(libtorrent::session& ses, char const* name);
+void EXPORT test_sleep(int millisec);
 
-extern boost::detail::atomic_count g_udp_tracker_requests;
-extern boost::detail::atomic_count g_http_tracker_requests;
+void EXPORT create_random_files(std::string const& path, const int file_sizes[], int num_files);
 
-void create_random_files(std::string const& path, const int file_sizes[], int num_files);
-
-boost::intrusive_ptr<libtorrent::torrent_info> create_torrent(std::ostream* file = 0
+boost::intrusive_ptr<libtorrent::torrent_info> EXPORT create_torrent(std::ostream* file = 0
 	, int piece_size = 16 * 1024, int num_pieces = 13, bool add_tracker = true
 	, std::string ssl_certificate = "");
 
 boost::tuple<libtorrent::torrent_handle
 	, libtorrent::torrent_handle
 	, libtorrent::torrent_handle>
-setup_transfer(libtorrent::session* ses1, libtorrent::session* ses2
+EXPORT setup_transfer(libtorrent::session* ses1, libtorrent::session* ses2
 	, libtorrent::session* ses3, bool clear_files, bool use_metadata_transfer = true
 	, bool connect = true, std::string suffix = "", int piece_size = 16 * 1024
 	, boost::intrusive_ptr<libtorrent::torrent_info>* torrent = 0, bool super_seeding = false
 	, libtorrent::add_torrent_params const* p = 0, bool stop_lsd = true, bool use_ssl_ports = false
 	, boost::intrusive_ptr<libtorrent::torrent_info>* torrent2 = 0);
 
-int start_web_server(bool ssl = false, bool chunked = false);
-void stop_web_server();
-void start_proxy(int port, int type);
-void stop_proxy(int port);
+int EXPORT start_web_server(bool ssl = false, bool chunked = false
+	, bool keepalive = true);
 
-void stop_tracker();
-int start_tracker();
+void EXPORT stop_web_server();
+int EXPORT start_proxy(int type);
+void EXPORT stop_proxy(int port);
+void EXPORT stop_all_proxies();
 
 #endif
 
