@@ -34,6 +34,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "setup_transfer.hpp"
 #include "dht_server.hpp"
 #include "peer_server.hpp"
+#include "udp_tracker.hpp"
 #include "libtorrent/alert.hpp"
 #include "libtorrent/alert_types.hpp"
 
@@ -75,16 +76,19 @@ enum flags_t
 	expect_peer_connection = 64,
 };
 
-void test_proxy(proxy_settings::proxy_type proxy_type, int flags)
+session_proxy test_proxy(proxy_settings::proxy_type proxy_type, int flags)
 {
+#ifdef TORRENT_DISABLE_DHT
+	// if DHT is disabled, we won't get any requests to it
+	flags &= ~expect_dht_msg;
+#endif
 	fprintf(stderr, "\n=== TEST == proxy: %s anonymous-mode: %s\n\n", proxy_name[proxy_type], (flags & anonymous_mode) ? "yes" : "no");
 	int http_port = start_web_server();
-	int udp_port = start_tracker();
+	int udp_port = start_udp_tracker();
 	int dht_port = start_dht();
 	int peer_port = start_peer();
 
-	int prev_udp_announces = g_udp_tracker_requests;
-	int prev_http_announces = g_http_tracker_requests;
+	int prev_udp_announces = num_udp_announces();
 
 	int const alert_mask = alert::all_categories
 		& ~alert::progress_notification
@@ -93,10 +97,14 @@ void test_proxy(proxy_settings::proxy_type proxy_type, int flags)
 	session* s = new libtorrent::session(fingerprint("LT", 0, 1, 0, 0), std::make_pair(48875, 49800), "0.0.0.0", 0, alert_mask);
 
 	session_settings sett;
+	sett.stop_tracker_timeout = 2;
+	sett.tracker_completion_timeout = 2;
+	sett.tracker_receive_timeout = 2;
 	sett.half_open_limit = 1;
 	sett.announce_to_all_trackers = true;
 	sett.announce_to_all_tiers = true;
 	sett.anonymous_mode = flags & anonymous_mode;
+	sett.force_proxy = flags & anonymous_mode;
 
 	// if we don't do this, the peer connection test
 	// will be delayed by several seconds, by first
@@ -104,6 +112,8 @@ void test_proxy(proxy_settings::proxy_type proxy_type, int flags)
 	sett.enable_outgoing_utp = false;
 	s->set_settings(sett);
 
+	// in non-anonymous mode we circumvent/ignore the proxy if it fails
+	// wheras in anonymous mode, we just fail
 	proxy_settings ps;
 	ps.hostname = "non-existing.com";
 	ps.port = 4444;
@@ -113,6 +123,7 @@ void test_proxy(proxy_settings::proxy_type proxy_type, int flags)
 	s->start_dht();
 
 	error_code ec;
+	remove_all("tmp1_privacy", ec);
 	create_directory("tmp1_privacy", ec);
 	std::ofstream file(combine_path("tmp1_privacy", "temporary").c_str());
 	boost::intrusive_ptr<torrent_info> t = ::create_torrent(&file, 16 * 1024, 13, false);
@@ -129,23 +140,39 @@ void test_proxy(proxy_settings::proxy_type proxy_type, int flags)
 	add_torrent_params addp;
 	addp.flags &= ~add_torrent_params::flag_paused;
 	addp.flags &= ~add_torrent_params::flag_auto_managed;
+
+	// we don't want to waste time checking the torrent, just go straight into
+	// seeding it, announcing to trackers and connecting to peers
+	addp.flags |= add_torrent_params::flag_seed_mode;
+
 	addp.ti = t;
 	addp.save_path = "tmp1_privacy";
 	addp.dht_nodes.push_back(std::pair<std::string, int>("127.0.0.1", dht_port));
 	torrent_handle h = s->add_torrent(addp);
 
+	printf("connect_peer: 127.0.0.1:%d\n", peer_port);
 	h.connect_peer(tcp::endpoint(address_v4::from_string("127.0.0.1"), peer_port));
 
 	rejected_trackers.clear();
-	for (int i = 0; i < 15; ++i)
+
+#ifdef TORRENT_USE_VALGRIND
+	const int timeout = 100;
+#else
+	const int timeout = 20;
+#endif
+
+	for (int i = 0; i < timeout; ++i)
 	{
 		print_alerts(*s, "s", false, false, false, &alert_predicate);
 		test_sleep(100);
+
+		if (num_udp_announces() >= prev_udp_announces + 1
+			&& num_peer_hits() > 0)
+			break;
 	}
 
 	// we should have announced to the tracker by now
-	TEST_EQUAL(g_udp_tracker_requests, prev_udp_announces + bool(flags & expect_udp_connection));
-	TEST_EQUAL(g_http_tracker_requests, prev_http_announces + bool(flags & expect_http_connection));
+	TEST_EQUAL(num_udp_announces(), prev_udp_announces + bool(flags & expect_udp_connection));
 	if (flags & expect_dht_msg)
 	{
 		TEST_CHECK(num_dht_hits() > 0);
@@ -171,40 +198,42 @@ void test_proxy(proxy_settings::proxy_type proxy_type, int flags)
 		TEST_CHECK(std::find(rejected_trackers.begin(), rejected_trackers.end(), http_tracker_url) != rejected_trackers.end());
 
 	fprintf(stderr, "%s: ~session\n", time_now_string());
+	session_proxy pr = s->abort();
 	delete s;
-	fprintf(stderr, "%s: ~session done\n", time_now_string());
 
 	stop_peer();
 	stop_dht();
-	stop_tracker();
+	stop_udp_tracker();
 	stop_web_server();
+	return pr;
 }
 
 int test_main()
 {
+	session_proxy pr[20];
 	// not using anonymous mode
 	// UDP fails open if we can't connect to the proxy
 	// or if the proxy doesn't support UDP
-	test_proxy(proxy_settings::none, expect_udp_connection | expect_http_connection | expect_dht_msg | expect_peer_connection);
-	test_proxy(proxy_settings::socks4, expect_udp_connection | expect_dht_msg);
-	test_proxy(proxy_settings::socks5, expect_udp_connection | expect_dht_msg);
-	test_proxy(proxy_settings::socks5_pw, expect_udp_connection | expect_dht_msg);
-	test_proxy(proxy_settings::http, expect_udp_connection | expect_dht_msg);
-	test_proxy(proxy_settings::http_pw, expect_udp_connection | expect_dht_msg);
-	test_proxy(proxy_settings::i2p_proxy, expect_udp_connection | expect_dht_msg);
+	pr[0] = test_proxy(proxy_settings::none, expect_udp_connection | expect_http_connection | expect_dht_msg | expect_peer_connection);
+	pr[1] = test_proxy(proxy_settings::socks4, expect_udp_connection | expect_dht_msg);
+	pr[2] = test_proxy(proxy_settings::socks5, expect_udp_connection | expect_dht_msg);
+	pr[3] = test_proxy(proxy_settings::socks5_pw, expect_udp_connection | expect_dht_msg);
+	pr[4] = test_proxy(proxy_settings::http, expect_udp_connection | expect_dht_msg);
+	pr[5] = test_proxy(proxy_settings::http_pw, expect_udp_connection | expect_dht_msg);
+	pr[6] = test_proxy(proxy_settings::i2p_proxy, expect_udp_connection | expect_dht_msg);
 
 	// using anonymous mode
 
 	// anonymous mode doesn't require a proxy when one isn't configured. It could be
 	// used with a VPN for instance. This will all changed in 1.0, where anonymous
 	// mode is separated from force_proxy
-	test_proxy(proxy_settings::none, anonymous_mode | expect_peer_connection);
-	test_proxy(proxy_settings::socks4, anonymous_mode | expect_udp_reject);
-	test_proxy(proxy_settings::socks5, anonymous_mode);
-	test_proxy(proxy_settings::socks5_pw, anonymous_mode);
-	test_proxy(proxy_settings::http, anonymous_mode | expect_udp_reject);
-	test_proxy(proxy_settings::http_pw, anonymous_mode | expect_udp_reject);
-	test_proxy(proxy_settings::i2p_proxy, anonymous_mode);
+	pr[7] = test_proxy(proxy_settings::none, anonymous_mode | expect_peer_connection);
+	pr[8] = test_proxy(proxy_settings::socks4, anonymous_mode | expect_udp_reject);
+	pr[9] = test_proxy(proxy_settings::socks5, anonymous_mode);
+	pr[10] = test_proxy(proxy_settings::socks5_pw, anonymous_mode);
+	pr[11] = test_proxy(proxy_settings::http, anonymous_mode | expect_udp_reject);
+	pr[12] = test_proxy(proxy_settings::http_pw, anonymous_mode | expect_udp_reject);
+	pr[13] = test_proxy(proxy_settings::i2p_proxy, anonymous_mode);
 	return 0;
 }
 
