@@ -30,6 +30,8 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
+#ifndef TORRENT_DISABLE_EXTENSIONS
+
 #include "libtorrent/session.hpp"
 #include "libtorrent/hasher.hpp"
 #include "libtorrent/thread.hpp"
@@ -48,6 +50,12 @@ int test_main()
 {
 	using namespace libtorrent;
 
+	// these are declared before the session objects
+	// so that they are destructed last. This enables
+	// the sessions to destruct in parallel
+	session_proxy p1;
+	session_proxy p2;
+
 	session ses1(fingerprint("LT", 0, 1, 0, 0), std::make_pair(48130, 49000), "0.0.0.0", 0);
 	session ses2(fingerprint("LT", 0, 1, 0, 0), std::make_pair(49130, 50000), "0.0.0.0", 0);
 	ses1.add_extension(create_lt_trackers_plugin);
@@ -114,8 +122,16 @@ int test_main()
 		test_sleep(1000);
 	}
 
-	TEST_EQUAL(tor1.trackers().size(), 1);
+	TEST_CHECK(tor1.trackers().size() == 1);
+
+	// this allows shutting down the sessions in parallel
+	p1 = ses1.abort();
+	p2 = ses2.abort();
 
 	return 0;
 }
 
+#else
+int test_main() { return 0; }
+#endif // TORRENT_DISABLE_EXTENSIONS
+
